#include <hpkeyb49.h>
#include <hpstdio.h>
#include <hpstdlib.h>
#include <hpstring.h>
#include <hptime.h>
#include <saturn.h>
#include <hpgraphics.h>

#define NULL			0

#define FALSE			0
#define NEUTRAL			2

#define TICKS			10

#define X1				0
#define X2				(X1 + 26)
#define X3				(X1 + 51)
#define X4				(X1 + 77)

#define Y1				1
#define Y2				(Y1 + 26)
#define Y3				(Y1 + 51)
#define Y4				(Y1 + 77)

#define C1				0 
#define C2				25  
#define C3				50 

#define OFF1		    5
#define OFF2		    13
#define OFF3		    21

#define BLOCK_NUMBER(x,y) (x/3 + y/3) + ((x/3) << 1) 

#define	IS_CANDIDATE	0
#define	IS_GIVEN		1
#define	IS_FOUND		2

#define	MASK_GIVEN		0
#define	MASK_FOUND		1
#define	MAX_MASK		3*9

#define START			0
#define EXCLUDED		1

#define ALLCANDIDATES 	511

#define RC_SPACE		0
#define RN_SPACE		1
#define CN_SPACE		2
#define BN_SPACE		3
#define	MAX_SPACE		4

#define DISPLAY_X		85
#define DISPLAY_Y		16

#define	ROW				0
#define	COLUMN			1
#define	BLOCK			2
#define	BOX				2
#define	MAX_UNIT		3

#define BUFFER_LENGTH	100

#define ISNUMBER(c)  	((c >= '1' && c <= '9'))

#define BITCOUNT_2(v)		((v * 0x200040008001ULL & 0x111111111111111ULL) % 0xf)
#define BITCOUNT(v)			(bit_table[v])

#define	MAX_METHOD		12

#define	NAKED			0
#define	HIDDEN			1
#define	POINTING		2
#define	CLAIMING		3
#define	NAKED_TUPLE		4
#define	HIDDEN_TUPLE	5
#define	FISH_ALL		6
#define	X_WING			7
#define	SWORDFISH		8
#define	JELLYFISH		9
#define XY_CHAIN		10
#define XYT_CHAIN		11
#define XYZT_CHAIN		12
#define C_CHAIN			13
#define NRC_CHAIN		14

#define MAX_PATTERN		15

#define	IS_PAIR			2
#define	IS_TRIPLET		3
#define	IS_QUAD			4

#define ERROR_MEMORY	0
#define ERROR_FILE		1
#define ERROR_GRID		2
#define GAME_SAVED		3
#define MESSAGE_SLOT	4
#define NO_HINT			5
#define FINISHED		6
#define FAILED			7
#define PUSHED			8
#define SNAPSHOT		9

#define L_CURSOR		22

typedef struct {
	int flag;
	int tuple;
	char *name;
} SHOW_TUPLE;

typedef struct {
	int type;
	int data;
} CELL;

typedef struct { 
	int x,y;
} COORDINATE;

typedef struct {
	int x,y;
} CURSOR;

typedef struct {
	int is_valid;
	int number;
	int row;
	int column;
	int alignement;
	int rank2;
	int block;	
} LOCKED;

typedef struct {
	int is_tuple;
	int size;
	CELL *p_cell;
	int peers[3];
} NAKED_SUBSET;

typedef struct {
	int type;
	int candidate;
	int zone;
	int tuple;
	int unit[3];
	char *zone_name[3];
	char *tuple_name[3];
	char *fish_name[3];
} HINT_INFO;

typedef struct {
	int type;
	int space_type;
	int candidate;
	int dim[4][2];
} FISH_ALL_INFO;

int naked_single(), hidden_single();
int pointing(), claiming();
int naked_subset(int unit_type);
int hidden_subset(int space_type);
int fish(int space_type);
int backtraking();

void get_grid_from_keyboard();
void load_grid();
void filter_candidate(int candidate), exclude_candidate(int candidate);
void input_found(int found);
void restart_game();
void save_game(), restore_game();
void show_subset();
int check_grid();
void hint();
void select_hint(int method);
int resolve_grid();
void puzzle_to_string(), string_to_puzzle(); 
void display_candidates();
void screen_snapshot();

void draw_grid(), fill_display_grid();
void clear_sdk_cellinfo();
int validate_grid();
void compute_candidates();
void clear_display_area();
int is_grid_finished();
void display_message(int message_index);
int compare_nakedsubset(const void *arg1, const void *arg2);
int is_subset();
char * fgets_local(char *buf, int n, FILE *f);
void restore_screen();
int resolve_singles();
void resolve_subset();
void highlight_locked();
int grid_consistent();
void fill_space_table(int space_type);
void highlight_fish_all();


COORDINATE coordinates[9][9] = {
// ROW 1
	{
		{C1+OFF1, C1+OFF1}, {C1+OFF2, C1+OFF1}, {C1+OFF3, C1+OFF1},
		{C2+OFF1, C1+OFF1}, {C2+OFF2, C1+OFF1}, {C2+OFF3, C1+OFF1},
		{C3+OFF1, C1+OFF1}, {C3+OFF2, C1+OFF1}, {C3+OFF3, C1+OFF1},
	},
// ROW 2
	{
		{C1+OFF1, C1+OFF2}, {C1+OFF2, C1+OFF2}, {C1+OFF3, C1+OFF2},
		{C2+OFF1, C1+OFF2}, {C2+OFF2, C1+OFF2}, {C2+OFF3, C1+OFF2},
		{C3+OFF1, C1+OFF2}, {C3+OFF2, C1+OFF2}, {C3+OFF3, C1+OFF2},
	},
// ROW 3
	{
		{C1+OFF1, C1+OFF3}, {C1+OFF2, C1+OFF3}, {C1+OFF3, C1+OFF3},
		{C2+OFF1, C1+OFF3}, {C2+OFF2, C1+OFF3}, {C2+OFF3, C1+OFF3},
		{C3+OFF1, C1+OFF3}, {C3+OFF2, C1+OFF3}, {C3+OFF3, C1+OFF3},
	},
// ROW 4
	{
		{C1+OFF1, C2+OFF1}, {C1+OFF2, C2+OFF1}, {C1+OFF3, C2+OFF1},
		{C2+OFF1, C2+OFF1}, {C2+OFF2, C2+OFF1}, {C2+OFF3, C2+OFF1},
		{C3+OFF1, C2+OFF1}, {C3+OFF2, C2+OFF1}, {C3+OFF3, C2+OFF1},
	},
// ROW 5
	{
		{C1+OFF1, C2+OFF2}, {C1+OFF2, C2+OFF2}, {C1+OFF3, C2+OFF2},
		{C2+OFF1, C2+OFF2}, {C2+OFF2, C2+OFF2}, {C2+OFF3, C2+OFF2},
		{C3+OFF1, C2+OFF2}, {C3+OFF2, C2+OFF2}, {C3+OFF3, C2+OFF2},
	},
// ROW 6
	{
		{C1+OFF1, C2+OFF3}, {C1+OFF2, C2+OFF3}, {C1+OFF3, C2+OFF3},
		{C2+OFF1, C2+OFF3}, {C2+OFF2, C2+OFF3}, {C2+OFF3, C2+OFF3},
		{C3+OFF1, C2+OFF3}, {C3+OFF2, C2+OFF3}, {C3+OFF3, C2+OFF3},
	},
// ROW 7
	{
		{C1+OFF1, C3+OFF1}, {C1+OFF2, C3+OFF1}, {C1+OFF3, C3+OFF1},
		{C2+OFF1, C3+OFF1}, {C2+OFF2, C3+OFF1}, {C2+OFF3, C3+OFF1},
		{C3+OFF1, C3+OFF1}, {C3+OFF2, C3+OFF1}, {C3+OFF3, C3+OFF1},
	},
// ROW 8
	{
		{C1+OFF1, C3+OFF2}, {C1+OFF2, C3+OFF2}, {C1+OFF3, C3+OFF2},
		{C2+OFF1, C3+OFF2}, {C2+OFF2, C3+OFF2}, {C2+OFF3, C3+OFF2},
		{C3+OFF1, C3+OFF2}, {C3+OFF2, C3+OFF2}, {C3+OFF3, C3+OFF2},
	},
// ROW 9
	{
		{C1+OFF1, C3+OFF3}, {C1+OFF2, C3+OFF3}, {C1+OFF3, C3+OFF3},
		{C2+OFF1, C3+OFF3}, {C2+OFF2, C3+OFF3}, {C2+OFF3, C3+OFF3},
		{C3+OFF1, C3+OFF3}, {C3+OFF2, C3+OFF3}, {C3+OFF3, C3+OFF3},
	},
}; 

CELL sdk_cellinfo[9][9]= {
	{{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},},
	{{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_GIVEN,6},{IS_GIVEN,2},{IS_GIVEN,3},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},},
	{{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_GIVEN,5},{IS_CANDIDATE,0},{IS_GIVEN,4},{IS_CANDIDATE,0},{IS_GIVEN,8},{IS_CANDIDATE,0},{IS_CANDIDATE,0},},
	{{IS_CANDIDATE,0},{IS_GIVEN,7},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_GIVEN,3},{IS_CANDIDATE,0},},
	{{IS_CANDIDATE,0},{IS_GIVEN,8},{IS_GIVEN,9},{IS_CANDIDATE,0},{IS_GIVEN,5},{IS_CANDIDATE,0},{IS_GIVEN,7},{IS_GIVEN,4},{IS_CANDIDATE,0},},
	{{IS_CANDIDATE,0},{IS_GIVEN,3},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_GIVEN,2},{IS_CANDIDATE,0},},
	{{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_GIVEN,6},{IS_CANDIDATE,0},{IS_GIVEN,8},{IS_CANDIDATE,0},{IS_GIVEN,9},{IS_CANDIDATE,0},{IS_CANDIDATE,0},},
	{{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_GIVEN,4},{IS_GIVEN,7},{IS_GIVEN,2},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},},
	{{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},{IS_CANDIDATE,0},},	
};


int block_start[9][2] = {{0,0}, {0,3}, {0,6}, {3,0}, {3,3}, {3,6}, {6,0}, {6,3}, {6,6}};

char sdk_digit[] = {'.','1','2','3','4','5','6','7','8','9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 27};

/*	type in : {NAKED, HIDDEN, POINTING, CLAIMING, NAKED_TUPLE, HIDDEN_TUPLE, X_WING, SWORDFISH JELLYFISH}		
	zone in : {ROW, COLUMN, BLOCK}
	tuple in: {IS_PAIR, IS_TRIPLET, IS_QUAD}
	unit[zone] is the index of the row column or block number
*/
HINT_INFO hint_info = {0,0,0,0,{0,0,0},{"Row","Column","Block"},{"Pair","Triplet","Quad"},{"X-Wing","Swordfish","Jellyfish"}};

char *space_title[] = {
	"Row-Col:",
	"Row-Num:",
	"Col-Num:",
	"Box-Num:",
};

char *message[] = {
	"Error: insufficient memory!",
	"Error: file not found!",
	"Error: the grid is invalid!",
	"The game is saved",
	"",
	"No hint",
	"The grid is solved",
	"Resolve failed",
	"The grid is pushed",
	"Snaphot done",
};

char *select_hint_message[] = {
	"no Naked single",
	"no Hidden single",
	"no Pointing",
	"no Claiming",
	"no Naked subset",
	"no Hidden subset",
	"no fish",
	"no X-Wing",
	"no Swordfish",
	"no jellyfish",
};

char buffer[BUFFER_LENGTH];
int mask_unit[2][MAX_MASK];
int unit_type = -1;
int candidates[2][9][9];
int grid_number = 1;
int next_load_file;
int filtered_candidate = 0;
int letter;
char *grid_filename;
int stack_level = 2;
SHOW_TUPLE show_tuple;
CURSOR cursor = {4,4};
LOCKED locked_candidate[9];
int flag_locked;
int flag_fish;
int flag_hint;
int pattern[MAX_PATTERN];
FISH_ALL_INFO fish_all_info;
CELL space_table[9][9];
int manu_space_type = RC_SPACE;
int bit_table[512];

COORDINATE row_coordinates[9][9] = {
	{{0,0},{0,1},{0,2},{0,3},{0,4},{0,5},{0,6},{0,7},{0,8},},
	{{1,0},{1,1},{1,2},{1,3},{1,4},{1,5},{1,6},{1,7},{1,8},},
	{{2,0},{2,1},{2,2},{2,3},{2,4},{2,5},{2,6},{2,7},{2,8},},
	{{3,0},{3,1},{3,2},{3,3},{3,4},{3,5},{3,6},{3,7},{3,8},},
	{{4,0},{4,1},{4,2},{4,3},{4,4},{4,5},{4,6},{4,7},{4,8},},
	{{5,0},{5,1},{5,2},{5,3},{5,4},{5,5},{5,6},{5,7},{5,8},},
	{{6,0},{6,1},{6,2},{6,3},{6,4},{6,5},{6,6},{6,7},{6,8},},
	{{7,0},{7,1},{7,2},{7,3},{7,4},{7,5},{7,6},{7,7},{7,8},},
	{{8,0},{8,1},{8,2},{8,3},{8,4},{8,5},{8,6},{8,7},{8,8},},
}; 

COORDINATE col_coordinates[9][9] = {
	{{0,0},{1,0},{2,0},{3,0},{4,0},{5,0},{6,0},{7,0},{8,0},},
	{{0,1},{1,1},{2,1},{3,1},{4,1},{5,1},{6,1},{7,1},{8,1},},
	{{0,2},{1,2},{2,2},{3,2},{4,2},{5,2},{6,2},{7,2},{8,2},},
	{{0,3},{1,3},{2,3},{3,3},{4,3},{5,3},{6,3},{7,3},{8,3},},
	{{0,4},{1,4},{2,4},{3,4},{4,4},{5,4},{6,4},{7,4},{8,4},},
	{{0,5},{1,5},{2,5},{3,5},{4,5},{5,5},{6,5},{7,5},{8,5},},
	{{0,6},{1,6},{2,6},{3,6},{4,6},{5,6},{6,6},{7,6},{8,6},},
	{{0,7},{1,7},{2,7},{3,7},{4,7},{5,7},{6,7},{7,7},{8,7},},
	{{0,8},{1,8},{2,8},{3,8},{4,8},{5,8},{6,8},{7,8},{8,8},},
};

COORDINATE box_coordinates[9][9] = {
	{{0,0},{0,1},{0,2},{1,0},{1,1},{1,2},{2,0},{2,1},{2,2},},
	{{0,3},{0,4},{0,5},{1,3},{1,4},{1,5},{2,3},{2,4},{2,5},},
	{{0,6},{0,7},{0,8},{1,6},{1,7},{1,8},{2,6},{2,7},{2,8},},
	{{3,0},{3,1},{3,2},{4,0},{4,1},{4,2},{5,0},{5,1},{5,2},},
	{{3,3},{3,4},{3,5},{4,3},{4,4},{4,5},{5,3},{5,4},{5,5},},
	{{3,6},{3,7},{3,8},{4,6},{4,7},{4,8},{5,6},{5,7},{5,8},},
	{{6,0},{6,1},{6,2},{7,0},{7,1},{7,2},{8,0},{8,1},{8,2},},
	{{6,3},{6,4},{6,5},{7,3},{7,4},{7,5},{8,3},{8,4},{8,5},},
	{{6,6},{6,7},{6,8},{7,6},{7,7},{7,8},{8,6},{8,7},{8,8},},
};


#define XY_CHAIN_FUNC		1
#define XYT_CHAIN_FUNC		2
#define XYZT_CHAIN_FUNC		3
#define MAX_BIVALUE_CELLS	81
#define XY_CHAIN_MAX_LENGTH	13
#define C_CHAIN_MAX_LENGTH	9

#define NRC_CHAIN_MAX_LENGTH	9
#define NRCT_CHAIN_MAX_LENGTH	11
#define NRC_CHAIN_FUNC			1
#define NRCT_CHAIN_FUNC			2
#define T_SIZE					9
#define COUNTER_STEP			25000


typedef struct {
	int cell_index;
	int data;
} BIVALUE_CELL;

typedef struct {
	int link;
	int next;
} XY_LINK;

BIVALUE_CELL xy_chain_array[NRCT_CHAIN_MAX_LENGTH*2];

int xy_chain_length,conjugate_chain_length;
int bivalue_number;
int flag_xychain,flag_cchain;

void fill_share_unit_array(int space_type);
void fill_buddies_array(int space_type);

int xy_chain(),xyt_chain(),xyzt_chain(),xy_chain_caller(int chain_function_index);
void highlight_xychain();

int compare_bivalue_cell(const void *arg1, const void *arg2);

typedef struct {
	char share_unit_array[81][81];
} SHARE_UNIT_ARRAY;

SHARE_UNIT_ARRAY * p_share_unit_array;
SHARE_UNIT_ARRAY RC_share_unit_array,RNCN_share_unit_array;

typedef struct {
	char buddies_array[81][20];
} BUDDIES_ARRAY;

BUDDIES_ARRAY * p_buddies_array;
BUDDIES_ARRAY RC_buddies_array,RNCN_buddies_array;

void draw_grid_img();

typedef struct {
	int c1_index;
	int c2_index;
} CONJUGATE_CELL;

typedef struct {
	int link;
	int next;
	int link_cell;
} CONJUGATE_LINK;

int conjugate_chain();
void highlight_cchain();
int c_chain_caller(), conjugate_chain_array[C_CHAIN_MAX_LENGTH+1];

int resolve_with_cchain(),resolve_with_xyztchain();

int verbose;
char chain_string[300];


typedef struct {
	int c1_index;
	int c1_number;
	int c2_index;
	int c2_number;
	int type;
} NRC_CELL;

typedef struct {
	int link;
	int next;
	int out_cell;
	int in_candidate;
} NRC_LINK;

typedef struct {
	int cell_index;
	int nrc_index;
} NRC_PATH;

int nrc_chain(),nrc_chain_caller(int chain_function_index);
int nrc_number,nrc_chain_length;

NRC_PATH nrc_path[NRCT_CHAIN_MAX_LENGTH*2];
void get_nrc_path(NRC_LINK * p_stack,NRC_CELL * p_nrc_array);
void do_verbose_nrc(NRC_CELL * p_nrc_array,int c,int row,int col);
void do_verbose_xy(BIVALUE_CELL *p_bivalue_array,XY_LINK *p_stack,int target_number,int row,int col);

int chain_number;
int flag_counter;

typedef struct {
	int c1_index;
	int c1_number;
	int c2_index;
	int c2_number;
	int type;
	int space;
	int data;
	int row;
	int number;
} NRCT_CELL;

typedef struct {
	int number;
	int cell;
} T_NUMBER;

void get_nrct_path(NRC_LINK *p_stack, NRCT_CELL *p_nrc_array);
void do_verbose_nrct(NRCT_CELL *p_nrc_array,int c,int row,int col);

int nrct_chain();

char bit_rank[256+1];


int main() {	 
	int i,j,key; 
	int row,col,val;
	int old_filtered = 0;
	FILE *handle;
	CURSOR c = cursor;
	unsigned char ticks = sys_setRTCTickPeriod(1);
	row = col = val = 0;
	hpg_set_mode_gray16(0); hpg_clear(); 	
	hpg_set_font(hpg_stdscreen, hpg_get_minifont());
	validate_grid(); draw_grid();  
	manu_space_type = RC_SPACE; fill_space_table(manu_space_type); fill_display_grid();
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR);
	fill_share_unit_array(RC_SPACE); fill_share_unit_array(RN_SPACE);
	fill_buddies_array(RC_SPACE); fill_buddies_array(RN_SPACE);
	for(i = 0; i < 512; i++) {bit_table[i] = BITCOUNT_2(i);}
	bit_rank[2] = 1; bit_rank[4] = 2; bit_rank[8] = 3; bit_rank[16] = 4; 
	bit_rank[32] = 5; bit_rank[64] = 6; bit_rank[128] = 7; bit_rank[256] = 8; 
	
	if(!(grid_filename = sat_pick_str_alloc(1))) grid_filename = "grid.sdk";
	while(!keyb_isON()) {	 
//									draw the cursor: get the letter under the cursor
		if(space_table[cursor.x][cursor.y].type == IS_CANDIDATE) {
		letter = 0;
//									if flag_xychain is on and the cursor is on a link
			if(flag_xychain) {
				for(i = xy_chain_length-1; i > -1; i--) {
					row = xy_chain_array[i].cell_index/9; col = xy_chain_array[i].cell_index%9;
					if(cursor.x == row && cursor.y == col) {letter = i+1; break;}
				}
			}
			if(letter == 0) {
//									if the filter is on and the cursor is on a filtered cell
				if(filtered_candidate && space_table[cursor.x][cursor.y].data & (1 << (filtered_candidate - 1))) letter = filtered_candidate;
				else {
					hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
					hpg_draw_letter(sdk_digit[0], coordinates[cursor.x][cursor.y].x, coordinates[cursor.x][cursor.y].y);
					hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); letter = L_CURSOR;
				}
			}
		}
		else letter = space_table[cursor.x][cursor.y].data;
		hpg_draw_letter(sdk_digit[letter], coordinates[cursor.x][cursor.y].x, coordinates[cursor.x][cursor.y].y);
		sys_waitRTCTicks(TICKS);
//									play the game
		if(( key = keyb_getkeyM(0) ) != 0) {		
			switch(key & 0xff) {
			
				case KB_N	:	if(key & 0x20000000) {flag_hint = TRUE; nrc_chain_caller(NRC_CHAIN_FUNC); flag_hint = 0;}
								else nrc_chain_caller(NRC_CHAIN_FUNC); break;

				case KB_L	:	if(key & 0x20000000) {flag_hint = TRUE; nrc_chain_caller(NRCT_CHAIN_FUNC); flag_hint = 0;}
								else nrc_chain_caller(NRCT_CHAIN_FUNC); break;
			
				case KB_O	:	if(key & 0x40000000) {flag_counter = ~flag_counter; 
									if(flag_counter) {message[MESSAGE_SLOT] = "Counter on";} 
									else message[MESSAGE_SLOT] = "Counter off"; display_message(MESSAGE_SLOT); restore_screen();
								}
								else if(key & 0x20000000) {
									if(flag_xychain) {
										hpg_clear(); hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); char *p,*q; p = q = chain_string; i = j = 0; int l;
										p = strchr(p,'-'); *p = '\0'; hpg_draw_text(q, 0, 6 * i++); *p = '-'; q = ++p;
										while((p = strchr(p,'-'))) {
											*p = '\0'; if((l=strlen(q)) > 16 && j) {++i; j = 0;} hpg_draw_text(q, j, 6 * i); 
											if(j == 0 && l <=16) {j = 63;} else {j = 0; ++i;} *p = '-'; q = ++p;
										} 
										hpg_draw_text(--q, 0, 6 * i); while (!(keyb_isKeyPressed(4,1))); restore_screen();
									}
								}
								else {
									verbose = ~verbose; if(verbose) message[MESSAGE_SLOT] = "Verbose on"; else message[MESSAGE_SLOT] = "Verbose off";
									display_message(MESSAGE_SLOT); restore_screen();
								} break;
			
				case KB_K	:	if(key & 0x20000000) {flag_hint = TRUE; c_chain_caller(); flag_hint = 0;}
								else c_chain_caller();
								break;
			
				case KB_V	:	if(filtered_candidate) filter_candidate(filtered_candidate);
								draw_grid_img(); fill_space_table(manu_space_type); restore_screen();
								break;
						
				case KB_X	:	if(key & 0x20000000) {flag_hint = TRUE; xy_chain_caller(XY_CHAIN_FUNC); flag_hint = 0;}
								else xy_chain_caller(XY_CHAIN_FUNC);
								break;

				case KB_T	:	if(key & 0x20000000) {flag_hint = TRUE; xy_chain_caller(XYT_CHAIN_FUNC); flag_hint = 0;}
								else xy_chain_caller(XYT_CHAIN_FUNC);
								break;

				case KB_Y	:	if(key & 0x20000000) {flag_hint = TRUE; xy_chain_caller(XYZT_CHAIN_FUNC); flag_hint = 0;}
								else xy_chain_caller(XYZT_CHAIN_FUNC);
								break;

				case KB_Z 	:	screen_snapshot(); break;
				
//									cycle through the spaces	
				case KB_J 	:	flag_xychain = 0; if(filtered_candidate) {filter_candidate(filtered_candidate);}
				
								if(key & 0x20000000) manu_space_type = RC_SPACE; else {++manu_space_type; manu_space_type %= MAX_SPACE;} 
								
								if(unit_type != -1) {
								if(manu_space_type == BN_SPACE) unit_type = ROW;
								else if(manu_space_type != RC_SPACE && unit_type == BLOCK) unit_type = ROW;
								}
								fill_space_table(manu_space_type); draw_grid(); fill_display_grid(); display_candidates(); 
								
								break;
								
//									turn on/off the display of candidates
				case KB_I 	:	if(key & 0x20000000) unit_type = -1; 
								else {
									unit_type++; 	
									if(manu_space_type == RC_SPACE) unit_type %= MAX_UNIT;
									else if(manu_space_type == BN_SPACE) unit_type %= (MAX_UNIT - 2);
									else unit_type %= (MAX_UNIT - 1);
								} 
								display_candidates(); break;
			
				case KB_W 	:	show_subset(); break;
			
				case KB_P 	:	if(key & 0x20000000) string_to_puzzle(); else puzzle_to_string(); break;
			
				case KB_H 	:	flag_hint = TRUE; hint(); flag_hint = 0; 
								if(is_grid_finished()) {display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4; 
									manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen();}
								break;
					
				case KB_A 	:	if(key & 0x20000000) {flag_hint = TRUE; 
									if(naked_single()) select_hint(NAKED); else select_hint(-(NAKED+1)); flag_hint = 0;}
				
								else if(key & 0x40000000){flag_hint = TRUE; 
									if(hidden_single()) select_hint(HIDDEN); else select_hint(-(HIDDEN+1)); flag_hint = 0;}
									
								//else if(!resolve_singles()) {check_grid();}
								else {resolve_singles(); check_grid();}
								
								fill_space_table(manu_space_type); restore_screen(); break;		
			
				case KB_B 	: 	if(key & 0x20000000) {flag_hint = TRUE; 
									if(pointing()) select_hint(POINTING); else select_hint(-(POINTING+1)); flag_hint = 0;}
				
								else if(key & 0x40000000){flag_hint = TRUE; 
									if(claiming()) select_hint(CLAIMING); else select_hint(-(CLAIMING+1)); flag_hint = 0;}
									
								else {if(pointing() | claiming()) {while(pointing() | claiming()); 
								if(!resolve_singles()) {check_grid();}}}

								fill_space_table(manu_space_type); restore_screen(); break;
								
				case KB_C 	: 	if(key & 0x20000000) {
									flag_hint = TRUE; 
									if(naked_subset(ROW)) select_hint(NAKED_TUPLE);
									else if(naked_subset(COLUMN)) select_hint(NAKED_TUPLE);
									else if(naked_subset(BLOCK)) select_hint(NAKED_TUPLE);
									else select_hint(-(NAKED_TUPLE+1)); 
									flag_hint = 0;
								}
				
								else if(key & 0x40000000){
									flag_hint = TRUE; 
									if(hidden_subset(ROW)) select_hint(HIDDEN_TUPLE);
									else if(hidden_subset(COLUMN)) select_hint(HIDDEN_TUPLE);
									else if(hidden_subset(BLOCK)) select_hint(HIDDEN_TUPLE);
									else select_hint(-(HIDDEN_TUPLE+1)); 
									flag_hint = 0;
								}
								
								else {if(is_subset()) {resolve_subset(); if(!resolve_singles()) {check_grid();}}}

								fill_space_table(manu_space_type); restore_screen(); break;
								
				case KB_D 	: 	if(key & 0x20000000) {flag_hint = TRUE; 
									if(fish(ROW)) 
										select_hint(FISH_ALL); else select_hint(-(FISH_ALL+1));  flag_hint = 0;}
								
								else if(key & 0x40000000) {flag_hint = TRUE; 
									if(fish(COLUMN)) 
										select_hint(FISH_ALL); else select_hint(-(FISH_ALL+1));  flag_hint = 0;}
							
								else {if(fish(COLUMN) | fish(ROW)) {while(fish(COLUMN) | fish(ROW)); 
								if(!resolve_singles()) {check_grid();}}}
								
								fill_space_table(manu_space_type); restore_screen(); break;
								
				case KB_E 	:	flag_xychain = 0; sys_slowOff(); hpg_clear(); hpg_draw_text("Computing...", 41, 36); 
								if(backtraking()) {manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); display_message(FINISHED); 
													unit_type = -1; cursor.x = 4; cursor.y = 4; restore_screen();}
								else {fill_space_table(manu_space_type); restore_screen(); check_grid(); display_message(FAILED); restore_screen();}; 
								sys_slowOn(); break;			
								
				case KB_F 	:	sys_slowOff(); flag_xychain = 0;
								if(resolve_grid()) {display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4;
								manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen();}
								else { fill_space_table(manu_space_type); check_grid(); display_message(FAILED); restore_screen();}  
								sys_slowOn(); break;			
			
				case KB_G 	:	flag_xychain = 0; clear_sdk_cellinfo(); get_grid_from_keyboard(); goto validate;
				case KB_U	:	flag_xychain = 0; if(key & 0x20000000) next_load_file = 1; load_grid(); goto validate;
				case KB_M 	:	flag_xychain = 0; for(i = 0; i  < 9; i++) for(j = 0; j < 9; j++) if(sdk_cellinfo[i][j].type == IS_FOUND) sdk_cellinfo[i][j].type = IS_GIVEN;
								get_grid_from_keyboard();
								
				validate	:	while(!validate_grid()) get_grid_from_keyboard();
								cursor.x = 4; cursor.y = 4; unit_type = -1; filtered_candidate = 0;  
								manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); break; 				
				
//								save the game
				case KB_S 	:	if((handle = fopen("saved.sdk", "w")) == NULL) display_message(ERROR_FILE);
								else {
									fwrite(sdk_cellinfo, 1, sizeof(sdk_cellinfo), handle);
									fwrite(mask_unit, 1, sizeof(mask_unit), handle);
									fwrite(candidates, 1, sizeof(candidates), handle); 
									fclose(handle); display_message(GAME_SAVED);
								}
								restore_screen(); break;

//								restore the game
				case KB_R 	:	if((handle = fopen("saved.sdk", "r")) == NULL) display_message(ERROR_FILE); 
								else {
									fread(sdk_cellinfo, 1, sizeof(sdk_cellinfo), handle); 				
									fread(mask_unit, 1, sizeof(mask_unit), handle); 
									fread(candidates, 1, sizeof(candidates), handle); fclose(handle); 									
									cursor.x = 4; cursor.y = 4; unit_type = -1;
									if(filtered_candidate) filter_candidate(filtered_candidate);
									manu_space_type = RC_SPACE; fill_space_table(manu_space_type);
								}
								flag_xychain = 0; restore_screen(); break;
							
				case KB_UP	:	c = cursor; if(key & 0x20000000) cursor.x -= 3; else cursor.x--; if(cursor.x < 0) cursor.x += 9; goto arrow;	
				case KB_DN	:	c = cursor; if(key & 0x20000000) cursor.x += 3; else  cursor.x++; cursor.x = cursor.x % 9; goto arrow;
				case KB_LF	:	c = cursor; if(key & 0x20000000) cursor.y -= 3; else  cursor.y--; if(cursor.y < 0) cursor.y += 9; goto arrow;
				case KB_RT	:	c = cursor; if(key & 0x20000000) cursor.y += 3; else cursor.y++; cursor.y = cursor.y % 9; goto arrow;
				
//									LSHIFT 0: restaure all excluded candidates in a cell 
//										   0: restaure all excluded candidates in all cells
				case KB_0 	:	old_filtered = 0;
//									if filter is on turn it off
								if(filtered_candidate) {old_filtered = filtered_candidate; filter_candidate(filtered_candidate);}
								
								if(key & 0x20000000) {
									if(manu_space_type == RC_SPACE) candidates[EXCLUDED][cursor.x][cursor.y] = 0;}  
								
								else {for(i = 0; i < 9; i++) for(j = 0; j < 9; candidates[EXCLUDED][i][j++] = 0);}
								
								compute_candidates(); fill_space_table(manu_space_type); display_candidates(); 
								
//									if there was a filter put it back
								if(old_filtered) filter_candidate(old_filtered);						
								break;
				
//									lshift toggle exlude candidate rshift toggle filter candidate else input a found
				case KB_1 	:	if(key & 0x20000000) exclude_candidate(1); else if(key & 0x40000000) filter_candidate(1); else input_found(1); break;
				case KB_2 	:	if(key & 0x20000000) exclude_candidate(2); else if(key & 0x40000000) filter_candidate(2); else input_found(2); break;
				case KB_3 	:	if(key & 0x20000000) exclude_candidate(3); else if(key & 0x40000000) filter_candidate(3); else input_found(3); break;
				case KB_4 	:	if(key & 0x20000000) exclude_candidate(4); else if(key & 0x40000000) filter_candidate(4); else input_found(4); break;
				case KB_5 	:	if(key & 0x20000000) exclude_candidate(5); else if(key & 0x40000000) filter_candidate(5); else input_found(5); break;
				case KB_6 	:	if(key & 0x20000000) exclude_candidate(6); else if(key & 0x40000000) filter_candidate(6); else input_found(6); break;
				case KB_7 	:	if(key & 0x20000000) exclude_candidate(7); else if(key & 0x40000000) filter_candidate(7); else input_found(7); break;
				case KB_8 	:	if(key & 0x20000000) exclude_candidate(8); else if(key & 0x40000000) filter_candidate(8); else input_found(8); break;
				case KB_9 	:	if(key & 0x20000000) exclude_candidate(9); else if(key & 0x40000000) filter_candidate(9); else input_found(9); break;
				
//									Lshift BKS: restart game 
//										   BKS: erase a found
				case KB_BKS:	flag_xychain = 0; 
								if(key & 0x20000000) restart_game();
								
								else {
									if(space_table[cursor.x][cursor.y].type == IS_FOUND) {
										old_filtered = 0;
								
//										if filter is on turn it off
										if(filtered_candidate) {old_filtered = filtered_candidate; filter_candidate(filtered_candidate);}
									
//										update screen
										hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
										hpg_draw_letter(sdk_digit[letter], coordinates[cursor.x][cursor.y].x, coordinates[cursor.x][cursor.y].y);
										hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
									
//										rcspace->rcspace
										if(manu_space_type == RC_SPACE) {row = cursor.x; col = cursor.y; val = space_table[cursor.x][cursor.y].data - 1;}
//										rnspace->rcspace
										else if(manu_space_type == RN_SPACE) {row = cursor.x; col = space_table[cursor.x][cursor.y].data - 1; val = cursor.y;}
//										cnspace->rcspace
										else if(manu_space_type == CN_SPACE) {col = cursor.x; row = space_table[cursor.x][cursor.y].data - 1;  val = cursor.y;}
//										bnspace->rcspace
										else if(manu_space_type == BN_SPACE) {
										row = block_start[cursor.x][0] + ((space_table[cursor.x][cursor.y].data - 1) / 3); 
										col = block_start[cursor.x][1] + ((space_table[cursor.x][cursor.y].data - 1) % 3);  
										val = cursor.y;
										}
										
//										update info
										mask_unit[MASK_FOUND][row] &= ~(1 << val); 
										mask_unit[MASK_FOUND][col + 9] &= ~(1 << val);
										mask_unit[MASK_FOUND][BLOCK_NUMBER(row,col) + 18] &= ~(1 << val); 	
									
										candidates[EXCLUDED][row][col] = 0;
									
										sdk_cellinfo[row][col].type = IS_CANDIDATE; compute_candidates(); 
									
										fill_space_table(manu_space_type); display_candidates();
								
//										if there was a filter put it back
										if(old_filtered) filter_candidate(old_filtered);
										
										check_grid(); restore_screen();
									}
								}
								break;	
								
				arrow		: 	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
								if(space_table[c.x][c.y].type == IS_CANDIDATE) hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
								if(letter == L_CURSOR) {
									hpg_draw_letter(sdk_digit[letter], coordinates[c.x][c.y].x, coordinates[c.x][c.y].y);
									letter = 0; hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
								}
								hpg_draw_letter(sdk_digit[letter], coordinates[c.x][c.y].x, coordinates[c.x][c.y].y);
								hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR);
								display_candidates(); break;
			}
		}	
	}
	sys_setRTCTickPeriod(ticks); return 1;
}
//
//										METHODS
//
//--------------------------------------------------------
//									chains functions
//--------------------------------------------------------		
//
//
void get_nrc_path(NRC_LINK *p_stack, NRC_CELL *p_nrc_array) {
	int i,path_index;

	nrc_path[0].nrc_index = p_stack[0].link;
	
	path_index = 1;
	if(p_nrc_array[p_stack[0].link].type == C_CHAIN) {
		if(p_nrc_array[p_stack[0].link].c1_index == nrc_path[0].cell_index) {
			nrc_path[path_index].nrc_index = p_stack[0].link;
			nrc_path[path_index++].cell_index = p_nrc_array[p_stack[0].link].c2_index;
		}
		else {
			nrc_path[path_index].nrc_index = p_stack[0].link;
			nrc_path[path_index++].cell_index = p_nrc_array[p_stack[0].link].c1_index;
		}
	}
	
	for(i = 1; i < nrc_chain_length; i++) {	
		if(p_nrc_array[p_stack[i].link].type == XY_CHAIN) {
			nrc_path[path_index].nrc_index = p_stack[i].link;
			nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
		}
		else {
			if(p_nrc_array[p_stack[i-1].link].type == XY_CHAIN) {
				if(RC_share_unit_array.share_unit_array[p_nrc_array[p_stack[i-1].link].c1_index][p_nrc_array[p_stack[i].link].c1_index]) {
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
				}
				else {
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
				}
			}
			else {
				if(p_nrc_array[p_stack[i-1].link].c1_number != p_nrc_array[p_stack[i].link].c1_number) {
				
					if(nrc_path[path_index-1].cell_index == p_nrc_array[p_stack[i].link].c1_index) {
						nrc_path[path_index].nrc_index = p_stack[i].link;
						nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
					}
					else {
						nrc_path[path_index].nrc_index = p_stack[i].link;
						nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
					}				
				}
				else {
					if(RC_share_unit_array.share_unit_array[nrc_path[path_index-1].cell_index][p_nrc_array[p_stack[i].link].c1_index]) {
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
					}
					else {
						nrc_path[path_index].nrc_index = p_stack[i].link;
						nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
						nrc_path[path_index].nrc_index = p_stack[i].link;
						nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
					}
				}
			}
		}
	}
	
	xy_chain_length = path_index;
	for(i = 0; i < path_index; i++) {xy_chain_array[i].cell_index = nrc_path[i].cell_index;}
	
}

void do_verbose_nrc(NRC_CELL *p_nrc_array,int c,int row,int col) {
	int idx,k;
	chain_string[0] = 0; sprintf(chain_string,"nrc[%d]",nrc_chain_length<<1);
	for(k = 0; k < xy_chain_length; k++) {
		sprintf(buffer,"(r%d,c%d)",	(nrc_path[k].cell_index/9)+1, (nrc_path[k].cell_index%9)+1);
		idx = strlen(buffer); buffer[idx++] = '{'; 
		if(p_nrc_array[nrc_path[k].nrc_index].type == XY_CHAIN) {
			buffer[idx++] = p_nrc_array[nrc_path[k].nrc_index].c1_number + 49; buffer[idx++] = ',';
			buffer[idx++] = p_nrc_array[nrc_path[k].nrc_index].c2_number + 49;
		}
		else {
			buffer[idx++] = p_nrc_array[nrc_path[k].nrc_index].c1_number + 49;
			if(k < xy_chain_length - 1 	&& p_nrc_array[nrc_path[k].nrc_index].c1_number != p_nrc_array[nrc_path[k+1].nrc_index].c1_number
										&& p_nrc_array[nrc_path[k+1].nrc_index].type != XY_CHAIN) {
				buffer[idx++] = '/'; buffer[idx++] = p_nrc_array[nrc_path[k+1].nrc_index].c1_number + 49;							
			}
		}
		buffer[idx] = '}'; buffer[++idx] = '-'; buffer[++idx] = '\0'; strcat(chain_string,buffer);
	}
	sprintf(buffer,"->(r%d,c%d)<>",row+1,col+1); idx = strlen(buffer); buffer[idx++] = '{'; 
	buffer[idx++] = bit_rank[c]+49;	
	buffer[idx++] = '}'; buffer[idx] = '\0'; strcat(chain_string,buffer);
}

void do_verbose_xy(BIVALUE_CELL *p_bivalue_array,XY_LINK *p_stack,int target_number,int row,int col) {
	int idx,l;	
	char *space_name[] = {"RC","RN","CN","BN"}; chain_string[0] = 0;
	strcpy(chain_string,space_name[manu_space_type]); sprintf(buffer,"[%d]",xy_chain_length); strcat(chain_string,buffer);
	for(l = 0; l < xy_chain_length; l++) {
		sprintf(buffer,"(r%d,c%d)",	(p_bivalue_array[p_stack[l].link].cell_index/9)+1,(p_bivalue_array[p_stack[l].link].cell_index%9)+1);
		idx = strlen(buffer); buffer[idx++] = '{'; int data = p_bivalue_array[p_stack[l].link].data;
		do {
			buffer[idx++] = bit_rank[data & -data]+49; buffer[idx++] = ',';
		} while((data &= data - 1));
		buffer[--idx] = '}'; buffer[++idx] = '-'; buffer[++idx] = '\0'; strcat(chain_string,buffer);
	}
	sprintf(buffer,"->(r%d,c%d)<>",row+1,col+1); idx = strlen(buffer); buffer[idx++] = '{'; 
	buffer[idx++] = bit_rank[target_number]+49;
	buffer[idx++] = '}'; buffer[idx] = '\0'; strcat(chain_string,buffer);
}	
//
//
int nrc_chain_caller(int chain_function_index) {
	int i,old_flag_hint,old_manu_space_type;
	int ret;
	int (*chain_function)() = nrc_chain;
	int chain_max_length = NRC_CHAIN_MAX_LENGTH;
	message[MESSAGE_SLOT] = "no nrc_chain";
	sys_slowOff(); 
	old_manu_space_type = manu_space_type;
	if(chain_function_index == NRCT_CHAIN_FUNC) {
		chain_function = nrct_chain; message[MESSAGE_SLOT] = "no nrct_chain"; chain_max_length = NRCT_CHAIN_MAX_LENGTH;
	}
	if(flag_xychain) {flag_xychain = 0; restore_screen();}
	old_flag_hint = flag_hint; flag_hint = 0;
//				before looking for nrc chain try to solve the grid with all the others methods
	chain_number = 0;
	if((ret = resolve_with_xyztchain())) {
		if(ret == -1) {sys_slowOn(); return 0;}
		else {
			display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4;	manu_space_type = RC_SPACE; 
			fill_space_table(manu_space_type); restore_screen(); sys_slowOn(); return 1;
		}	
	}
	else {if(check_grid()) {sys_slowOn(); return 0;}}
	flag_hint = old_flag_hint; 	
	for(i = 2; i < chain_max_length; i++) {	
		nrc_chain_length = i;
		if((ret = chain_function())) {
			if(ret == -1) {sys_slowOn(); return 0;}
			if(flag_hint) {sys_slowOn(); return 1;}
			if((ret = resolve_with_xyztchain())) {
				if(ret == -1) {sys_slowOn(); return 0;}
				else {
					display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4;	manu_space_type = RC_SPACE; 
					fill_space_table(manu_space_type); restore_screen(); sys_slowOn(); return 1;
				}
			}
			i = 1;
		}
	}	
	
	manu_space_type = old_manu_space_type; fill_space_table(manu_space_type); sys_slowOn();
	if(!old_flag_hint) message[MESSAGE_SLOT] = "resolve failed";
	display_message(MESSAGE_SLOT); restore_screen(); return 0;
}

//
//
void get_nrct_path(NRC_LINK *p_stack, NRCT_CELL *p_nrc_array) {
	int i,path_index;

	nrc_path[0].nrc_index = p_stack[0].link;
	
	path_index = 1;
	if(p_nrc_array[p_stack[0].link].type == C_CHAIN) {
		if(p_nrc_array[p_stack[0].link].c1_index == nrc_path[0].cell_index) {
			nrc_path[path_index].nrc_index = p_stack[0].link;
			nrc_path[path_index++].cell_index = p_nrc_array[p_stack[0].link].c2_index;
		}
		else {
			nrc_path[path_index].nrc_index = p_stack[0].link;
			nrc_path[path_index++].cell_index = p_nrc_array[p_stack[0].link].c1_index;
		}
	}
	
	for(i = 1; i < nrc_chain_length; i++) {	
		if(p_nrc_array[p_stack[i].link].type == XY_CHAIN) {
			nrc_path[path_index].nrc_index = p_stack[i].link;
			nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
		}
		else {
			if(p_nrc_array[p_stack[i-1].link].type == XY_CHAIN) {
				if(RC_share_unit_array.share_unit_array[p_nrc_array[p_stack[i-1].link].c1_index][p_nrc_array[p_stack[i].link].c1_index]) {
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
				}
				else {
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
				}
			}
			else {
				if(p_nrc_array[p_stack[i-1].link].c1_number != p_nrc_array[p_stack[i].link].c1_number) {
				
					if(nrc_path[path_index-1].cell_index == p_nrc_array[p_stack[i].link].c1_index) {
						nrc_path[path_index].nrc_index = p_stack[i].link;
						nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
					}
					else {
						nrc_path[path_index].nrc_index = p_stack[i].link;
						nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
					}				
				}
				else {
					if(RC_share_unit_array.share_unit_array[nrc_path[path_index-1].cell_index][p_nrc_array[p_stack[i].link].c1_index]) {
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
					nrc_path[path_index].nrc_index = p_stack[i].link;
					nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
					}
					else {
						nrc_path[path_index].nrc_index = p_stack[i].link;
						nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c2_index;
						nrc_path[path_index].nrc_index = p_stack[i].link;
						nrc_path[path_index++].cell_index = p_nrc_array[p_stack[i].link].c1_index;
					}
				}
			}
		}
	}
	
	xy_chain_length = path_index;
	for(i = 0; i < path_index; i++) {xy_chain_array[i].cell_index = nrc_path[i].cell_index;}
}
void do_verbose_nrct(NRCT_CELL *p_nrc_array,int c,int row,int col) {
	int idx,k;
	chain_string[0] = 0; sprintf(chain_string,"nrct[%d]",nrc_chain_length<<1);
	for(k = 0; k < xy_chain_length; k++) {
		sprintf(buffer,"(r%d,c%d)",	(nrc_path[k].cell_index/9)+1, (nrc_path[k].cell_index%9)+1);
		idx = strlen(buffer); buffer[idx++] = '{'; 
		if(p_nrc_array[nrc_path[k].nrc_index].type == XY_CHAIN) {
			buffer[idx++] = p_nrc_array[nrc_path[k].nrc_index].c1_number + 49; buffer[idx++] = ',';
			buffer[idx++] = p_nrc_array[nrc_path[k].nrc_index].c2_number + 49;
		}
		else {
			buffer[idx++] = p_nrc_array[nrc_path[k].nrc_index].c1_number + 49;
			if(k < xy_chain_length - 1 	&& p_nrc_array[nrc_path[k].nrc_index].c1_number != p_nrc_array[nrc_path[k+1].nrc_index].c1_number
										&& p_nrc_array[nrc_path[k+1].nrc_index].type != XY_CHAIN) {
				buffer[idx++] = '/'; buffer[idx++] = p_nrc_array[nrc_path[k+1].nrc_index].c1_number + 49;							
			}
		}
		buffer[idx] = '}'; buffer[++idx] = '-'; buffer[++idx] = '\0'; strcat(chain_string,buffer);
	}
	sprintf(buffer,"->(r%d,c%d)<>",row+1,col+1); idx = strlen(buffer); buffer[idx++] = '{'; 
	buffer[idx++] = bit_rank[c]+49;	
	buffer[idx++] = '}'; buffer[idx] = '\0'; strcat(chain_string,buffer);
}

//
//
int nrct_chain() {
	int i,j,k;
	int n1n2,n1,n2;
	int row,col;
	int first_cell,first_candidate;
	int out_candidate,last_candidate;
	int c,data;
	int buddy;
	int lsb,rc_index;
	int bi,bj;
	pattern[NRC_CHAIN] = 0;
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
	first_cell = first_candidate = 0;	
	NRCT_CELL nrc_array[81+(3*81)]; memset((char*) nrc_array, 0, sizeof(nrc_array)); 
	NRC_LINK stack[nrc_chain_length]; int stack_index; 
	nrc_number = 0;
	T_NUMBER t_array[nrc_chain_length];
	char candidates_path[81][9];
//				look for bivalue in abstract spaces (bilocation in rc space): for each digit	
	for(i = 0; i < 9; i++) {
//				scan the spaces rn,cn,bn
		for(k = 1; k < MAX_SPACE; k++) {
			manu_space_type = k; fill_space_table(manu_space_type); 
//				for each row col or box		
			for(j = 0; j < 9; j++) {	
				row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;
//				get the conjugate cells
				if(space_table[row][col].type == IS_CANDIDATE && BITCOUNT(space_table[row][col].data) < T_SIZE) {
					nrc_array[nrc_number].type = C_CHAIN;
					if(BITCOUNT(space_table[row][col].data) == 2) {
						n1n2 = space_table[row][col].data; n1 = bit_rank[n1n2 & -n1n2]; n2 = bit_rank[n1n2 & (n1n2 - 1)];
						switch(manu_space_type) {
							case RN_SPACE	:	nrc_array[nrc_number].c1_index 	= &space_table[row][n1] - &space_table[0][0];
												nrc_array[nrc_number].c2_index	= &space_table[row][n2] - &space_table[0][0];
												nrc_array[nrc_number].c1_number = i; nrc_array[nrc_number++].c2_number = i;
												break;
							case CN_SPACE	:	nrc_array[nrc_number].c1_index 	= &space_table[n1][row] - &space_table[0][0];
												nrc_array[nrc_number].c2_index	= &space_table[n2][row] - &space_table[0][0];
												nrc_array[nrc_number].c1_number = i; nrc_array[nrc_number++].c2_number = i;
												break;
							case BN_SPACE	: 	bi = block_start[row][0] + (n1 / 3); bj = block_start[row][1] + (n1 % 3);
												nrc_array[nrc_number].c1_index 	= &space_table[bi][bj] - &space_table[0][0];
												bi = block_start[row][0] + (n2 / 3);bj = block_start[row][1] + (n2 % 3);
												nrc_array[nrc_number].c2_index	= &space_table[bi][bj] - &space_table[0][0];
												nrc_array[nrc_number].c1_number = i; nrc_array[nrc_number++].c2_number = i;
												break;
						}		
					}
					else {
						nrc_array[nrc_number].space = manu_space_type; nrc_array[nrc_number].row = row;
						nrc_array[nrc_number].number = i; nrc_array[nrc_number++].data = space_table[row][col].data;
					}
				}
			}
		}
	}
//				look for bivalue in rc space
	manu_space_type = RC_SPACE; fill_space_table(manu_space_type); 
	for(i = 0; i < 9; i++) { for(j = 0; j < 9; j++) {		
			if(space_table[i][j].type == IS_CANDIDATE && BITCOUNT(space_table[i][j].data) < T_SIZE) {
				nrc_array[nrc_number].type = XY_CHAIN;	
				if(BITCOUNT(space_table[i][j].data) == 2) {
					nrc_array[nrc_number].c1_index = nrc_array[nrc_number].c2_index = &space_table[i][j] - &space_table[0][0]; n1n2 = space_table[i][j].data; 
					nrc_array[nrc_number].c1_number = bit_rank[n1n2 & -n1n2]; nrc_array[nrc_number++].c2_number = bit_rank[n1n2 & (n1n2 - 1)]; 
				}
				else {
					nrc_array[nrc_number].space = manu_space_type; nrc_array[nrc_number].row = i;
					nrc_array[nrc_number].number = j; nrc_array[nrc_number++].data = space_table[i][j].data;
				}
			}
	}}	
//-------------------------------------------------------------------------------------------------
	if(nrc_number >= nrc_chain_length) {
		if(flag_counter) {
			sprintf(buffer,"nt%d-%d",nrc_chain_length<<1,chain_number); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
			hpg_fill_rect(79,10,129,24); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); hpg_draw_text(buffer, 80, 16);
		}
//			build the nrct chain
		for(i = 0; i < nrc_number; i++) {	
//			first link must be bivalue
			if(nrc_array[i].data) continue; memset((char*) stack, 0, sizeof(stack)); stack_index = 0; stack[0].link = i;
			while(1) {
//			a nrct_chain of length nrc_chain_length has been formed: look for target candidate
				if(stack_index == nrc_chain_length - 1) {		
				++chain_number;		
//			display a counter 
					if(flag_counter && ((chain_number%COUNTER_STEP) == 0)) {
						sprintf(buffer,"nt%d-%d",nrc_chain_length<<1,chain_number); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
						hpg_fill_rect(79,10,129,24); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); hpg_draw_text(buffer, 80, 16);
					}
//			abort if [on] pressed
					if(keyb_isON()) {
						sprintf(buffer,"search aborted: %d",chain_number); message[MESSAGE_SLOT] = buffer; display_message(MESSAGE_SLOT); 
						manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); return -1;
					}
//			plot the candidate path
					memset((char*) candidates_path, 0, sizeof(candidates_path));
					for(j = 0; j < nrc_chain_length; j++) {
						candidates_path[nrc_array[stack[j].link].c1_index][nrc_array[stack[j].link].c1_number] = 1;
						candidates_path[nrc_array[stack[j].link].c2_index][nrc_array[stack[j].link].c2_number] = 1;
					}
//			get the chain last candidate
					last_candidate = nrc_array[stack[stack_index].link].c1_number;
					if(nrc_array[stack[stack_index].link].type == XY_CHAIN) {
						if(stack[stack_index].in_candidate == nrc_array[stack[stack_index].link].c1_number) {
							last_candidate = nrc_array[stack[stack_index].link].c2_number;
						}
					}	
//			first and last cell are the same && it is not a bivalue && first and last candidate are distincts
					if(first_cell == stack[stack_index].out_cell) {
						if(nrc_array[stack[0].link].type != XY_CHAIN && first_candidate != last_candidate) {
							row = first_cell/9; col = first_cell%9; data = sdk_cellinfo[row][col].data & ~(first_candidate | last_candidate);
							do { c = data & -data;
								if(!candidates_path[first_cell][(int)bit_rank[c]]) {
									nrc_path[0].cell_index = first_cell; get_nrct_path(stack,nrc_array);
									if(verbose) {do_verbose_nrct(nrc_array,c,row,col); sat_stack_push_string(chain_string);}
									if(flag_hint) {
										do_verbose_nrct(nrc_array,c,row,col);
										sprintf(buffer, "nrct%d:(%d,%d) -> (%d,%d): <> %d",nrc_chain_length<<1,
											(first_cell/9)+1, (first_cell%9)+1, (stack[stack_index].out_cell/9)+1, 
											(stack[stack_index].out_cell%9)+1, bit_rank[c]+1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
										hint_info.unit[0] = row; hint_info.unit[1] = col;
										cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;
									}				
									candidates[EXCLUDED][row][col] |= c; ++pattern[NRC_CHAIN];
								}
							} while((data &= data - 1));
						}
					}
					else {
//			first and last cell are linked
						if(RC_share_unit_array.share_unit_array[first_cell][stack[stack_index].out_cell]) {							
							row = first_cell/9; col = first_cell%9; data = sdk_cellinfo[row][col].data & ~first_candidate; 
							do { c = data & -data;
								if(!candidates_path[first_cell][(int)bit_rank[c]] && c == (1 << last_candidate)) {
									nrc_path[0].cell_index = first_cell; get_nrct_path(stack,nrc_array);
									if(verbose) {do_verbose_nrct(nrc_array,c,row,col); sat_stack_push_string(chain_string);}			
									if(flag_hint){
										do_verbose_nrct(nrc_array,c,row,col);
										sprintf(buffer, "nrct%d:(%d,%d) -> (%d,%d): <> %d",nrc_chain_length<<1,
											(first_cell/9)+1, (first_cell%9)+1,
											(stack[stack_index].out_cell/9)+1, (stack[stack_index].out_cell%9)+1, bit_rank[c]+1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
										hint_info.unit[0] = row; hint_info.unit[1] = col;
										cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;									
									}					
									candidates[EXCLUDED][row][col] |= c; ++pattern[NRC_CHAIN];				
								}
							} while((data &= data - 1));
							row = stack[stack_index].out_cell/9; col = stack[stack_index].out_cell%9; data = sdk_cellinfo[row][col].data & ~last_candidate; 
							do { c = data & -data;
								if(!candidates_path[stack[stack_index].out_cell][(int)bit_rank[c]] && c == (1 << first_candidate)) {
									nrc_path[0].cell_index = first_cell; get_nrct_path(stack,nrc_array);
									if(verbose) {do_verbose_nrct(nrc_array,c,row,col); sat_stack_push_string(chain_string);}						
									if(flag_hint) {
										do_verbose_nrct(nrc_array,c,row,col);
										sprintf(buffer, "nrct%d:(%d,%d) -> (%d,%d): <> %d",nrc_chain_length<<1,
											(first_cell/9)+1, (first_cell%9)+1,
											(stack[stack_index].out_cell/9)+1, (stack[stack_index].out_cell%9)+1, bit_rank[c]+1);
											message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
											hint_info.unit[0] = row; hint_info.unit[1] = col;
											cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;
									}	
									candidates[EXCLUDED][row][col] |= c; ++pattern[NRC_CHAIN];
								}
							} while((data &= data - 1));
						}
//			others chain cells 
						if(first_candidate == last_candidate) {
							for(j = 0; j < 20; j++) {		
								if((buddy = RC_buddies_array.buddies_array[first_cell][j]) == stack[stack_index].out_cell) continue;
								if(candidates_path[buddy][first_candidate]) continue;
								row = buddy/9; col = buddy%9;int digit = 1 << first_candidate;
								if(sdk_cellinfo[row][col].type == IS_CANDIDATE && sdk_cellinfo[row][col].data & digit) {
									if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][buddy]) {			
										nrc_path[0].cell_index = first_cell; get_nrct_path(stack,nrc_array);
										if(verbose) {do_verbose_nrct(nrc_array,digit,row,col); sat_stack_push_string(chain_string);}			
										if(flag_hint) {	
											do_verbose_nrct(nrc_array,digit,row,col);
											sprintf(buffer, "nrct%d:(%d,%d) -> (%d,%d): <> %d",nrc_chain_length<<1,
												(first_cell/9)+1, (first_cell%9)+1,
												(stack[stack_index].out_cell/9)+1, (stack[stack_index].out_cell%9)+1, bit_rank[digit]+1);
											message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
											hint_info.unit[0] = row; hint_info.unit[1] = col;
											cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;
											}
										candidates[EXCLUDED][row][col] |= digit; ++pattern[NRC_CHAIN];
									}
								}								
							}
						}
					}
					if(pattern[NRC_CHAIN]) {compute_candidates(); return pattern[NRC_CHAIN];}
					--stack_index;
				}
				else {
next_link:			if(stack[stack_index].next < nrc_number) {
						if(BITCOUNT(nrc_array[stack[stack_index].next].data) > (stack_index + 3)) {++stack[stack_index].next; goto next_link;}
//			check for loop
						for(j = 0; j <= stack_index; j++) {
							if(stack[stack_index].next == stack[j].link) {++stack[stack_index].next; goto next_link;}
						}
//			first link
						if(stack_index == 0) {
						
							if((data = nrc_array[stack[0].next].data)) {
								if(nrc_array[stack[0].link].type == C_CHAIN && nrc_array[stack[0].next].type == C_CHAIN
									&& nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].number) {
										
									row = nrc_array[stack[0].next].row; c = nrc_array[stack[0].next].number;
									int data1 = data;
									switch(nrc_array[stack[0].next].space) {
										case RN_SPACE	:	do { 
																lsb = data & -data; rc_index = &space_table[row][(int)bit_rank[lsb]] - &space_table[0][0];
																if(nrc_array[stack[0].link].c1_index != rc_index && RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][rc_index]) {
																	data1 &= ~lsb;
																	nrc_array[stack[0].next].c1_index = &space_table[row][(int)bit_rank[data1 & -data1]] - &space_table[0][0];
																	nrc_array[stack[0].next].c2_index = &space_table[row][(int)bit_rank[data1 & (data1 - 1)]] - &space_table[0][0];
																	nrc_array[stack[0].next].c1_number = nrc_array[stack[0].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index &&
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}
																	}
																	else {++stack[0].next; goto next_link;}
																}
																else if(nrc_array[stack[0].link].c2_index != rc_index && RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][rc_index]) {
																	data1 &= ~lsb;																	 
																	nrc_array[stack[0].next].c1_index = &space_table[row][(int)bit_rank[data1 & -data1]] - &space_table[0][0];
																	nrc_array[stack[0].next].c2_index = &space_table[row][(int)bit_rank[data1 & (data1 - 1)]] - &space_table[0][0];
																	nrc_array[stack[0].next].c1_number = nrc_array[stack[0].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index &&
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index) {	
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}																		
																	}
																	else {++stack[0].next; goto next_link;}
																}
															} while((data &= data - 1));
															++stack[0].next; goto next_link;
										case CN_SPACE	:	do { 
																lsb = data & -data; rc_index = &space_table[(int)bit_rank[lsb]][row] - &space_table[0][0];
																if(nrc_array[stack[0].link].c1_index != rc_index && RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][rc_index]) {
																	data1 &= ~lsb;
																	nrc_array[stack[0].next].c1_index = &space_table[(int)bit_rank[data1 & -data1]][row] - &space_table[0][0];
																	nrc_array[stack[0].next].c2_index = &space_table[(int)bit_rank[data1 & (data1 - 1)]][row] - &space_table[0][0];
																	nrc_array[stack[0].next].c1_number = nrc_array[stack[0].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index &&
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {
																			
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}
																	}	
																	else {++stack[0].next; goto next_link;}
																}
																else if(nrc_array[stack[0].link].c2_index != rc_index && RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][rc_index]) {
																	data1 &= ~lsb;
																	nrc_array[stack[0].next].c1_index = &space_table[(int)bit_rank[data1 & -data1]][row] - &space_table[0][0];
																	nrc_array[stack[0].next].c2_index = &space_table[(int)bit_rank[data1 & (data1 - 1)]][row] - &space_table[0][0];
																	nrc_array[stack[0].next].c1_number = nrc_array[stack[0].next].c2_number = c;	
																		
																	if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index &&
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {
																		
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}
																	}
																	else {++stack[0].next; goto next_link;}
																}
															} while((data &= data - 1));
															++stack[0].next; goto next_link;
										case BN_SPACE	: 	do { 
																lsb = data & -data; col = bit_rank[lsb]; 
																bi = block_start[row][0] + (col / 3); bj = block_start[row][1] + (col % 3);
																rc_index = &space_table[bi][bj] - &space_table[0][0];
																if(nrc_array[stack[0].link].c1_index != rc_index && RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][rc_index]) {
																	data1 &= ~lsb;
																		
																	n1 =  bit_rank[data1 & -data1]; n2 =  bit_rank[data1 & (data1 - 1)]; 
																	bi = block_start[row][0] + (n1 / 3); bj = block_start[row][1] + (n1 % 3);
																	nrc_array[stack[stack_index].next].c1_index = &space_table[bi][bj] - &space_table[0][0];
																	bi = block_start[row][0] + (n2 / 3); bj = block_start[row][1] + (n2 % 3);
																	nrc_array[stack[stack_index].next].c2_index = &space_table[bi][bj] - &space_table[0][0];
																	nrc_array[stack[stack_index].next].c1_number = nrc_array[stack[stack_index].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index &&
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {
																	
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}
																	}
																	else {++stack[0].next; goto next_link;}
																}
																else if(nrc_array[stack[0].link].c2_index != rc_index && RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][rc_index]) {
																	data1 &= ~lsb;
																		
																	n1 =  bit_rank[data1 & -data1]; n2 =  bit_rank[data1 & (data1 - 1)]; 
																	bi = block_start[row][0] + (n1 / 3); bj = block_start[row][1] + (n1 % 3);
																	nrc_array[stack[stack_index].next].c1_index = &space_table[bi][bj] - &space_table[0][0];
																	bi = block_start[row][0] + (n2 / 3); bj = block_start[row][1] + (n2 % 3);
																	nrc_array[stack[stack_index].next].c2_index = &space_table[bi][bj] - &space_table[0][0];
																	nrc_array[stack[stack_index].next].c1_number = nrc_array[stack[stack_index].next].c2_number = c;	
															
																	if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index &&
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {
																			
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}
																	}
																	else {++stack[0].next; goto next_link;}
																}
															} while((data &= data - 1));
															++stack[0].next; goto next_link;
									}	
								} 
								else if(nrc_array[stack[0].link].type == XY_CHAIN && nrc_array[stack[0].next].type == C_CHAIN &&
													(nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].number ||
											    	nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].number)) {
								
									row = nrc_array[stack[0].next].row; c = nrc_array[stack[0].next].number;
									int data1 = data;
									switch(nrc_array[stack[0].next].space) {
										case RN_SPACE	:	do { 
																lsb = data & -data; rc_index = &space_table[row][(int)bit_rank[lsb]] - &space_table[0][0];
																if(	nrc_array[stack[0].link].c1_number == c && nrc_array[stack[0].link].c1_index != rc_index && 
																	RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][rc_index]) {
																	
																	data1 &= ~lsb;
																	nrc_array[stack[0].next].c1_index = &space_table[row][(int)bit_rank[data1 & -data1]] - &space_table[0][0];
																	nrc_array[stack[0].next].c2_index = &space_table[row][(int)bit_rank[data1 & (data1 - 1)]] - &space_table[0][0];
																	nrc_array[stack[0].next].c1_number = nrc_array[stack[0].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index) {
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c1_number;
																			first_candidate = nrc_array[stack[0].link].c2_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c1_number;
																			first_candidate = nrc_array[stack[0].link].c2_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}
																	}
																	else {++stack[0].next; goto next_link;}
																}
																else if(nrc_array[stack[0].link].c2_number == c && nrc_array[stack[0].link].c2_index != rc_index && 
																		RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][rc_index]) {
																	
																	data1 &= ~lsb;																	 
																	nrc_array[stack[0].next].c1_index = &space_table[row][(int)bit_rank[data1 & -data1]] - &space_table[0][0];
																	nrc_array[stack[0].next].c2_index = &space_table[row][(int)bit_rank[data1 & (data1 - 1)]] - &space_table[0][0];
																	nrc_array[stack[0].next].c1_number = nrc_array[stack[0].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {	
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c2_number;
																			first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c2_number;
																			first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}																		
																	}
																	else {++stack[0].next; goto next_link;}
																}
															} while((data &= data - 1));
															++stack[0].next; goto next_link;
										case CN_SPACE	:	do { 
																lsb = data & -data; rc_index = &space_table[(int)bit_rank[lsb]][row] - &space_table[0][0];
																if(	nrc_array[stack[0].link].c1_number == c && nrc_array[stack[0].link].c1_index != rc_index && 
																	RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][rc_index]) {
																	
																	data1 &= ~lsb;
																	nrc_array[stack[0].next].c1_index = &space_table[(int)bit_rank[data1 & -data1]][row] - &space_table[0][0];
																	nrc_array[stack[0].next].c2_index = &space_table[(int)bit_rank[data1 & (data1 - 1)]][row] - &space_table[0][0];
																	nrc_array[stack[0].next].c1_number = nrc_array[stack[0].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index) {
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c1_number;
																			first_candidate = nrc_array[stack[0].link].c2_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c1_number;
																			first_candidate = nrc_array[stack[0].link].c2_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}
																	}
																	else {++stack[0].next; goto next_link;}
																}
																else if(nrc_array[stack[0].link].c2_number == c && nrc_array[stack[0].link].c2_index != rc_index && 
																		RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][rc_index]) {
																	
																	data1 &= ~lsb;
																	nrc_array[stack[0].next].c1_index = &space_table[(int)bit_rank[data1 & -data1]][row] - &space_table[0][0];
																	nrc_array[stack[0].next].c2_index = &space_table[(int)bit_rank[data1 & (data1 - 1)]][row] - &space_table[0][0];
																	nrc_array[stack[0].next].c1_number = nrc_array[stack[0].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {	
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c2_number;
																			first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c2_number;
																			first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}																		
																	}
																	else {++stack[0].next; goto next_link;}
																}
															} while((data &= data - 1));
															++stack[0].next; goto next_link;
										case BN_SPACE	:	do { 
																lsb = data & -data; col = bit_rank[lsb]; 
																bi = block_start[row][0] + (col / 3); bj = block_start[row][1] + (col % 3);
																rc_index = &space_table[bi][bj] - &space_table[0][0];
																if(	nrc_array[stack[0].link].c1_number == c && nrc_array[stack[0].link].c1_index != rc_index && 
																	RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][rc_index]) {
																	
																	data1 &= ~lsb;
																	n1 =  bit_rank[data1 & -data1]; n2 =  bit_rank[data1 & (data1 - 1)]; 
																	bi = block_start[row][0] + (n1 / 3); bj = block_start[row][1] + (n1 % 3);
																	nrc_array[stack[stack_index].next].c1_index = &space_table[bi][bj] - &space_table[0][0];
																	bi = block_start[row][0] + (n2 / 3); bj = block_start[row][1] + (n2 % 3);
																	nrc_array[stack[stack_index].next].c2_index = &space_table[bi][bj] - &space_table[0][0];
																	nrc_array[stack[stack_index].next].c1_number = nrc_array[stack[stack_index].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index) {
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c1_number;
																			first_candidate = nrc_array[stack[0].link].c2_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
																			first_cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c1_number;
																			first_candidate = nrc_array[stack[0].link].c2_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}
																	}
																	else {++stack[0].next; goto next_link;}
																}
																else if(nrc_array[stack[0].link].c2_number == c && nrc_array[stack[0].link].c2_index != rc_index && 
																	RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][rc_index]) {
																	
																	data1 &= ~lsb;
																	n1 =  bit_rank[data1 & -data1]; n2 =  bit_rank[data1 & (data1 - 1)]; 
																	bi = block_start[row][0] + (n1 / 3); bj = block_start[row][1] + (n1 % 3);
																	nrc_array[stack[stack_index].next].c1_index = &space_table[bi][bj] - &space_table[0][0];
																	bi = block_start[row][0] + (n2 / 3); bj = block_start[row][1] + (n2 % 3);
																	nrc_array[stack[stack_index].next].c2_index = &space_table[bi][bj] - &space_table[0][0];
																	nrc_array[stack[stack_index].next].c1_number = nrc_array[stack[stack_index].next].c2_number = c;	
																	
																	if(	nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index && 
																		nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {	
																		if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c2_number;
																			first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c2_index]) {
																			t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;		
																			first_cell = nrc_array[stack[0].link].c1_index;
																			t_array[0].cell = nrc_array[stack[0].link].c2_index;
																			t_array[0].number = t_array[1].number = nrc_array[stack[0].link].c2_number;
																			first_candidate = nrc_array[stack[0].link].c1_number;
																			goto stack_in;
																		}
																		else {++stack[0].next; goto next_link;}																		
																	}
																	else {++stack[0].next; goto next_link;}
																}
															} while((data &= data - 1));
															++stack[0].next; goto next_link;
									}
								}
								else {++stack[0].next; goto next_link;}
							}
							
							if(nrc_array[stack[0].link].type == C_CHAIN && nrc_array[stack[0].next].type == C_CHAIN) {		
//			if the candidates are the same: the cells must be distincts but rc link								
								if(nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number) {
									if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
										nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index &&
										nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index &&
										nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {
										
										if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
											t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
											first_cell = nrc_array[stack[0].link].c2_index;
											t_array[0].cell = nrc_array[stack[0].link].c1_index;
											t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
										}
										else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index]) {
											t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
											first_cell = nrc_array[stack[0].link].c2_index;
											t_array[0].cell = nrc_array[stack[0].link].c1_index;
											t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
										}
										else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index]) {
											t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;		
											first_cell = nrc_array[stack[0].link].c1_index;
											t_array[0].cell = nrc_array[stack[0].link].c2_index;
											t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
										}
										else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c2_index]) {
											t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;		
											first_cell = nrc_array[stack[0].link].c1_index;
											t_array[0].cell = nrc_array[stack[0].link].c2_index;
											t_array[0].number = t_array[1].number = first_candidate = nrc_array[stack[0].link].c1_number;
										}
										else {++stack[0].next; goto next_link;}
									}
									else {++stack[0].next; goto next_link;}
								}
//			else if they are distincts: the cells must be the same
								else {
									if(nrc_array[stack[0].link].c1_index == nrc_array[stack[0].next].c1_index) {
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;	
										first_cell = nrc_array[stack[0].link].c2_index;
										t_array[0].cell = nrc_array[stack[0].link].c1_index;
										t_array[0].number = first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[1].number = nrc_array[stack[0].next].c1_number;
									}
									else if(nrc_array[stack[0].link].c1_index == nrc_array[stack[0].next].c2_index) {
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										first_cell = nrc_array[stack[0].link].c2_index;
										t_array[0].cell = nrc_array[stack[0].link].c1_index;
										t_array[0].number = first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[1].number = nrc_array[stack[0].next].c1_number;
									}
									else if(nrc_array[stack[0].link].c2_index == nrc_array[stack[0].next].c1_index) {
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
										first_cell = nrc_array[stack[0].link].c1_index;
										t_array[0].cell = nrc_array[stack[0].link].c2_index;
										t_array[0].number = first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[1].number = nrc_array[stack[0].next].c1_number;
									}
									else if(nrc_array[stack[0].link].c2_index == nrc_array[stack[0].next].c2_index) {
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										first_cell = nrc_array[stack[0].link].c1_index;
										t_array[0].cell = nrc_array[stack[0].link].c2_index;
										t_array[0].number = first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[1].number = nrc_array[stack[0].next].c1_number;
									}
									else {++stack[0].next; goto next_link;}
								}
							}//C_CHAIN && C_CHAIN 
							if(nrc_array[stack[0].link].type == C_CHAIN && nrc_array[stack[0].next].type == XY_CHAIN) {
//			the cells must be distincts have one same candidate with cells rc link							
								if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
									nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index) {
									if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index] &&
										(	nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number ||
											nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c2_number)) {		
										first_cell = nrc_array[stack[0].link].c2_index;
										t_array[0].cell = nrc_array[stack[0].link].c1_index;
										t_array[0].number = first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										stack[1].in_candidate = nrc_array[stack[0].link].c1_number;
										if(nrc_array[stack[0].next].c1_number == stack[1].in_candidate) t_array[1].number = nrc_array[stack[0].next].c2_number;
										else t_array[1].number = nrc_array[stack[0].next].c1_number;
									}
									else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index] &&
										(	nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c1_number ||
											nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c2_number)) {
										first_cell = nrc_array[stack[0].link].c1_index;
										t_array[0].cell = nrc_array[stack[0].link].c2_index;
										t_array[0].number = first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										stack[1].in_candidate = nrc_array[stack[0].link].c2_number;
										if(nrc_array[stack[0].next].c1_number == stack[1].in_candidate) t_array[1].number = nrc_array[stack[0].next].c2_number;
										else t_array[1].number = nrc_array[stack[0].next].c1_number;
									}
									else {++stack[0].next; goto next_link;}
								}
								else {++stack[0].next; goto next_link;}
							}//C_CHAIN && XY_CHAIN 		
//			the cells must be distincts have one same candidate with cells rc link														
							if(nrc_array[stack[0].link].type == XY_CHAIN && nrc_array[stack[0].next].type == C_CHAIN) {
								if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
									nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index) {
									if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index] &&
										(	nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number ||
											nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c1_number)) {
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c2_index;
										t_array[0].cell = first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[0].number = nrc_array[stack[0].link].c2_number;
										if(nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number)
											{first_candidate = nrc_array[stack[0].link].c2_number; t_array[0].number = nrc_array[stack[0].link].c1_number;}
										t_array[1].number =  nrc_array[stack[0].next].c1_number;
									}
									else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index] &&
										(	nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c2_number ||
											nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c2_number)) {
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										t_array[0].cell = first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[0].number = nrc_array[stack[0].link].c2_number;
										if(nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c2_number)
											{first_candidate = nrc_array[stack[0].link].c2_number; t_array[0].number = nrc_array[stack[0].link].c1_number;}
										t_array[1].number =  nrc_array[stack[0].next].c1_number;	
									}
									else {++stack[0].next; goto next_link;}
								}
								else {++stack[0].next; goto next_link;}
							}//XY_CHAIN && C_CHAIN
//			the cells must be distincts have one same candidate with cells rc link														
							if(nrc_array[stack[0].link].type == XY_CHAIN && nrc_array[stack[0].next].type == XY_CHAIN) {
								if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
									if (	nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number ||
											nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c2_number) {
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										stack[1].in_candidate = nrc_array[stack[0].link].c1_number;
										t_array[0].cell = first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c2_number;
										t_array[0].number = nrc_array[stack[0].link].c1_number;
										if(nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number) 
											t_array[1].number =  nrc_array[stack[0].next].c2_number;
										else t_array[1].number =  nrc_array[stack[0].next].c1_number;	
									}
									else if (	nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c1_number ||
												nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c2_number) {
										t_array[1].cell = stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										stack[1].in_candidate = nrc_array[stack[0].link].c2_number;
										t_array[0].cell = first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
										t_array[0].number = nrc_array[stack[0].link].c2_number; 
										if(nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c1_number) 
											t_array[1].number =  nrc_array[stack[0].next].c2_number;
										else t_array[1].number =  nrc_array[stack[0].next].c1_number;	
									}
									else {++stack[0].next; goto next_link;}
								}
								else {++stack[0].next; goto next_link;}
							}//XY_CHAIN && XY_CHAIN

stack_in:					stack[1].link = stack[0].next; ++stack[0].next; stack[++stack_index].next = 0;
						}//stack_index == 0
//			others links than the first
						else {
							if((data = nrc_array[stack[stack_index].next].data)) {
								row = nrc_array[stack[stack_index].next].row; c = nrc_array[stack[stack_index].next].number;
								int data1 = data;
								switch(nrc_array[stack[stack_index].next].space) {
									case RN_SPACE	:	do { 
															lsb = data & -data;
															rc_index = &space_table[row][(int)bit_rank[lsb]] - &space_table[0][0];
															for(j = 0; j <= stack_index; j++) {
																if(t_array[j].number == c) {
																	if(t_array[j].cell != rc_index && RC_share_unit_array.share_unit_array[t_array[j].cell][rc_index]) {
																		if(BITCOUNT((data1 &= ~lsb)) == 2) goto rn_fill;
																		else break;
																	}
																}
																else {
																	if(t_array[j].cell == rc_index) {
																		if(BITCOUNT((data1 &= ~lsb)) == 2) goto rn_fill;
																		else break;																	
																	}
																}
															}
														} while((data &= data - 1));
														++stack[stack_index].next; goto next_link;
										rn_fill		:	nrc_array[stack[stack_index].next].c1_index = &space_table[row][(int)bit_rank[data1 & -data1]] - &space_table[0][0];
														nrc_array[stack[stack_index].next].c2_index = &space_table[row][(int)bit_rank[data1 & (data1 - 1)]] - &space_table[0][0];
														nrc_array[stack[stack_index].next].c1_number = nrc_array[stack[stack_index].next].c2_number = c;	
														break;	
									case CN_SPACE	:	do { 
															lsb = data & -data;
															rc_index = &space_table[(int)bit_rank[lsb]][row] - &space_table[0][0];
															for(j = 0; j <= stack_index; j++) {
																if(t_array[j].number == c) {
																	if(t_array[j].cell != rc_index && RC_share_unit_array.share_unit_array[t_array[j].cell][rc_index]) {
																		if(BITCOUNT((data1 &= ~lsb)) == 2) goto cn_fill;
																		else break;
																	}
																}
																else {
																	if(t_array[j].cell == rc_index) {
																		if(BITCOUNT((data1 &= ~lsb)) == 2) goto cn_fill;
																		else break;																	
																	}
																}
															}
														} while((data &= data - 1));
														++stack[stack_index].next; goto next_link;
										cn_fill		:	nrc_array[stack[stack_index].next].c1_index = &space_table[(int)bit_rank[data1 & -data1]][row] - &space_table[0][0];
														nrc_array[stack[stack_index].next].c2_index = &space_table[(int)bit_rank[data1 & (data1 - 1)]][row] - &space_table[0][0];
														nrc_array[stack[stack_index].next].c1_number = nrc_array[stack[stack_index].next].c2_number = c;
														break;	
									case BN_SPACE	: 	do { 
															lsb = data & -data; col =  bit_rank[lsb]; 
															bi = block_start[row][0] + (col / 3); bj = block_start[row][1] + (col % 3);
															rc_index = &space_table[bi][bj] - &space_table[0][0];
															for(j = 0; j <= stack_index; j++) {
																if(t_array[j].number == c) {
																	if(t_array[j].cell != rc_index && RC_share_unit_array.share_unit_array[t_array[j].cell][rc_index]) {
																		if(BITCOUNT((data1 &= ~lsb)) == 2) goto bn_fill;
																		else break;
																	}
																}
																else {
																	if(t_array[j].cell == rc_index) {
																		if(BITCOUNT((data1 &= ~lsb)) == 2) goto bn_fill;
																		else break;																	
																	}
																}
															}
														} while((data &= data - 1));
														++stack[stack_index].next; goto next_link;
										bn_fill		:	n1 =  bit_rank[data1 & -data1]; n2 =  bit_rank[data1 & (data1 - 1)]; 
														bi = block_start[row][0] + (n1 / 3); bj = block_start[row][1] + (n1 % 3);
														nrc_array[stack[stack_index].next].c1_index = &space_table[bi][bj] - &space_table[0][0];
														bi = block_start[row][0] + (n2 / 3);bj = block_start[row][1] + (n2 % 3);
														nrc_array[stack[stack_index].next].c2_index = &space_table[bi][bj] - &space_table[0][0];
														nrc_array[stack[stack_index].next].c1_number = nrc_array[stack[stack_index].next].c2_number = c;
														break;	
									case RC_SPACE	:	col = c; rc_index = &space_table[row][col] - &space_table[0][0];
														do { 
															lsb = data & -data; c = bit_rank[lsb]; 
															for(j = 0; j <= stack_index; j++) {
																if(t_array[j].number == c) {
																	if(t_array[j].cell != rc_index && RC_share_unit_array.share_unit_array[t_array[j].cell][rc_index]) {
																		if(BITCOUNT((data1 &= ~lsb)) == 2)goto rc_fill;
																		else break;
																	}
																}
																else {
																	if(t_array[j].cell == rc_index) {
																		if(BITCOUNT((data1 &= ~lsb)) == 2) goto rc_fill;
																		else break;																	
																	}
																}
															}
														} while((data &= data - 1));
														++stack[stack_index].next; goto next_link;
										rc_fill		:	nrc_array[stack[stack_index].next].c1_index = nrc_array[stack[stack_index].next].c2_index = rc_index;
														nrc_array[stack[stack_index].next].c1_number = bit_rank[data1 & -data1];
														nrc_array[stack[stack_index].next].c2_number = bit_rank[data1 & (data1 - 1)];
														break;	
								}									
							}						
							if(nrc_array[stack[stack_index].link].type == C_CHAIN && nrc_array[stack[stack_index].next].type == C_CHAIN) {
//			if the candidates are the same: the cells must be distincts but rc link
								if(nrc_array[stack[stack_index].link].c1_number == nrc_array[stack[stack_index].next].c1_number) {
									if(	nrc_array[stack[stack_index].link].c1_index != nrc_array[stack[stack_index].next].c1_index && 
										nrc_array[stack[stack_index].link].c1_index != nrc_array[stack[stack_index].next].c2_index && 
										nrc_array[stack[stack_index].link].c2_index != nrc_array[stack[stack_index].next].c1_index && 
										nrc_array[stack[stack_index].link].c2_index != nrc_array[stack[stack_index].next].c2_index) {
										if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c1_index]) {
											t_array[stack_index+1].cell = stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c2_index;
											t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c1_number;
										}
										else if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c2_index]) {
											t_array[stack_index+1].cell = stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
											t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c1_number;
										}
										else {++stack[stack_index].next; goto next_link;}
									}
									else {++stack[stack_index].next; goto next_link;}
								}
//			else if they are distincts: the cells must be the same
								else {
									if(stack[stack_index].out_cell == nrc_array[stack[stack_index].next].c1_index) {
										t_array[stack_index+1].cell = stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c2_index;
										t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c1_number;
									}
									else if(stack[stack_index].out_cell == nrc_array[stack[stack_index].next].c2_index) {
										t_array[stack_index+1].cell = stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
										t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c1_number;
									}
									else {++stack[stack_index].next; goto next_link;}
								}
							}//C_CHAIN && C_CHAIN 
							else if(nrc_array[stack[stack_index].link].type == C_CHAIN && nrc_array[stack[stack_index].next].type == XY_CHAIN) {
//			the cells must be distincts have one same candidate with cells rc link
								if(	nrc_array[stack[stack_index].link].c1_index != nrc_array[stack[stack_index].next].c1_index &&
									nrc_array[stack[stack_index].link].c2_index != nrc_array[stack[stack_index].next].c1_index &&
										RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c1_index] &&
										(nrc_array[stack[stack_index].link].c1_number == nrc_array[stack[stack_index].next].c1_number ||
										nrc_array[stack[stack_index].link].c1_number == nrc_array[stack[stack_index].next].c2_number)) {
									t_array[stack_index+1].cell = stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
									stack[stack_index+1].in_candidate = nrc_array[stack[stack_index].link].c1_number;
									if(nrc_array[stack[stack_index].next].c1_number == stack[stack_index+1].in_candidate) 
										t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c2_number;
									else t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c1_number;
								}
								else {++stack[stack_index].next; goto next_link;}
							}//C_CHAIN && XY_CHAIN  
							else if(nrc_array[stack[stack_index].link].type == XY_CHAIN && nrc_array[stack[stack_index].next].type == C_CHAIN) {
								if(	stack[stack_index].out_cell != nrc_array[stack[stack_index].next].c1_index && 
									stack[stack_index].out_cell != nrc_array[stack[stack_index].next].c2_index) {
									
									if(stack[stack_index].in_candidate == nrc_array[stack[stack_index].link].c1_number) 
										{out_candidate = nrc_array[stack[stack_index].link].c2_number;}
									else {out_candidate = nrc_array[stack[stack_index].link].c1_number;}
									
									if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c1_index] &&
										out_candidate == nrc_array[stack[stack_index].next].c1_number) {
										t_array[stack_index+1].cell = stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c2_index;
										t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c2_number;
									}									
									else if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c2_index] &&
										out_candidate == nrc_array[stack[stack_index].next].c2_number) {
										t_array[stack_index+1].cell = stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
										t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c1_number;									
									}									
									else {++stack[stack_index].next; goto next_link;}
								}
								else {++stack[stack_index].next; goto next_link;}
							}//XY_CHAIN && C_CHAIN
							else if(nrc_array[stack[stack_index].link].type == XY_CHAIN && nrc_array[stack[stack_index].next].type == XY_CHAIN) {
								if(stack[stack_index].in_candidate == nrc_array[stack[stack_index].link].c1_number) 
									{out_candidate = nrc_array[stack[stack_index].link].c2_number;}
								else {out_candidate = nrc_array[stack[stack_index].link].c1_number;}
							
								if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c1_index] &&						
								(out_candidate == nrc_array[stack[stack_index].next].c1_number || out_candidate == nrc_array[stack[stack_index].next].c2_number)) {
									
									t_array[stack_index+1].cell = stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
									stack[stack_index+1].in_candidate = out_candidate;
									if(nrc_array[stack[stack_index].next].c1_number == stack[stack_index+1].in_candidate)
										t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c2_number;
									else t_array[stack_index+1].number = nrc_array[stack[stack_index].next].c1_number;		
								}
								else {++stack[stack_index].next; goto next_link;}
							}//XY_CHAIN && XY_CHAIN
					
							stack[stack_index+1].link = stack[stack_index].next; ++stack[stack_index].next; stack[++stack_index].next = 0;
						}//stack_index != 0
					}
					else {
						if(stack_index - 1 == -1) break; else {--stack_index; goto next_link;}
					}
				}//next_link
			}//while(1)
		}//build the nrct chain
	}//	if there are enough cells
	return pattern[NRC_CHAIN];
}

int nrc_chain() {
	int i,j,k;
	int n1n2,n1,n2;
	int row,col;
	int first_cell,first_candidate;
	int out_candidate,last_candidate;
	int c,data;
	int buddy;
	int bi,bj;
	pattern[NRC_CHAIN] = 0; 
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
	first_cell = first_candidate = 0;	
	NRC_CELL nrc_array[81+(3*81)]; memset((char*) nrc_array, 0, sizeof(nrc_array)); 
	NRC_LINK stack[nrc_chain_length]; int stack_index; 
	nrc_number = 0;
	char candidates_path[81][9];
//				look for bivalue in abstract spaces (bilocation in rc space)
//				for each digit	
	for(i = 0; i < 9; i++) {
//				scan the spaces rn,cn,bn
		for(k = 1; k < MAX_SPACE; k++) {
			manu_space_type = k; fill_space_table(manu_space_type); 
//				for each row col or box		
			for(j = 0; j < 9; j++) {	
				row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;
//				get the conjugate cells
				if(space_table[row][col].type == IS_CANDIDATE && BITCOUNT(space_table[row][col].data) == 2) {
					nrc_array[nrc_number].type = C_CHAIN; n1n2 = space_table[row][col].data; n1 = bit_rank[n1n2 & -n1n2]; n2 = bit_rank[n1n2 & (n1n2 - 1)];
					switch(manu_space_type) {
						case RN_SPACE	:	nrc_array[nrc_number].c1_index 	= &space_table[row][n1] - &space_table[0][0];
											nrc_array[nrc_number].c2_index	= &space_table[row][n2] - &space_table[0][0];
											nrc_array[nrc_number].c1_number = i; nrc_array[nrc_number++].c2_number = i;
											break;
						case CN_SPACE	:	nrc_array[nrc_number].c1_index 	= &space_table[n1][row] - &space_table[0][0];
											nrc_array[nrc_number].c2_index	= &space_table[n2][row] - &space_table[0][0];
											nrc_array[nrc_number].c1_number = i; nrc_array[nrc_number++].c2_number = i;
											break;
						case BN_SPACE	: 	bi = block_start[row][0] + (n1 / 3); bj = block_start[row][1] + (n1 % 3);
											nrc_array[nrc_number].c1_index 	= &space_table[bi][bj] - &space_table[0][0];
											bi = block_start[row][0] + (n2 / 3);bj = block_start[row][1] + (n2 % 3);
											nrc_array[nrc_number].c2_index	= &space_table[bi][bj] - &space_table[0][0];
											nrc_array[nrc_number].c1_number = i; nrc_array[nrc_number++].c2_number = i;
											break;
					}						
				}
			}
		}
	}
//				look for bivalue in rc space
	manu_space_type = RC_SPACE; fill_space_table(manu_space_type); 
	for(i = 0; i < 9; i++) { for(j = 0; j < 9; j++) {		
			if(space_table[i][j].type == IS_CANDIDATE && BITCOUNT(space_table[i][j].data) == 2) {
				nrc_array[nrc_number].type = XY_CHAIN;	
				nrc_array[nrc_number].c1_index = nrc_array[nrc_number].c2_index = &space_table[i][j] - &space_table[0][0]; n1n2 = space_table[i][j].data; 
				nrc_array[nrc_number].c1_number = bit_rank[n1n2 & -n1n2]; nrc_array[nrc_number++].c2_number = bit_rank[n1n2 & (n1n2 - 1)]; 				
			}
	}}	
//-------------------------------------------------------------------------------------------------
	if(nrc_number >= nrc_chain_length) {
		if(flag_counter) {
			sprintf(buffer,"n%d-%d",nrc_chain_length<<1,chain_number); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
			hpg_fill_rect(79,10,129,24); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); hpg_draw_text(buffer, 80, 16);
		}
//			build the nrc chain
		for(i = 0; i < nrc_number; i++) {	
			memset((char*) stack, 0, sizeof(stack)); stack_index = 0; stack[0].link = i;
			while(1) {
//			a nrc_chain of length nrc_chain_length has been formed: look for target candidate
				if(stack_index == nrc_chain_length - 1) {
				++chain_number; 
//			display a counter 
					if(flag_counter && ((chain_number%COUNTER_STEP) == 0)) {
						sprintf(buffer,"n%d-%d",nrc_chain_length<<1,chain_number); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
						hpg_fill_rect(79,10,129,24); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); hpg_draw_text(buffer, 80, 16);
					}
//			abort if [on] pressed
					if(keyb_isON()) {
						sprintf(buffer,"search aborted: %d",chain_number); message[MESSAGE_SLOT] = buffer; display_message(MESSAGE_SLOT); 
						manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); return -1;
					}
//			plot the candidate path
					memset((char*) candidates_path, 0, sizeof(candidates_path));
					for(j = 0; j < nrc_chain_length; j++) {
						candidates_path[nrc_array[stack[j].link].c1_index][nrc_array[stack[j].link].c1_number] = 1;
						candidates_path[nrc_array[stack[j].link].c2_index][nrc_array[stack[j].link].c2_number] = 1;
					}
//			get the chain last candidate
					last_candidate = nrc_array[stack[stack_index].link].c1_number;
					if(nrc_array[stack[stack_index].link].type == XY_CHAIN) {
						if(stack[stack_index].in_candidate == nrc_array[stack[stack_index].link].c1_number) {
							last_candidate = nrc_array[stack[stack_index].link].c2_number;
						}
					}	
//			first and last cell are the same && it is not a bivalue && first and last candidate are distincts
					if(first_cell == stack[stack_index].out_cell) {
						if(nrc_array[stack[0].link].type != XY_CHAIN && first_candidate != last_candidate) {
							row = first_cell/9; col = first_cell%9; data = sdk_cellinfo[row][col].data & ~(first_candidate | last_candidate);
							do { c = data & -data;
								if(!candidates_path[first_cell][(int)bit_rank[c]]) {
									nrc_path[0].cell_index = first_cell; get_nrc_path(stack,nrc_array);
									if(verbose) {do_verbose_nrc(nrc_array,c,row,col); sat_stack_push_string(chain_string);}								
									if(flag_hint) {
										do_verbose_nrc(nrc_array,c,row,col);
										sprintf(buffer, "nrc%d:(%d,%d) -> (%d,%d): <> %d",nrc_chain_length<<1,
											(first_cell/9)+1, (first_cell%9)+1, (stack[stack_index].out_cell/9)+1, 
											(stack[stack_index].out_cell%9)+1, (int)bit_rank[c]+1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
										hint_info.unit[0] = row; hint_info.unit[1] = col;
										cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;
									}				
									candidates[EXCLUDED][row][col] |= c; ++pattern[NRC_CHAIN];
								}
							} while((data &= data - 1));
						}
					}

					else {
//			first and last cell are linked
						if(RC_share_unit_array.share_unit_array[first_cell][stack[stack_index].out_cell]) {							
							row = first_cell/9; col = first_cell%9; data = sdk_cellinfo[row][col].data & ~first_candidate; 
							do { c = data & -data;
								if(!candidates_path[first_cell][(int)bit_rank[c]] && c == (1 << last_candidate)) {
									nrc_path[0].cell_index = first_cell; get_nrc_path(stack,nrc_array);
									if(verbose) {do_verbose_nrc(nrc_array,c,row,col); sat_stack_push_string(chain_string);}								
									if(flag_hint){
										do_verbose_nrc(nrc_array,c,row,col);
										sprintf(buffer, "nrc%d:(%d,%d) -> (%d,%d): <> %d",nrc_chain_length<<1,
											(first_cell/9)+1, (first_cell%9)+1,
											(stack[stack_index].out_cell/9)+1, (stack[stack_index].out_cell%9)+1, (int)bit_rank[c]+1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
										hint_info.unit[0] = row; hint_info.unit[1] = col;
										cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;									
									}					
									candidates[EXCLUDED][row][col] |= c; ++pattern[NRC_CHAIN];				
								}
							} while((data &= data - 1));
							row = stack[stack_index].out_cell/9; col = stack[stack_index].out_cell%9; data = sdk_cellinfo[row][col].data & ~last_candidate; 
							do { c = data & -data;
								if(!candidates_path[stack[stack_index].out_cell][(int)bit_rank[c]] && c == (1 << first_candidate)) {
									nrc_path[0].cell_index = first_cell; get_nrc_path(stack,nrc_array);
									if(verbose) {do_verbose_nrc(nrc_array,c,row,col); sat_stack_push_string(chain_string);}								
									if(flag_hint) {
										do_verbose_nrc(nrc_array,c,row,col);
										sprintf(buffer, "nrc%d:(%d,%d) -> (%d,%d): <> %d",nrc_chain_length<<1,
											(first_cell/9)+1, (first_cell%9)+1,
											(stack[stack_index].out_cell/9)+1, (stack[stack_index].out_cell%9)+1, (int)bit_rank[c]+1);
											message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
											hint_info.unit[0] = row; hint_info.unit[1] = col;
											cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;
									}	
									candidates[EXCLUDED][row][col] |= c; ++pattern[NRC_CHAIN];
								}
							} while((data &= data - 1));
						}

//			others chain cells 
						if(first_candidate == last_candidate) {
							for(j = 0; j < 20; j++) {		
								if((buddy = RC_buddies_array.buddies_array[first_cell][j]) == stack[stack_index].out_cell) continue;
								if(candidates_path[buddy][first_candidate]) continue;
								row = buddy/9; col = buddy%9;int digit = 1 << first_candidate;
								if(sdk_cellinfo[row][col].type == IS_CANDIDATE && sdk_cellinfo[row][col].data & digit) {
									if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][buddy]) {			
										nrc_path[0].cell_index = first_cell; get_nrc_path(stack,nrc_array);
									if(verbose) {do_verbose_nrc(nrc_array,digit,row,col); sat_stack_push_string(chain_string);}								
										if(flag_hint) {	
											do_verbose_nrc(nrc_array,digit,row,col);
											sprintf(buffer, "nrc%d:(%d,%d) -> (%d,%d): <> %d",nrc_chain_length<<1,
												(first_cell/9)+1, (first_cell%9)+1,
												(stack[stack_index].out_cell/9)+1, (stack[stack_index].out_cell%9)+1, (int)bit_rank[digit]+1);
											message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
											hint_info.unit[0] = row; hint_info.unit[1] = col;
											cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;
											}
										candidates[EXCLUDED][row][col] |= digit; ++pattern[NRC_CHAIN];
									}
								}								
							}
						}
					}

					if(pattern[NRC_CHAIN]) {compute_candidates(); return pattern[NRC_CHAIN];}
					--stack_index;
				}
				else {
next_link:			if(stack[stack_index].next < nrc_number) {
//			check for loop
						for(j = 0; j <= stack_index; j++) {
							if(stack[stack_index].next == stack[j].link) {++stack[stack_index].next; goto next_link;}
						}
//			first link
						if(stack_index == 0) {
							if(nrc_array[stack[0].link].type == C_CHAIN && nrc_array[stack[0].next].type == C_CHAIN) {		
//			if the candidates are the same: the cells must be distincts but rc link								
								if(nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number) {
									if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
										nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index &&
										nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index &&
										nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c2_index) {
										
										if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
											stack[1].out_cell = nrc_array[stack[0].next].c2_index;
											first_cell = nrc_array[stack[0].link].c2_index;
											first_candidate = nrc_array[stack[0].link].c1_number;
										}
										else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index]) {
											stack[1].out_cell = nrc_array[stack[0].next].c1_index;
											first_cell = nrc_array[stack[0].link].c2_index;
											first_candidate = nrc_array[stack[0].link].c1_number;
										}
										else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index]) {
											stack[1].out_cell = nrc_array[stack[0].next].c2_index;		
											first_cell = nrc_array[stack[0].link].c1_index;
											first_candidate = nrc_array[stack[0].link].c1_number;
										}
										else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c2_index]) {
											stack[1].out_cell = nrc_array[stack[0].next].c1_index;		
											first_cell = nrc_array[stack[0].link].c1_index;
											first_candidate = nrc_array[stack[0].link].c1_number;
										}
										else {++stack[0].next; goto next_link;}
									}
									else {++stack[0].next; goto next_link;}
								}
//			else if they are distincts: the cells must be the same
								else {
									if(nrc_array[stack[0].link].c1_index == nrc_array[stack[0].next].c1_index) {
										stack[1].out_cell = nrc_array[stack[0].next].c2_index;	
										first_cell = nrc_array[stack[0].link].c2_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
									}
									else if(nrc_array[stack[0].link].c1_index == nrc_array[stack[0].next].c2_index) {
										stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										first_cell = nrc_array[stack[0].link].c2_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
									}
									else if(nrc_array[stack[0].link].c2_index == nrc_array[stack[0].next].c1_index) {
										stack[1].out_cell = nrc_array[stack[0].next].c2_index;
										first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
									}
									else if(nrc_array[stack[0].link].c2_index == nrc_array[stack[0].next].c2_index) {
										stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
									}
									else {++stack[0].next; goto next_link;}
								}
							}//C_CHAIN && C_CHAIN 
							if(nrc_array[stack[0].link].type == C_CHAIN && nrc_array[stack[0].next].type == XY_CHAIN) {
//			the cells must be distincts have one same candidate with cells rc link							
								if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
									nrc_array[stack[0].link].c2_index != nrc_array[stack[0].next].c1_index) {
									if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index] &&
										(	nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number ||
											nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c2_number)) {		
										first_cell = nrc_array[stack[0].link].c2_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
										stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										stack[1].in_candidate = nrc_array[stack[0].link].c1_number;
									}
									else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c2_index][nrc_array[stack[0].next].c1_index] &&
										(	nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c1_number ||
											nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c2_number)) {
										first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
										stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										stack[1].in_candidate = nrc_array[stack[0].link].c2_number;
									}
									else {++stack[0].next; goto next_link;}
								}
								else {++stack[0].next; goto next_link;}
							}//C_CHAIN && XY_CHAIN 		
//			the cells must be distincts have one same candidate with cells rc link														
							if(nrc_array[stack[0].link].type == XY_CHAIN && nrc_array[stack[0].next].type == C_CHAIN) {
								if(	nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c1_index && 
									nrc_array[stack[0].link].c1_index != nrc_array[stack[0].next].c2_index) {
									if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index] &&
										(	nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number ||
											nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c1_number)) {
										stack[1].out_cell = nrc_array[stack[0].next].c2_index;
										first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
										if(nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number)
											{first_candidate = nrc_array[stack[0].link].c2_number;}
									}
									else if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c2_index] &&
										(	nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c2_number ||
											nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c2_number)) {
										stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
										if(nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c2_number)
											{first_candidate = nrc_array[stack[0].link].c2_number;}
									}
									else {++stack[0].next; goto next_link;}
								}
								else {++stack[0].next; goto next_link;}
							}//XY_CHAIN && C_CHAIN
//			the cells must be distincts have one same candidate with cells rc link														
							if(nrc_array[stack[0].link].type == XY_CHAIN && nrc_array[stack[0].next].type == XY_CHAIN) {
								if(RC_share_unit_array.share_unit_array[nrc_array[stack[0].link].c1_index][nrc_array[stack[0].next].c1_index]) {
									if (	nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c1_number ||
											nrc_array[stack[0].link].c1_number == nrc_array[stack[0].next].c2_number) {
										stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										stack[1].in_candidate = nrc_array[stack[0].link].c1_number;
										first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c2_number;
									}
									else if (	nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c1_number ||
												nrc_array[stack[0].link].c2_number == nrc_array[stack[0].next].c2_number) {
										stack[1].out_cell = nrc_array[stack[0].next].c1_index;
										stack[1].in_candidate = nrc_array[stack[0].link].c2_number;
										first_cell = nrc_array[stack[0].link].c1_index;
										first_candidate = nrc_array[stack[0].link].c1_number;
									}
									else {++stack[0].next; goto next_link;}
								}
								else {++stack[0].next; goto next_link;}
							}//XY_CHAIN && XY_CHAIN

							stack[1].link = stack[0].next; ++stack[0].next; stack[++stack_index].next = 0;
						}//stack_index == 0
//			others links than the first
						else {
							if(nrc_array[stack[stack_index].link].type == C_CHAIN && nrc_array[stack[stack_index].next].type == C_CHAIN) {
//			if the candidates are the same: the cells must be distincts but rc link
								if(nrc_array[stack[stack_index].link].c1_number == nrc_array[stack[stack_index].next].c1_number) {
									if(	nrc_array[stack[stack_index].link].c1_index != nrc_array[stack[stack_index].next].c1_index && 
										nrc_array[stack[stack_index].link].c1_index != nrc_array[stack[stack_index].next].c2_index && 
										nrc_array[stack[stack_index].link].c2_index != nrc_array[stack[stack_index].next].c1_index && 
										nrc_array[stack[stack_index].link].c2_index != nrc_array[stack[stack_index].next].c2_index) {
										if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c1_index]) {
											stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c2_index;
										}
										else if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c2_index]) {
											stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
										}
										else {++stack[stack_index].next; goto next_link;}
									}
									else {++stack[stack_index].next; goto next_link;}
								}
//			else if they are distincts: the cells must be the same
								else {
									if(stack[stack_index].out_cell == nrc_array[stack[stack_index].next].c1_index) {
										stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c2_index;
									}
									else if(stack[stack_index].out_cell == nrc_array[stack[stack_index].next].c2_index) {
										stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
									}
									else {++stack[stack_index].next; goto next_link;}
								}
							}//C_CHAIN && C_CHAIN 
							else if(nrc_array[stack[stack_index].link].type == C_CHAIN && nrc_array[stack[stack_index].next].type == XY_CHAIN) {
//			the cells must be distincts have one same candidate with cells rc link
								if(	nrc_array[stack[stack_index].link].c1_index != nrc_array[stack[stack_index].next].c1_index &&
									nrc_array[stack[stack_index].link].c2_index != nrc_array[stack[stack_index].next].c1_index &&
										RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c1_index] &&
										(nrc_array[stack[stack_index].link].c1_number == nrc_array[stack[stack_index].next].c1_number ||
										nrc_array[stack[stack_index].link].c1_number == nrc_array[stack[stack_index].next].c2_number)) {
									stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
									stack[stack_index+1].in_candidate = nrc_array[stack[stack_index].link].c1_number;
								}
								else {++stack[stack_index].next; goto next_link;}
							}//C_CHAIN && XY_CHAIN  
							else if(nrc_array[stack[stack_index].link].type == XY_CHAIN && nrc_array[stack[stack_index].next].type == C_CHAIN) {
								if(	stack[stack_index].out_cell != nrc_array[stack[stack_index].next].c1_index && 
									stack[stack_index].out_cell != nrc_array[stack[stack_index].next].c2_index) {
									
									if(stack[stack_index].in_candidate == nrc_array[stack[stack_index].link].c1_number) 
										{out_candidate = nrc_array[stack[stack_index].link].c2_number;}
									else {out_candidate = nrc_array[stack[stack_index].link].c1_number;}
									
									if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c1_index] &&
										out_candidate == nrc_array[stack[stack_index].next].c1_number) {
										stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c2_index;
									}									
									else if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c2_index] &&
										out_candidate == nrc_array[stack[stack_index].next].c2_number) {
										stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;										
									}									
									else {++stack[stack_index].next; goto next_link;}
								}
								else {++stack[stack_index].next; goto next_link;}
							}//XY_CHAIN && C_CHAIN
							else if(nrc_array[stack[stack_index].link].type == XY_CHAIN && nrc_array[stack[stack_index].next].type == XY_CHAIN) {
								if(stack[stack_index].in_candidate == nrc_array[stack[stack_index].link].c1_number) 
									{out_candidate = nrc_array[stack[stack_index].link].c2_number;}
								else {out_candidate = nrc_array[stack[stack_index].link].c1_number;}
							
								if(RC_share_unit_array.share_unit_array[stack[stack_index].out_cell][nrc_array[stack[stack_index].next].c1_index] &&						
								(out_candidate == nrc_array[stack[stack_index].next].c1_number || out_candidate == nrc_array[stack[stack_index].next].c2_number)) {
									
									stack[stack_index+1].out_cell = nrc_array[stack[stack_index].next].c1_index;
									stack[stack_index+1].in_candidate = out_candidate;
								}
								else {++stack[stack_index].next; goto next_link;}
							}//XY_CHAIN && XY_CHAIN
					
							stack[stack_index+1].link = stack[stack_index].next; ++stack[stack_index].next; stack[++stack_index].next = 0;
						}//stack_index != 0
					}
					else {
						if(stack_index - 1 == -1) break; else {--stack_index; goto next_link;}
					}
				}//next_link
			}//while(1)
		}//build the nrc chain
	}//	if there are enough cells
	return pattern[NRC_CHAIN];
}
//
//
int resolve_with_xyztchain() {
	int i,j;
	if(resolve_with_cchain()) {return 1;}
	int success = 0; int ret;
	while(1) {
		for(i = 3; i < XY_CHAIN_MAX_LENGTH; i++) {	
			xy_chain_length = i;
			for(j = 0; j < MAX_SPACE-1; j++) {
				manu_space_type = j; fill_space_table(manu_space_type); 
//				chain found
				if((ret = xyzt_chain())) {
					if(ret == -1) return -1;				
					if(resolve_with_cchain()) {return 1;}
//				this chain was not enough to solve the grid: try to find another one
					success = 1;
				}
			}
		}
		if(success) success = 0; else break;
	}
	return 0;
}

int resolve_with_cchain() {
	int i;
	if(resolve_grid()) {return 1;}
	for(i = 3; i < C_CHAIN_MAX_LENGTH; i += 2) {	
		conjugate_chain_length = i;
//				chain found
		if(conjugate_chain()) {
			if(resolve_grid()) {return 1;}
//				this chain was not enough to solve the grid: try to find another one
			else {i = 1;}
		}
	}
	return 0;
}
//
//
int c_chain_caller() {
	int i,old_flag_hint,old_manu_space_type;
	sys_slowOff(); 
	old_flag_hint = flag_hint; flag_hint = 0;
//				before looking for chains try to solve the grid with all the others methods
	if(resolve_grid()) {
		display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4; manu_space_type = RC_SPACE; 
		fill_space_table(manu_space_type); flag_xychain = 0; restore_screen(); return 1;
	}
	else {if(check_grid()) return 0;}
	flag_hint = old_flag_hint; old_manu_space_type = manu_space_type;
	for(i = 3; i < C_CHAIN_MAX_LENGTH; i += 2) {	
		conjugate_chain_length = i;
//				chain found
		if(conjugate_chain()) {
			if(flag_hint) {sys_slowOn(); return 1;}
			if(resolve_grid()) {
				display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4;	manu_space_type = RC_SPACE; 
				fill_space_table(manu_space_type); restore_screen(); sys_slowOn(); return 1;
			}
//				this chain was not enough to solve the grid: try to find another one
			else {i = 1;}
		}
	}
//				no chain found
	manu_space_type = old_manu_space_type; fill_space_table(manu_space_type); sys_slowOn();
	if(!old_flag_hint) message[MESSAGE_SLOT] = "resolve failed";
	else message[MESSAGE_SLOT] = "no conjugacy chain"; 
	display_message(MESSAGE_SLOT); restore_screen(); return 0;
}

int conjugate_chain() {
	int i,j,k,l;
	int bi,bj;
	int row,col,old_manu_space_type,first_cell;
	first_cell = 0; CONJUGATE_CELL conjugate_cell_array[27]; int conjugate_cell_index;
	CONJUGATE_LINK stack[conjugate_chain_length]; int stack_index; 
	pattern[C_CHAIN] = 0; old_manu_space_type = manu_space_type;
//				for each digit
	for(i = 0; i < 9; i++) {
		memset((char*) conjugate_cell_array, 0, sizeof(conjugate_cell_array)); conjugate_cell_index = 0;
//				scan the spaces rn,cn,bn
		for(k = 1; k < MAX_SPACE; k++) {
			manu_space_type = k; fill_space_table(manu_space_type); 
//				for each row col or box		
			for(j = 0; j < 9; j++) {	
				row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;
//				get the conjugate cells
				if(space_table[row][col].type == IS_CANDIDATE && BITCOUNT(space_table[row][col].data) == 2) {
					int _0 = space_table[row][col].data; int _1 = bit_rank[_0 & -_0]; int _2 = bit_rank[_0 & (_0 - 1)]; 
					switch(manu_space_type) {
						case RN_SPACE	:	conjugate_cell_array[conjugate_cell_index].c1_index 	= &space_table[row][_1] - &space_table[0][0];
											conjugate_cell_array[conjugate_cell_index++].c2_index	= &space_table[row][_2] - &space_table[0][0];
											break;
						case CN_SPACE	:	conjugate_cell_array[conjugate_cell_index].c1_index 	= &space_table[_1][row] - &space_table[0][0];
											conjugate_cell_array[conjugate_cell_index++].c2_index	= &space_table[_2][row] - &space_table[0][0];
											break;
						case BN_SPACE	: 	bi = block_start[row][0] + (_1 / 3); bj = block_start[row][1] + (_1 % 3);
											conjugate_cell_array[conjugate_cell_index].c1_index 	= &space_table[bi][bj] - &space_table[0][0];
											bi = block_start[row][0] + (_2 / 3);bj = block_start[row][1] + (_2 % 3);
											conjugate_cell_array[conjugate_cell_index++].c2_index	= &space_table[bi][bj] - &space_table[0][0];
											break;
					}
				}
			}//for each row col or box
		}//scan the spaces rn,cn,bn
//			if there are enough cells
		if(conjugate_cell_index >= conjugate_chain_length) {
//			build the chains for each conjugate cell
			for(j = 0; j < conjugate_cell_index; j++) {	
				memset((char*) stack, 0, sizeof(stack)); stack_index = 0; stack[stack_index].link = j;
				while(1) {
//			a conjugate_chain of length conjugate_chain_length has been formed
					if(stack_index == conjugate_chain_length - 1) {
						manu_space_type = RC_SPACE; fill_space_table(manu_space_type); 
//			look for target cells
						for(k = 0; k < 20; k++) {		
							int buddy = RC_buddies_array.buddies_array[first_cell][k];
							int row = buddy/9; int col = buddy%9; int digit = 1 << i;
							if(space_table[row][col].type == IS_CANDIDATE && space_table[row][col].data & digit) {
								if(RC_share_unit_array.share_unit_array[stack[stack_index].link_cell][buddy]) {							
//			the target cell must not be in the chain
									for(l = 0; l < conjugate_chain_length; l++) {
										if(	buddy == conjugate_cell_array[stack[l].link].c1_index || buddy == conjugate_cell_array[stack[l].link].c2_index)
											break;
									}
//			an elimination has been found
									if(l == conjugate_chain_length) {
										if(flag_hint) {
											if(flag_xychain) {flag_xychain = 0; restore_screen();}
											conjugate_chain_array[0] = conjugate_cell_array[stack[0].link].c1_index; 
											conjugate_chain_array[1] = conjugate_cell_array[stack[0].link].c2_index; 
											for(l = 1; l < conjugate_chain_length; l++) {conjugate_chain_array[l+1] = stack[l].link_cell;}
											hint_info.type = C_CHAIN; hint_info.candidate = i; filtered_candidate = i + 1;
											sprintf(buffer, "c%d_chain:(%d,%d) -> (%d,%d): <> %d",conjugate_chain_length+1,
												(first_cell/9)+1, (first_cell%9)+1,
												(stack[stack_index].link_cell/9)+1, (stack[stack_index].link_cell%9)+1, i+1);
											message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
											flag_cchain = 1; restore_screen(); cursor.x = row; cursor.y = col; unit_type = ROW; 
											display_candidates(); return 1;
										}
										candidates[EXCLUDED][row][col] |= digit; ++pattern[C_CHAIN];
									}
								}		
							}			
						}//	look for target cells
						if(pattern[C_CHAIN]) {compute_candidates(); return pattern[C_CHAIN];}
						--stack_index; 
					}
					else {
next_link:				if(stack[stack_index].next < conjugate_cell_index) {
//			check for loop
							for(l = 0; l <= stack_index; l++) {
								if(stack[stack_index].next == stack[l].link) {++stack[stack_index].next; goto next_link;}
							}
//			first link							
							if(stack_index == 0) {
								if(conjugate_cell_array[stack[0].link].c1_index == conjugate_cell_array[stack[0].next].c1_index) {
									stack[1].link_cell = conjugate_cell_array[stack[0].next].c2_index;
									first_cell = conjugate_cell_array[stack[0].link].c2_index;
								}
								else if(conjugate_cell_array[stack[0].link].c1_index == conjugate_cell_array[stack[0].next].c2_index) {	
									stack[1].link_cell = conjugate_cell_array[stack[0].next].c1_index;
									first_cell = conjugate_cell_array[stack[0].link].c2_index;
								}
								else if(conjugate_cell_array[stack[0].link].c2_index == conjugate_cell_array[stack[0].next].c1_index) {
									stack[1].link_cell = conjugate_cell_array[stack[0].next].c2_index;		
									first_cell = conjugate_cell_array[stack[0].link].c1_index;
								}
								else if(conjugate_cell_array[stack[0].link].c2_index == conjugate_cell_array[stack[0].next].c2_index) {
									stack[1].link_cell = conjugate_cell_array[stack[0].next].c1_index;		
									first_cell = conjugate_cell_array[stack[0].link].c1_index;
								}							
								else {++stack[0].next; goto next_link;}
								stack[1].link = stack[0].next; ++stack[0].next; stack[++stack_index].next = 0;
							}
							else {
								if(stack[stack_index].link_cell == conjugate_cell_array[stack[stack_index].next].c1_index) {
									stack[stack_index+1].link_cell = conjugate_cell_array[stack[stack_index].next].c2_index;
								}
								else if(stack[stack_index].link_cell == conjugate_cell_array[stack[stack_index].next].c2_index) {
									stack[stack_index+1].link_cell = conjugate_cell_array[stack[stack_index].next].c1_index;
								}
								else {++stack[stack_index].next; goto next_link;}
								stack[stack_index+1].link = stack[stack_index].next; ++stack[stack_index].next; stack[++stack_index].next = 0;
							}						
						}
						else {
							if(stack_index - 1 == -1) break; else {--stack_index; goto next_link;}
						}
					}
				}//while(1)
			}//	build the chains for each conjugate cell
		}//	if there are enough cells
	}//for each digit
	return pattern[C_CHAIN];
}

void highlight_cchain() {
	int i,row,col;
	flag_cchain = 0;
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
	for(i = 0; i < conjugate_chain_length+1; i++) {
		row = conjugate_chain_array[i]/9; col = conjugate_chain_array[i]%9;
		hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
		hpg_fill_rect(coordinates[row][col].x-3,coordinates[row][col].y-2,coordinates[row][col].x+4,coordinates[row][col].y+5);
		hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
		hpg_draw_letter(sdk_digit[hint_info.candidate+1], coordinates[row][col].x, coordinates[row][col].y);
	}
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); 
}
//
//
int xy_chain_caller(int chain_function_index) {
	int i,j;
	int old_flag_hint,old_manu_space_type;
	int (*chain_function)() = xy_chain;
	sys_slowOff(); 
	old_manu_space_type = manu_space_type;
	switch(chain_function_index) {
		case XY_CHAIN_FUNC		: chain_function = xy_chain; 	message[MESSAGE_SLOT] = "no xy_chain";   break;
		case XYT_CHAIN_FUNC		: chain_function = xyt_chain;	message[MESSAGE_SLOT] = "no xyt_chain";  break;
		case XYZT_CHAIN_FUNC	: chain_function = xyzt_chain; 	message[MESSAGE_SLOT] = "no xyzt_chain"; break;
	}
	if(flag_xychain) {flag_xychain = 0; restore_screen();}
	old_flag_hint = flag_hint; flag_hint = 0;
//				before looking for chains try to solve the grid with all the others methods
	if(resolve_with_cchain()) {
		display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4;	manu_space_type = RC_SPACE; 
		fill_space_table(manu_space_type); restore_screen(); return 1;
	}
	else {if(check_grid()) return 0;}
	flag_hint = old_flag_hint; 
//				look for chains from length 3 to max_length in rc rn cn spaces
	chain_number = 0; int ret; int success = 0;
	while(1) {
		for(i = 3; i < XY_CHAIN_MAX_LENGTH; i++) {	
			xy_chain_length = i;
			for(j = 0; j < MAX_SPACE-1; j++) {
				manu_space_type = j; fill_space_table(manu_space_type); 
//				chain found
				if((ret = chain_function())) {
					if(ret == -1) 	{sys_slowOn(); return 0;}
					if(flag_hint) {sys_slowOn(); return 1;}
					if(resolve_with_cchain()) {
						display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4; manu_space_type = RC_SPACE; 
						fill_space_table(manu_space_type); restore_screen(); sys_slowOn(); return 1;
					}
					success = 1;
				}
			}
		}
		if(success) success = 0; else break;
	}
//				no more chain found and grid not solved
	manu_space_type = old_manu_space_type; fill_space_table(manu_space_type); sys_slowOn();	
	if(!old_flag_hint) message[MESSAGE_SLOT] = "resolve failed";
	display_message(MESSAGE_SLOT); restore_screen(); return 0;
}

int xyzt_chain() {
	int i,j,k,l;
	int c,target_number,candidate_link;	
	int z_link;
	int t_array[XY_CHAIN_MAX_LENGTH];
	int split_flag,split_c1,split_c2,swap;
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
	pattern[XY_CHAIN] = 0; 
	BIVALUE_CELL bivalue_array[MAX_BIVALUE_CELLS]; memset((char*) bivalue_array, 0, sizeof(bivalue_array)); 
	for(i = 0, bivalue_number = 0; i < 9; i++) { for(j = 0; j < 9; j++) {		
			if(space_table[i][j].type == IS_CANDIDATE) {
				bivalue_array[bivalue_number].cell_index = &space_table[i][j] - &space_table[0][0];
				bivalue_array[bivalue_number++].data = space_table[i][j].data;		
			}
	}}
//						sort the cells according to their size
	qsort((void *) bivalue_array, bivalue_number, sizeof(BIVALUE_CELL), compare_bivalue_cell);		
	int adjacency_array[bivalue_number][21]; 
	if(manu_space_type == RC_SPACE) {p_share_unit_array = &RC_share_unit_array; p_buddies_array = &RC_buddies_array;}
	else {p_share_unit_array = &RNCN_share_unit_array;  p_buddies_array = &RNCN_buddies_array;}
//			build the graph of unit link	
	for(i = 0; i < bivalue_number; i++) { for(j = 0, k = 0; j < bivalue_number; j++) {	
			if(i == j) continue;
			if(p_share_unit_array->share_unit_array[bivalue_array[i].cell_index][bivalue_array[j].cell_index]) {adjacency_array[i][k++] = j;}}
		adjacency_array[i][k] = -1;
	}
	XY_LINK stack[xy_chain_length]; int stack_index, next_link;
//--------------------------------------------------------------------------------------------------------------------------------------------------	
//			build all the chains for each value
	for(i = 0; i < bivalue_number; i++) {
//			an isolated value
		if(adjacency_array[i][0] == -1) continue; memset((char*) stack, 0, sizeof(stack)); stack_index = 0; stack[stack_index].link = i;
		while(1) {
//			a xyt_chain of length xy_chain_length has been formed
			if(stack_index == xy_chain_length - 1) {	
			++chain_number;
//			display a counter 
					if(flag_counter && ((chain_number%COUNTER_STEP) == 0)) {
					sprintf(buffer,"xz%d-%d",xy_chain_length,chain_number); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
					hpg_fill_rect(79,10,129,24); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); hpg_draw_text(buffer, 80, 16);
				}
				if(keyb_isON()) {
					sprintf(buffer,"search aborted: %d",chain_number); message[MESSAGE_SLOT] = buffer; display_message(MESSAGE_SLOT); 
					manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); return -1;
				}		
				split_flag = split_c1 = split_c2 = 0;
//			first 2 links
				if((c = bivalue_array[stack[0].link].data & bivalue_array[stack[1].link].data)) {	
					if(!(l = bivalue_array[stack[0].link].data & bivalue_array[stack[stack_index].link].data)) {--stack_index; goto next_link;}
					if(BITCOUNT(bivalue_array[stack[1].link].data) == 3) {
						if(BITCOUNT(c) == 2 && BITCOUNT(l) == 1) {
							t_array[0] = c & ~l; target_number = l; 
							t_array[1] = candidate_link = bivalue_array[stack[1].link].data & ~c;
							z_link = 1; goto end_of_chain;
						}
						else {stack_index = 0; goto next_link;}
					}	
//			the first 2 links are the same: split the chain so the variables can be initialised
					if(BITCOUNT(c) == 2) {split_c1 = c & -c; split_c2 = c & (c - 1); split_flag = 1;}
//			the first 2 links are distincts: initialise the variables then process the chain from link 2	
					else {
						t_array[0] = c; t_array[1] = candidate_link = bivalue_array[stack[1].link].data & ~c;
						target_number = bivalue_array[stack[0].link].data & ~c; z_link = 0; goto end_of_chain;
					}
split_label: 		swap = split_c1; split_c1 = split_c2; split_c2 = swap;
					t_array[0] = split_c1; target_number = t_array[1] = candidate_link = split_c2; z_link = 0;
//			process the chain from link 2
end_of_chain:		for(j = 2; j < xy_chain_length; j++) {
						if(candidate_link & bivalue_array[stack[j].link].data) {		
//			xyt link							
							if(BITCOUNT(bivalue_array[stack[j].link].data) > 2) {
//			remove left link
								int link_data = bivalue_array[stack[j].link].data & ~candidate_link;	
//			last link
								if(j == stack_index) {
									if(!(target_number & link_data)) {--stack_index; goto next_link;}
									link_data &= ~target_number;
//			check the t candidates
									for(k = 0; k < j - 1; k++) {
										if(p_share_unit_array->share_unit_array	[bivalue_array[stack[j].link].cell_index][bivalue_array[stack[k].link].cell_index]) {
											if(BITCOUNT((link_data &= ~t_array[k])) == 0) break; 											
										}
									}
//			all the extra candidates are accounted for
									if(!link_data) {candidate_link = target_number;}
//			not a valide link
									else {
										if(split_flag) {split_flag = 0; goto split_label;} else {--stack_index; goto next_link;}
									}
								}//	last link
//			intermediat link
								else {
									if((c = link_data & bivalue_array[stack[j+1].link].data)) {
// 			1 cas: only one right linking
										if(BITCOUNT(c) == 1) {
//			remove right link and fixe variables
											link_data &= ~c; t_array[j] = candidate_link = c;
//			check the t candidates
											for(k = 0; k < j - 1; k++) {
												if(p_share_unit_array->share_unit_array	[bivalue_array[stack[j].link].cell_index]
																						[bivalue_array[stack[k].link].cell_index]) {
													if(BITCOUNT((link_data &= ~t_array[k])) == 0) break; 											
												}
											}
											if(link_data) {
												if(BITCOUNT(link_data) == 1 && !z_link && link_data & target_number) {z_link = j;}
												else {if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = j; goto next_link;}}
											}	
										}
//			2 cas: 2 possibles right linking
										else {
//			check the t candidates
											for(k = 0; k < j - 1; k++) {
												if(p_share_unit_array->share_unit_array	[bivalue_array[stack[j].link].cell_index]
																						[bivalue_array[stack[k].link].cell_index]) {
													if(BITCOUNT((link_data &= ~t_array[k])) == 1) break; 											
												}
											}
//			all the extra candidates are accounted for
											if(BITCOUNT(link_data) == 1) {t_array[j] = candidate_link = link_data;}
//			one is not: can be the z candidate									
											else if(!z_link && BITCOUNT(link_data) == 2 && link_data & target_number) {
												t_array[j] = candidate_link = link_data & ~target_number; z_link = j;
											}
//			not a valide link
											else {
												if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = j - 1; goto next_link;}
											}
										}
									}
									else {
										if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = j; goto next_link;}
									}
								}//	intermediat link
							}//xyt link
//			xy link
							else {t_array[j] = candidate_link = bivalue_array[stack[j].link].data & ~candidate_link;}	
						}//links are linked
						else {
							if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = j - 1; goto next_link;}
						}
					}//process the chain from link 2: end_of_chain	
//----------------------------------------------------------------------------------------------------------------------------------------------
//														all links have been validated
//----------------------------------------------------------------------------------------------------------------------------------------------
//			look for target cells if last right candidate == first left candidate
					if(candidate_link == target_number) {
						c = bivalue_array[stack[0].link].cell_index; int max_j = manu_space_type == RC_SPACE ? 20 : 16;
//			scan the buddies of the start cell for any target cell
						for(j = 0; j < max_j; j++) {		
							int buddy = p_buddies_array->buddies_array[c][j]; int row = buddy/9; int col = buddy%9;
							if(space_table[row][col].type == IS_CANDIDATE && space_table[row][col].data & target_number && 
									p_share_unit_array->share_unit_array[bivalue_array[stack[xy_chain_length - 1].link].cell_index][buddy]) {		
//			the target cell must not be in the chain
								for(l = 0; l < xy_chain_length; l++) {if(buddy == bivalue_array[stack[l].link].cell_index) break;}
//			an elimination has been found
								if(l == xy_chain_length) {
									if(z_link) {
										if(!p_share_unit_array->share_unit_array[bivalue_array[stack[z_link].link].cell_index][buddy]) {
											if(split_flag) {split_flag = 0; goto split_label;}	 else {stack_index = z_link - 1; goto next_link;}
										}
									}	
									if(verbose) {do_verbose_xy(bivalue_array,stack,target_number,row,col); sat_stack_push_string(chain_string);}
									if(flag_hint) {
										do_verbose_xy(bivalue_array,stack,target_number,row,col);
										for(l = 0; l < xy_chain_length; l++) {xy_chain_array[l] = bivalue_array[stack[l].link];}
										hint_info.type = XYZT_CHAIN;		
										sprintf(buffer, "hxyzt%d:(%d,%d) -> (%d,%d): <> %d",xy_chain_length,
												(xy_chain_array[0].cell_index/9)+1, (xy_chain_array[0].cell_index%9)+1,
												(xy_chain_array[xy_chain_length-1].cell_index/9)+1, (xy_chain_array[xy_chain_length-1].cell_index%9)+1,
												bit_rank[target_number]+1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
										hint_info.unit[0] = row; hint_info.unit[1] = col;
										cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;
									}
//											rcspace->rcspace
									if(manu_space_type == RC_SPACE) {candidates[EXCLUDED][row][col] |= target_number;}
//											rnspace->rcspace
									else if(manu_space_type == RN_SPACE) {	candidates[EXCLUDED][row][(int)bit_rank[target_number]] |= 1 << col;}
//											cnspace->rcspace
									else if(manu_space_type == CN_SPACE) {candidates[EXCLUDED][(int)bit_rank[target_number]][row] |= 1 << col;}
									++pattern[XY_CHAIN];
								}//a target found
							}//a potential target found
						}//search all buddies for target
						if(pattern[XY_CHAIN]) {compute_candidates(); fill_space_table(manu_space_type);}
					}//look for target
					if(split_flag) {split_flag = 0; goto split_label;}	 else {--stack_index; goto next_link;}
				}//if(link_0 & link_1)
//			no common candidate between first and second link
				else {stack_index = 0; goto next_link;}
			}//chain of xy_chain_length formed
//			if there is a successor for the current link			
			else {
next_link:		if((next_link = adjacency_array[stack[stack_index].link][stack[stack_index].next]) != -1) {
					++stack[stack_index].next; 
//			check for loop
					for(l = 0; l < stack_index; l++) {if(stack[l].link == next_link) goto next_link;}
//			first link must be bivalue
					if(stack_index == 0) {if(BITCOUNT(bivalue_array[stack[stack_index].link].data) > 2) goto next_link;}
					else {
						if(BITCOUNT(bivalue_array[stack[stack_index].link].data) > (stack_index + 2)) {goto next_link;}
					}
					stack[++stack_index].link = next_link; stack[stack_index].next = 0;
				}
				else {
					if(stack_index - 1 == -1) break; else {--stack_index; goto next_link;}
				}
			}
		}//while(1) 
	}//build all the chains for each value
	return pattern[XY_CHAIN];	
}

int xyt_chain() {
	int i,j,k,l;
	int c,target_number,candidate_link;	
	int z_link;
	int t_array[XY_CHAIN_MAX_LENGTH];
	int split_flag,split_c1,split_c2,swap;
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
	pattern[XY_CHAIN] = 0; 
	BIVALUE_CELL bivalue_array[MAX_BIVALUE_CELLS]; memset((char*) bivalue_array, 0, sizeof(bivalue_array)); 
	for(i = 0, bivalue_number = 0; i < 9; i++) { for(j = 0; j < 9; j++) {		
			if(space_table[i][j].type == IS_CANDIDATE) {
				bivalue_array[bivalue_number].cell_index = &space_table[i][j] - &space_table[0][0];
				bivalue_array[bivalue_number++].data = space_table[i][j].data;		
			}
	}}
//						sort the cells according to their size
	qsort((void *) bivalue_array, bivalue_number, sizeof(BIVALUE_CELL), compare_bivalue_cell);		
	int adjacency_array[bivalue_number][21]; 
	if(manu_space_type == RC_SPACE) {p_share_unit_array = &RC_share_unit_array; p_buddies_array = &RC_buddies_array;}
	else {p_share_unit_array = &RNCN_share_unit_array;  p_buddies_array = &RNCN_buddies_array;}
//			build the graph of unit link	
	for(i = 0; i < bivalue_number; i++) { for(j = 0, k = 0; j < bivalue_number; j++) {	
			if(i == j) continue;
			if(p_share_unit_array->share_unit_array[bivalue_array[i].cell_index][bivalue_array[j].cell_index]) {adjacency_array[i][k++] = j;}}
		adjacency_array[i][k] = -1;
	}
	XY_LINK stack[xy_chain_length]; int stack_index, next_link;
//--------------------------------------------------------------------------------------------------------------------------------------------------	
//			build all the chains for each value
	for(i = 0; i < bivalue_number; i++) {
//			an isolated value
		if(adjacency_array[i][0] == -1) continue; memset((char*) stack, 0, sizeof(stack)); stack_index = 0; stack[stack_index].link = i;		
		while(1) {
//			a xyt_chain of length xy_chain_length has been formed
			if(stack_index == xy_chain_length - 1) {	
				++chain_number; 
//			display a counter 
					if(flag_counter && ((chain_number%COUNTER_STEP) == 0)) {
					sprintf(buffer,"xt%d-%d",xy_chain_length,chain_number); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
					hpg_fill_rect(79,10,129,24); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); hpg_draw_text(buffer, 80, 16);
				}
				if(keyb_isON()) {
					sprintf(buffer,"search aborted: %d",chain_number); message[MESSAGE_SLOT] = buffer; display_message(MESSAGE_SLOT); 
					manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); return -1;
				}			
				split_flag = split_c1 = split_c2 = 0;
//			first 2 links
				if((c = bivalue_array[stack[0].link].data & bivalue_array[stack[1].link].data)) {	
//			the first 2 links are the same: split the chain so the variables can be initialised
					if(BITCOUNT(c) == 2) {split_c1 = c & -c; split_c2 = c & (c - 1); split_flag = 1;}
//			the first 2 links are distincts: initialise the variables then process the chain from link 2	
					else {
						t_array[0] = c; t_array[1] = candidate_link = bivalue_array[stack[1].link].data & ~c;
						target_number = bivalue_array[stack[0].link].data & ~c; z_link = 0; goto end_of_chain;
					}
split_label: 		swap = split_c1; split_c1 = split_c2; split_c2 = swap;
					t_array[0] = split_c1; target_number = t_array[1] = candidate_link = split_c2; z_link = 0;
//			process the chain from link 2
end_of_chain:		for(j = 2; j < xy_chain_length; j++) {
						if(candidate_link & bivalue_array[stack[j].link].data) {		
//			xyt link							
							if(BITCOUNT(bivalue_array[stack[j].link].data) > 2) {
//			remove left link
								int link_data = bivalue_array[stack[j].link].data & ~candidate_link;	
//			last link
								if(j == stack_index) {
									if(!(target_number & link_data)) {--stack_index; goto next_link;} link_data &= ~target_number;
//			check the t candidates
									for(k = 0; k < j - 1; k++) {
										if(p_share_unit_array->share_unit_array	[bivalue_array[stack[j].link].cell_index][bivalue_array[stack[k].link].cell_index]) {
											if(BITCOUNT((link_data &= ~t_array[k])) == 0) break; 											
										}
									}
//			all the extra candidates are accounted for
									if(!link_data) {candidate_link = target_number;}
//			not a valide link
									else {
										if(split_flag) {split_flag = 0; goto split_label;} else {--stack_index; goto next_link;}
									}
								}//	last link
//			intermediat link
								else {
									if((c = link_data & bivalue_array[stack[j+1].link].data)) {
// 			1 cas: only one right linking
										if(BITCOUNT(c) == 1) {
//			remove right link and fixe variables
											link_data &= ~c; t_array[j] = candidate_link = c;
//			check the t candidates
											for(k = 0; k < j - 1; k++) {
												if(p_share_unit_array->share_unit_array	[bivalue_array[stack[j].link].cell_index]
																						[bivalue_array[stack[k].link].cell_index]) {
													if(BITCOUNT((link_data &= ~t_array[k])) == 0) break; 											
												}
											}
											if(link_data) {
												if(BITCOUNT(link_data) == 1 && !z_link && link_data & target_number) {z_link = j;}
												else {if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = j; goto next_link;}}
											}	
										}
//			2 cas: 2 possibles right linking
										else {
//			check the t candidates
											for(k = 0; k < j - 1; k++) {
												if(p_share_unit_array->share_unit_array	[bivalue_array[stack[j].link].cell_index]
																						[bivalue_array[stack[k].link].cell_index]) {
													if(BITCOUNT((link_data &= ~t_array[k])) == 1) break; 											
												}
											}
//			all the extra candidates are accounted for
											if(BITCOUNT(link_data) == 1) {t_array[j] = candidate_link = link_data;}
//			one is not: can be the z candidate									
											else if(!z_link && BITCOUNT(link_data) == 2 && link_data & target_number) {
												t_array[j] = candidate_link = link_data & ~target_number; z_link = j;
											}
//			not a valide link
											else {
												if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = j - 1; goto next_link;}
											}
										}
									}
									else {
										if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = j; goto next_link;}
									}
								}//	intermediat link
							}//xyt link
//			xy link
							else {t_array[j] = candidate_link = bivalue_array[stack[j].link].data & ~candidate_link;}	
						}//links are linked
						else {
							if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = j - 1; goto next_link;}
						}
					}//process the chain from link 2: end_of_chain
//----------------------------------------------------------------------------------------------------------------------------------------------
//														all links have been validated
//----------------------------------------------------------------------------------------------------------------------------------------------
//			look for target cells if last right candidate == first left candidate	
					if(candidate_link == target_number) {
						c = bivalue_array[stack[0].link].cell_index; int max_j = manu_space_type == RC_SPACE ? 20 : 16;				
//			scan the buddies of the start cell for any target cell
						for(j = 0; j < max_j; j++) {		
							int buddy = p_buddies_array->buddies_array[c][j]; int row = buddy/9; int col = buddy%9;
							if(space_table[row][col].type == IS_CANDIDATE && space_table[row][col].data & target_number && 
									p_share_unit_array->share_unit_array[bivalue_array[stack[xy_chain_length - 1].link].cell_index][buddy]) {		
//			the target cell must not be in the chain
								for(l = 0; l < xy_chain_length; l++) {if(buddy == bivalue_array[stack[l].link].cell_index) break;}
//			an elimination has been found
								if(l == xy_chain_length) {
									if(z_link) {
										if(!p_share_unit_array->share_unit_array[bivalue_array[stack[z_link].link].cell_index][buddy]) 
											{if(split_flag) {split_flag = 0; goto split_label;} else {stack_index = z_link - 1; goto next_link;}
										}
									}	
									if(verbose) {do_verbose_xy(bivalue_array,stack,target_number,row,col); sat_stack_push_string(chain_string);}
									if(flag_hint) {
										do_verbose_xy(bivalue_array,stack,target_number,row,col);
										for(l = 0; l < xy_chain_length; l++) {xy_chain_array[l] = bivalue_array[stack[l].link];}
										hint_info.type = XYT_CHAIN;		
										sprintf(buffer, "hxyt%d:(%d,%d) -> (%d,%d): <> %d",xy_chain_length,
											(xy_chain_array[0].cell_index/9)+1, (xy_chain_array[0].cell_index%9)+1,
											(xy_chain_array[xy_chain_length-1].cell_index/9)+1, (xy_chain_array[xy_chain_length-1].cell_index%9)+1,
											bit_rank[target_number]+1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); flag_xychain = 1; restore_screen(); 
										hint_info.unit[0] = row; hint_info.unit[1] = col;
										cursor.x = row; cursor.y = col; unit_type = ROW; display_candidates(); return 1;
									}
//											rcspace->rcspace
									if(manu_space_type == RC_SPACE) {candidates[EXCLUDED][row][col] |= target_number;}
//											rnspace->rcspace
									else if(manu_space_type == RN_SPACE) {	candidates[EXCLUDED][row][(int)bit_rank[target_number]] |= 1 << col;}
//											cnspace->rcspace
									else if(manu_space_type == CN_SPACE) {candidates[EXCLUDED][(int)bit_rank[target_number]][row] |= 1 << col;}
									++pattern[XY_CHAIN];
								}//a target found
							}//a potential target found
						}//search all buddies for target
						if(pattern[XY_CHAIN]) {compute_candidates(); fill_space_table(manu_space_type);}
					}//look for target
					if(split_flag) {split_flag = 0; goto split_label;}	 else {--stack_index; goto next_link;}
				}//if(link_0 & link_1)
//			no common candidate between first and second link
				else {stack_index = 0; goto next_link;}
			}//chain of xy_chain_length formed
//			if there is a successor for the current link			
			else {
next_link:		if((next_link = adjacency_array[stack[stack_index].link][stack[stack_index].next]) != -1) {
					++stack[stack_index].next; 
//			check for loop
					for(l = 0; l < stack_index; l++) {if(stack[l].link == next_link) goto next_link;}
//			first 2 links must be bivalue
					if(stack_index == 0 || stack_index == 1 ) {if(BITCOUNT(bivalue_array[stack[stack_index].link].data) > 2) goto next_link;}
//			number of candidates must match link rank
					else {if(BITCOUNT(bivalue_array[stack[stack_index].link].data) > (stack_index + 1)) {goto next_link;}}
					stack[++stack_index].link = next_link; stack[stack_index].next = 0;
				}
				else {if(--stack_index == -1) break; else {goto next_link;}}
			}
		}//while(1) 
	}//build all the chains for each value
	return pattern[XY_CHAIN];	
}

int xy_chain() {
	int i,j,k,l;
	int c,target_number,candidate_link;	
	int t1,t2;
	t1 = t2 = 0; pattern[XY_CHAIN] = 0; 
	BIVALUE_CELL bivalue_array[MAX_BIVALUE_CELLS]; memset((char*) bivalue_array, 0, sizeof(bivalue_array)); 
	for(i = 0, bivalue_number = 0; i < 9; i++) { for(j = 0; j < 9; j++) {		
			if(space_table[i][j].type == IS_CANDIDATE && BITCOUNT(space_table[i][j].data) == 2) {
				bivalue_array[bivalue_number].cell_index = &space_table[i][j] - &space_table[0][0];
				bivalue_array[bivalue_number++].data = space_table[i][j].data;		
			}
	}}
//--------------------------------------------------------------------------	
	int adjacency_array[bivalue_number][21]; 
	if(manu_space_type == RC_SPACE) {p_share_unit_array = &RC_share_unit_array; p_buddies_array = &RC_buddies_array;}
	else {p_share_unit_array = &RNCN_share_unit_array;  p_buddies_array = &RNCN_buddies_array;}
//			build the graph of unit link	
	for(i = 0; i < bivalue_number; i++) {
		for(j = 0, k = 0; j < bivalue_number; j++) {	
			if(i == j) continue;
			if(p_share_unit_array->share_unit_array[bivalue_array[i].cell_index][bivalue_array[j].cell_index]) adjacency_array[i][k++] = j;
		}
		adjacency_array[i][k] = -1;
	}
	XY_LINK stack[xy_chain_length];	int stack_index ,next_link;
//			build all the chains for each bivalue
	for(i = 0; i < bivalue_number; i++) {
//			an isolated bivalue
		if(adjacency_array[i][0] == -1) continue; memset((char*) stack, 0, sizeof(stack)); stack_index = 0; stack[stack_index].link = i;
		while(1) {
//			a xy_chain of length xy_chain_length has been formed
			if(stack_index == xy_chain_length - 1) {
//			first link
				if((c = bivalue_array[stack[0].link].data & bivalue_array[stack[1].link].data)) {		
//			the 2 candidates are the same	
					if(BITCOUNT(c) == 2) {
						target_number = candidate_link = c;
						for(j = 2; j < xy_chain_length; j++) {
							if((c = candidate_link & bivalue_array[stack[j].link].data) ) {
//			chain is resolved before the end
								if(BITCOUNT(c) == 1) {
//			even or odd number of links
									if(j & 1) target_number = bivalue_array[stack[0].link].data & ~c; else target_number = c;
									candidate_link = bivalue_array[stack[j].link].data & ~c;
//			the rest of the chain
									for(j = j + 1; j < xy_chain_length; j++) {
										if((c = candidate_link & bivalue_array[stack[j].link].data)) {candidate_link = bivalue_array[stack[j].link].data & ~c;}	
										else {stack_index = j - 1; goto next_link;}
									}	
								}
							}	
							else {stack_index = j - 1; goto next_link;}
						}
//			the 2 candidates are the same for each link
						if(xy_chain_length & 1) {--stack_index; goto next_link;}
					}
//			the 2 candidates are distincts
					else {
						target_number = bivalue_array[stack[0].link].data & ~c; candidate_link = bivalue_array[stack[1].link].data & ~c;
						for(j = 2; j < xy_chain_length; j++) {
							if((c = candidate_link & bivalue_array[stack[j].link].data)) {candidate_link = bivalue_array[stack[j].link].data & ~c;}	
							else {stack_index = j - 1; goto next_link;}
						}	
					}					
//			look for target cells		
					if(candidate_link == target_number) {
						c = bivalue_array[stack[0].link].cell_index; int max_j = manu_space_type == RC_SPACE ? 20 : 16;
						for(j = 0; j < max_j; j++) {		
							int buddy = p_buddies_array->buddies_array[c][j]; int row = buddy/9; int col = buddy%9;
							if(space_table[row][col].type == IS_CANDIDATE && space_table[row][col].data & target_number) {
								if(p_share_unit_array->share_unit_array[bivalue_array[stack[xy_chain_length - 1].link].cell_index][buddy]) {							
//			the target cell must not be in the chain
									for(l = 0; l < xy_chain_length; l++) {if(buddy == bivalue_array[stack[l].link].cell_index) break;}
//			an elimination has been found
									if(l == xy_chain_length) {	
										target_number &= space_table[row][col].data;
										if(BITCOUNT(target_number) == 2) {
											int t = target_number; t1 = bit_rank[t & -t]; t2 = bit_rank[t & (t - 1)]; 
										}
									if(verbose) {do_verbose_xy(bivalue_array,stack,target_number,row,col); sat_stack_push_string(chain_string);}
									if(flag_hint) {
										do_verbose_xy(bivalue_array,stack,target_number,row,col);
											for(l = 0; l < xy_chain_length; l++) {xy_chain_array[l] = bivalue_array[stack[l].link];} 
											hint_info.type = XY_CHAIN;
											if(BITCOUNT(target_number) == 2) {
												sprintf(buffer, "hxy%d:(%d,%d) -> (%d,%d): <> %d,%d",xy_chain_length,
													(xy_chain_array[0].cell_index/9)+1, (xy_chain_array[0].cell_index%9)+1,
													(xy_chain_array[xy_chain_length-1].cell_index/9)+1, (xy_chain_array[xy_chain_length-1].cell_index%9)+1,
													t1+1,t2+1);
											}
											else {			
											sprintf(buffer, "hxy%d:(%d,%d) -> (%d,%d): <> %d",xy_chain_length,
												(xy_chain_array[0].cell_index/9)+1, (xy_chain_array[0].cell_index%9)+1,
												(xy_chain_array[xy_chain_length-1].cell_index/9)+1, (xy_chain_array[xy_chain_length-1].cell_index%9)+1,
												bit_rank[target_number]+1);
											}
											message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
											flag_xychain = 1; restore_screen(); cursor.x = row; cursor.y = col;
											hint_info.unit[0] = row; hint_info.unit[1] = col;
											unit_type = ROW; display_candidates(); return 1;
										}
//											rcspace->rcspace
										if(manu_space_type == RC_SPACE) {candidates[EXCLUDED][row][col] |= target_number;}
//											rnspace->rcspace
										else if(manu_space_type == RN_SPACE) {	
											if(BITCOUNT(target_number) == 2) {candidates[EXCLUDED][row][t1] |= 1 << col; candidates[EXCLUDED][row][t2] |= 1 << col;}
											else {candidates[EXCLUDED][row][(int)bit_rank[target_number]] |= 1 << col;}
										}
//											cnspace->rcspace
										else if(manu_space_type == CN_SPACE) {
											if(BITCOUNT(target_number) == 2) {candidates[EXCLUDED][t1][row] |= 1 << col; candidates[EXCLUDED][t2][row] |= 1 << col;}
											else {candidates[EXCLUDED][(int)bit_rank[target_number]][row] |= 1 << col;}
										}
										++pattern[XY_CHAIN];
									}
								}
							}
						}
						if(pattern[XY_CHAIN]) {compute_candidates(); fill_space_table(manu_space_type);}
					}//search target
					--stack_index; goto next_link;
				}//check candidate link
				stack_index = 0; goto next_link;
			}//chain found
//			if there is a successor for the current link			
			else {
next_link:		if((next_link = adjacency_array[stack[stack_index].link][stack[stack_index].next]) != -1) {
					++stack[stack_index].next; 
//			check for loop
					for(l = 0; l < stack_index; l++) {if(stack[l].link == next_link) goto next_link;}
					stack[++stack_index].link = next_link; stack[stack_index].next = 0;
				}
				else {if(stack_index - 1 == -1) break; else {--stack_index; goto next_link;}}
			}
		}//while(1) 
	}//for each bivalue
	return pattern[XY_CHAIN];	
}

void highlight_xychain() {
	int i,row,col;
	if(filtered_candidate) filter_candidate(filtered_candidate);
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
	for(i = 0; i < xy_chain_length; i++) {
		row = xy_chain_array[i].cell_index/9; col = xy_chain_array[i].cell_index%9;
		hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
		hpg_fill_rect(coordinates[row][col].x-3,coordinates[row][col].y-2,coordinates[row][col].x+4,coordinates[row][col].y+5);
		hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
		hpg_draw_letter(sdk_digit[i+1], coordinates[row][col].x, coordinates[row][col].y);
	}
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); 
}
//
//
int naked_single() {
	int i,j;
	int value;
	pattern[NAKED] = 0;
	int row, col, val;

//						scan the grid
	for(i = 0; i < 9; i++) {
		for(j = 0; j < 9; j++) {
			if(sdk_cellinfo[i][j].type == IS_CANDIDATE && BITCOUNT(sdk_cellinfo[i][j].data) == 1) {
			
				value = bit_rank[sdk_cellinfo[i][j].data]; 	
				
//						rcspace->rcspace
				row = i; col = j; val = value;

				if(flag_hint) {
					hint_info.type = NAKED; hint_info.zone = ROW;
					hint_info.unit[ROW] = row; hint_info.unit[COLUMN] = col; 
					return 1;
				}

				sdk_cellinfo[row][col].type = IS_FOUND; sdk_cellinfo[row][col].data = val + 1;
				mask_unit[MASK_FOUND][row] |= (1 << val); 
				mask_unit[MASK_FOUND][col + 9] |= (1 << val);
				mask_unit[MASK_FOUND][BLOCK_NUMBER(row,col) + 18] |= (1 << val); 	
				candidates[EXCLUDED][row][col] = 0; ++pattern[NAKED];
			}
		}	
		compute_candidates();
	}
	return pattern[NAKED];
}

int hidden_single() {
	int i,j,value;
	int row,col,val;
	pattern[HIDDEN] = 0;
	
	fill_space_table(RN_SPACE);
//						scan the rn grid
	for(i = 0; i < 9; i++) {
		for(j = 0; j < 9; j++) {		
			if(space_table[i][j].type == IS_CANDIDATE && BITCOUNT(space_table[i][j].data) == 1) {
//						get the column
				value = bit_rank[space_table[i][j].data]; 	
				
//						rnspace->rcspace
				row = i; col = value; val = j;
				
				if(flag_hint) {
					hint_info.type = HIDDEN; hint_info.zone = ROW;
					hint_info.unit[ROW] = row; hint_info.unit[COLUMN] = col; 
					return 1;
				}
				sdk_cellinfo[row][col].type = IS_FOUND; sdk_cellinfo[row][col].data = val + 1;
				mask_unit[MASK_FOUND][row] |= (1 << val); 
				mask_unit[MASK_FOUND][col + 9] |= (1 << val);
				mask_unit[MASK_FOUND][BLOCK_NUMBER(row,col) + 18] |= (1 << val); 	
				candidates[EXCLUDED][row][col] = 0; ++pattern[HIDDEN];
			}
		}	
		compute_candidates();
	}

	fill_space_table(CN_SPACE);
//						scan the cn grid
	for(i = 0; i < 9; i++) {
		for(j = 0; j < 9; j++) {		
			if(space_table[i][j].type == IS_CANDIDATE && BITCOUNT(space_table[i][j].data) == 1) {
//						get the row
				value = bit_rank[space_table[i][j].data]; 	
				
//						cnspace->rcspace
				row = value; col = i; val = j;

				if(flag_hint) {
					hint_info.type = HIDDEN; hint_info.zone = COLUMN;
					hint_info.unit[COLUMN] = col; hint_info.unit[ROW] = row; 
					return 1;
				}
				
				sdk_cellinfo[row][col].type = IS_FOUND; sdk_cellinfo[row][col].data = val + 1;
				mask_unit[MASK_FOUND][row] |= (1 << val); 
				mask_unit[MASK_FOUND][col + 9] |= (1 << val);
				mask_unit[MASK_FOUND][BLOCK_NUMBER(row,col) + 18] |= (1 << val); 	
				candidates[EXCLUDED][row][col] = 0; ++pattern[HIDDEN];
			}
		}	
		compute_candidates();
	}
	
	fill_space_table(BN_SPACE);
//						scan the bn grid
	for(i = 0; i < 9; i++) {
		for(j = 0; j < 9; j++) {		
			if(space_table[i][j].type == IS_CANDIDATE && BITCOUNT(space_table[i][j].data) == 1) {
//						get the square
				value = bit_rank[space_table[i][j].data]; 	
				
//						bnspace->rcspace
				row = block_start[i][0] + (value / 3); col = block_start[i][1] + (value % 3);  val = j;
				
				if(flag_hint) {
					hint_info.type = HIDDEN; hint_info.zone = BLOCK; hint_info.unit[BLOCK] = i; 
					hint_info.unit[ROW] = row; hint_info.unit[COLUMN] = col; 
					return 1;
				}
				
				sdk_cellinfo[row][col].type = IS_FOUND; sdk_cellinfo[row][col].data = val + 1;
				mask_unit[MASK_FOUND][row] |= (1 << val); 
				mask_unit[MASK_FOUND][col + 9] |= (1 << val);
				mask_unit[MASK_FOUND][BLOCK_NUMBER(row,col) + 18] |= (1 << val); 	
				candidates[EXCLUDED][row][col] = 0; ++pattern[HIDDEN];
			}
		}	
		compute_candidates();
	}

	fill_space_table(manu_space_type);
	return	pattern[HIDDEN];
}


int pointing() {
	int i,j,block,index;
	int candidate;
	pattern[POINTING] = 0;
//								first scan the block for potentiel locked candidates	
	for(block = 0; block < 9; block++) {

//								intialise data	
		for(i = 0; i < 9; i++) {locked_candidate[i].is_valid = NEUTRAL; locked_candidate[i].number = 0;}
		
			for(i = block_start[block][0]; i < block_start[block][0] + 3; i++) {
				for(j = block_start[block][1]; j < block_start[block][1] + 3; j++) {
				
					if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {	

//								get bit mask of candidate
						candidate = sdk_cellinfo[i][j].data;

//								get lsb and find the bit rank
						do {index = bit_rank[candidate & -candidate];
						
							if(locked_candidate[index].is_valid != FALSE) {

//								there cant be more than 3 instances of a candidate to form a locked
								if(++locked_candidate[index].number < 4) {

//								collect info upon instances of candidate 
									switch(locked_candidate[index].number) {
										case 1 	:	locked_candidate[index].row = i;
													locked_candidate[index].column = j;
													locked_candidate[index].is_valid = TRUE;
													break;
													
										case 2 	:	if(locked_candidate[index].row == i) {
														locked_candidate[index].alignement = ROW;
														locked_candidate[index].rank2 = j; 
													}
													else if(locked_candidate[index].column == j) {
														locked_candidate[index].alignement = COLUMN;
														locked_candidate[index].rank2 = i;
													}	
													else locked_candidate[index].is_valid = FALSE;									
													break;
													
										case 3 	: 	if(locked_candidate[index].alignement == ROW && (i != locked_candidate[index].row))
														locked_candidate[index].is_valid = FALSE;
													else if(locked_candidate[index].alignement == COLUMN && (j != locked_candidate[index].column))
														locked_candidate[index].is_valid = FALSE;
													break;
									}
								}
								else {
									locked_candidate[index].is_valid = FALSE;
								}		
							}
							
//							erase lsb from bit mask
						} while((candidate &= candidate - 1));
					} 
				}
			}
			
//							then scan for locked candidates in the potentiel ones 	
		for(index = 0; index < 9; index++) {
		
			if(locked_candidate[index].is_valid == TRUE) {			
			
				if(locked_candidate[index].alignement == ROW) {
				
					i = locked_candidate[index].row;
					for(j = 0; j < 9; j++) {
					
//							pass the block under examination					
						if(j == block_start[block][1]) j += 3; if(j == 9) break;
						
						if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {
						
//							if the candidate exists in the cell add it to the excluded list and flag it
							if(sdk_cellinfo[i][j].data & (1 << index)) {
							
								if(flag_hint) {
									hint_info.type = POINTING; hint_info.candidate = index;
									hint_info.zone = BLOCK; hint_info.unit[BLOCK] = block;
									return 1;
								}	
	
								candidates[EXCLUDED][i][j] |= (1 << index);
								pattern[POINTING]++;
							}
						}	
					}
				}
				else {
				
					j = locked_candidate[index].column;
					for(i = 0; i < 9; i++) {
					
//							pass the block under examination					
						if(i == block_start[block][0]) i += 3; if(i == 9) break;
						
						if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {
						
//							if the candidate exists in the cell add it to the excluded list and flag it
							if(sdk_cellinfo[i][j].data & (1 << index)) {
								
								if(flag_hint) {
									hint_info.type = POINTING; hint_info.candidate = index;
									hint_info.zone = BLOCK; hint_info.unit[BLOCK] = block;
									return 1;
								}
								
								candidates[EXCLUDED][i][j] |= (1 << index); 
								pattern[POINTING]++;
							}
						}	
					}
				}	
			}
			compute_candidates();
		}					
	}// next block	
	return pattern[POINTING];
}


int claiming() {
	int i,j,k,index;
	int candidate;
	pattern[CLAIMING] = 0;
	
//								first scan the rows for potentiel locked candidates
	for(i = 0; i < 9; i++) {
	
//								intialise data	
		for(j = 0; j < 9; j++) {locked_candidate[j].is_valid = NEUTRAL; locked_candidate[j].number = 0;}
		
		for(j = 0; j < 9; j++) {
		
			if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {
			
//								get bit mask of candidate
				candidate = sdk_cellinfo[i][j].data;
				
//								get lsb and find the bit rank
				do {index = bit_rank[candidate & -candidate];
				
							if(locked_candidate[index].is_valid != FALSE) {
							
//								there cant be more than 3 instances of a candidate to form a locked
								if(++locked_candidate[index].number < 4) {
								
//								collect info upon instances of candidate 
									switch(locked_candidate[index].number) {
										case 1 	:	locked_candidate[index].block = BLOCK_NUMBER(i,j); locked_candidate[index].is_valid = TRUE;
													locked_candidate[index].row = i; locked_candidate[index].column = j; 
													locked_candidate[index].alignement = ROW; 
													break;
										case 2 	:	locked_candidate[index].rank2 = j; 
										case 3	:	if(locked_candidate[index].block != BLOCK_NUMBER(i,j)) locked_candidate[index].is_valid = FALSE;
													break;
									}
								}
								else {
									locked_candidate[index].is_valid = FALSE;
								}		
							}
							
//							erase lsb from bit mask
				} while((candidate &= candidate - 1));
			}
		}	
		
//							 then scan for locked candidates in the potentiel ones 	
		for(index = 0; index < 9; index++) {
		
			if(locked_candidate[index].is_valid == TRUE) {
			
//							scan the block			
				for(k = block_start[locked_candidate[index].block][0]; k < block_start[locked_candidate[index].block][0] + 3; k++) {
					for(j = block_start[locked_candidate[index].block][1]; j < block_start[locked_candidate[index].block][1] + 3; j++) {
					
//							pass the row under examination
						if(k != i) {
						
							if(sdk_cellinfo[k][j].type == IS_CANDIDATE) {	
							
//							if the candidate exists in the cell add it to the excluded list and flag it
								if(sdk_cellinfo[k][j].data & (1 << index)) {
								
									if(flag_hint) {
										hint_info.type = CLAIMING; hint_info.candidate = index; hint_info.zone = BLOCK;
										hint_info.unit[BLOCK] = locked_candidate[index].block; return 1;
									}
		
									candidates[EXCLUDED][k][j] |= (1 << index); 
									pattern[CLAIMING]++;
								}
							}
						}
					}
				}
				compute_candidates();	
			}
		}
	}
	
//								first scan the columns for potentiel locked candidates
	for(j = 0; j < 9; j++) {
	
//								intialise data	
		for(i = 0; i < 9; i++) {locked_candidate[i].is_valid = NEUTRAL; locked_candidate[i].number = 0;}
		
		for(i = 0; i < 9; i++) {
		
			if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {
			
//								get bit mask of candidate
				candidate = sdk_cellinfo[i][j].data;
				
//								get lsb and find the bit rank
				do {index = bit_rank[candidate & -candidate];
				
							if(locked_candidate[index].is_valid != FALSE) {
							
//								there cant be more than 3 instances of a candidate to form a locked
								if(++locked_candidate[index].number < 4) {
								
//								collect info upon instances of candidate 
									switch(locked_candidate[index].number) {
										case 1 	:	locked_candidate[index].block = BLOCK_NUMBER(i,j); locked_candidate[index].is_valid = TRUE;
													locked_candidate[index].row = i; locked_candidate[index].column = j; 
													locked_candidate[index].alignement = COLUMN; break;
										case 2 	:	locked_candidate[index].rank2 = i; 
										case 3	:	if(locked_candidate[index].block != BLOCK_NUMBER(i,j)) locked_candidate[index].is_valid = FALSE;
													break;
									}
								}
								else {
									locked_candidate[index].is_valid = FALSE;
								}		
							}
							
//							erase lsb from bit mask
				} while((candidate &= candidate - 1));
			}
		}
		
//							 then scan for locked candidates in the potentiel ones 	
		for(index = 0; index < 9; index++) {
		
			if(locked_candidate[index].is_valid == TRUE) {
			
//							scan the block
				for(i = block_start[locked_candidate[index].block][0]; i < block_start[locked_candidate[index].block][0] + 3; i++) {
					for(k = block_start[locked_candidate[index].block][1]; k < block_start[locked_candidate[index].block][1] + 3; k++) {
					
//							pass the column under examination
						if(k != j) {
						
							if(sdk_cellinfo[i][k].type == IS_CANDIDATE) {	
							
//							if the candidate exists in the cell add it to the excluded list and flag it
								if(sdk_cellinfo[i][k].data & (1 << index)) {
								
									if(flag_hint) {
										hint_info.type = CLAIMING; hint_info.candidate = index; hint_info.zone = BLOCK;
										hint_info.unit[BLOCK] = locked_candidate[index].block; return 1;
									}		
									
									candidates[EXCLUDED][i][k] |= (1 << index); 
									pattern[CLAIMING]++;
								}
							}
						}
					}
				}
				compute_candidates();	
			}
		}
	}
	return pattern[CLAIMING];
}

int naked_subset(int unit_type) {
	int i,j,k,index,size;
	int row,col;
	int bitcount;
	int c1,c2,c3,c4;
	int size_cells[4];
	int to_exclude;
	int  tuple;
	NAKED_SUBSET naked_subset_info[9];
	pattern[NAKED_TUPLE] = 0;
	
	row = col = 0;

	fill_space_table(RC_SPACE);
	
//						scan the grid
	for(i = 0; i < 9; i++) {
//						initialise data
		memset((char*) naked_subset_info, 0, sizeof(naked_subset_info));
		memset((char*) size_cells, 0, sizeof(size_cells));
		index = 0;
		
		for(j = 0; j < 9; j++) {

			if(unit_type == ROW) 			{row = row_coordinates[i][j].x; col = row_coordinates[i][j].y;}
			else if(unit_type == COLUMN) 	{row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;}
			else if(unit_type == BLOCK) 	{row = box_coordinates[i][j].x; col = box_coordinates[i][j].y;}
			
//						look for candidate cells with at most 4 candidates
			if(space_table[row][col].type == IS_CANDIDATE) {
//						number of candidate cells in the row
				++size_cells[3];
				if((bitcount = BITCOUNT(space_table[row][col].data)) < 5  && bitcount > 1) {
//						number of cells with 2 3 4 candidates 
					++size_cells[bitcount-2];
//						cell address and size
					naked_subset_info[index].p_cell = &space_table[row][col];
					naked_subset_info[index++].size = bitcount;
				}
			}
		}
		
//						sort the cells according to their size
		qsort((void *) naked_subset_info, index, sizeof(NAKED_SUBSET), compare_nakedsubset);	

//-------------------------------------------------PAIRS---------------------------------------------------
		
//						look for naked pairs: at least two pairs and three candidate cells
		if( size_cells[0] > 1 && size_cells[3] > 2) {
		
//						form the combinaisons: C(size_cells[0],PAIR)
			for(c1 = 0; c1 < size_cells[0] - 1; c1++) {
				for(c2 = c1 + 1; c2 < size_cells[0]; c2++) {
//						if the two cells form a pair
					if( BITCOUNT((	naked_subset_info[c1].p_cell->data |
									naked_subset_info[c2].p_cell->data)) == IS_PAIR) {
//						collect info: cell c1 forms a pair with cell c2
						naked_subset_info[c1].is_tuple = TRUE;
						naked_subset_info[c1].peers[0] = c2;
					}
				}
			}
			
//						find the pairs
			for(k = 0; k < index; k++) {
				if(naked_subset_info[k].is_tuple == TRUE) {
//						look for cells in the row that have candidates to exclude
					for(j = 0; j < 9; j++) {
						
						if(unit_type == ROW ) 			{row = row_coordinates[i][j].x; col = row_coordinates[i][j].y;}
						else if(unit_type == COLUMN ) 	{row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;}
						else if(unit_type == BLOCK ) 	{row = box_coordinates[i][j].x; col = box_coordinates[i][j].y;}
					
//						pass cells c1 and c2
						if(space_table[row][col].type == IS_CANDIDATE && 	&space_table[row][col] != naked_subset_info[k].p_cell && 
																		&space_table[row][col] != naked_subset_info[naked_subset_info[k].peers[0]].p_cell) {
//						if there are candidates to exclude in this cell do it

//						needed for hint method
							tuple = naked_subset_info[k].p_cell->data | naked_subset_info[naked_subset_info[k].peers[0]].p_cell->data;
							
							if((to_exclude = space_table[row][col].data & tuple)) {
							
								if(flag_hint) {
									show_tuple.tuple = tuple;
									hint_info.type = NAKED_TUPLE; hint_info.tuple = IS_PAIR;
									
									if(unit_type == ROW) {
										hint_info.zone = ROW; hint_info.unit[ROW] = row;
										hint_info.unit[COLUMN] = (naked_subset_info[k].p_cell - &space_table[0][0]) % 9; return 1;
									}
									else if(unit_type == COLUMN) {
										hint_info.zone = COLUMN; hint_info.unit[COLUMN] = col;
										hint_info.unit[ROW] = (naked_subset_info[k].p_cell - &space_table[0][0]) / 9; return 1;
									}
									else if(unit_type == BLOCK) {
										hint_info.zone = BLOCK; hint_info.unit[BLOCK] = BLOCK_NUMBER(row,col);
										hint_info.unit[ROW] 	= (naked_subset_info[k].p_cell - &space_table[0][0]) / 9;
										hint_info.unit[COLUMN] 	= (naked_subset_info[k].p_cell - &space_table[0][0]) % 9; return 1;
									}
								}
								
								candidates[EXCLUDED][row][col] |= to_exclude;
								pattern[NAKED_TUPLE]++;
							}
						}
					}
				}
			}

		}//look for pairs	
		

//-------------------------------------------------TRIPLETS---------------------------------------------------
		
//						look for naked triplets
			if( (size_cells[0] + size_cells[1]) > 2 && size_cells[3] > 3) {
//						initialise flag			
				for(k = 0; k < index; naked_subset_info[k++].is_tuple = FALSE); 
//						form the combinaisons: C(size_cells[0]+size_cells[1],TRIPLET)
				size = size_cells[0] + size_cells[1];
				for(c1 = 0; c1 < size - 2; c1++) {
					for(c2 = c1 + 1; c2 < size - 1; c2++) {
						for(c3 = c2 + 1; c3 < size; c3++) {
//						if the three cells form a triplet
							if(BITCOUNT((	naked_subset_info[c1].p_cell->data |
											naked_subset_info[c2].p_cell->data |
											naked_subset_info[c3].p_cell->data)) == IS_TRIPLET ) {
//						collect info: cell c1 forms a triplet with cell c2 and cell C3
								naked_subset_info[c1].is_tuple = TRUE;
								naked_subset_info[c1].peers[0] = c2;
								naked_subset_info[c1].peers[1] = c3;
							}
						}
					}
				}			
//						find the triplets
				for(k = 0; k < index; k++) {
					if(naked_subset_info[k].is_tuple == TRUE) {
//						look for cells in the row that have candidates to exclude
						for(j = 0; j < 9; j++) {
						
							if(unit_type == ROW) 			{row = row_coordinates[i][j].x; col = row_coordinates[i][j].y;}
							else if(unit_type == COLUMN) 	{row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;}
							else if(unit_type == BLOCK) 	{row = box_coordinates[i][j].x; col = box_coordinates[i][j].y;}

//						pass cells c1 c2 and c3
							if(space_table[row][col].type == 	IS_CANDIDATE &&  
															&space_table[row][col] != naked_subset_info[k].p_cell && 
															&space_table[row][col] != naked_subset_info[naked_subset_info[k].peers[0]].p_cell && 
															&space_table[row][col] != naked_subset_info[naked_subset_info[k].peers[1]].p_cell) {
//						if there are candidates to exclude in this cell do it
//						needed for hint method
							tuple = naked_subset_info[k].p_cell->data |
												naked_subset_info[naked_subset_info[k].peers[0]].p_cell->data |
												naked_subset_info[naked_subset_info[k].peers[1]].p_cell->data;				
	
								if((to_exclude = space_table[row][col].data & tuple)) {
								
									if(flag_hint) {
										show_tuple.tuple = tuple;
										hint_info.type = NAKED_TUPLE; hint_info.tuple = IS_TRIPLET; 
									
										if(unit_type == ROW) {
											hint_info.zone = ROW; hint_info.unit[ROW] = row;
											hint_info.unit[COLUMN] = (naked_subset_info[k].p_cell - &space_table[0][0]) % 9; return 1;
										}
										else if(unit_type == COLUMN) {
											hint_info.zone = COLUMN; hint_info.unit[COLUMN] = col;
											hint_info.unit[ROW] = (naked_subset_info[k].p_cell - &space_table[0][0]) / 9; return 1;
										}
										else if(unit_type == BLOCK) {
											hint_info.zone = BLOCK; hint_info.unit[BLOCK] = BLOCK_NUMBER(row,col);
											hint_info.unit[ROW] 	= (naked_subset_info[k].p_cell - &space_table[0][0]) / 9;
											hint_info.unit[COLUMN] 	= (naked_subset_info[k].p_cell - &space_table[0][0]) % 9; return 1;
										}
									}
									candidates[EXCLUDED][row][col] |= to_exclude;
									pattern[NAKED_TUPLE]++;
								}
							}
						}
					}
				}
			}//look for triplets
			
//-------------------------------------------------QUADS-------------------------------------------------
			
//						look for naked quads
			if( (size_cells[0] + size_cells[1] + size_cells[2]) > 3 && size_cells[3] > 4) {
//						initialise flag			
				for(k = 0; k < index; naked_subset_info[k++].is_tuple = FALSE); 
//						form the combinaisons: C(size_cells[0]+size_cells[1]+size_cells[2],QUAD)
				size = size_cells[0] + size_cells[1] + size_cells[2];
				for(c1 = 0; c1 < size - 3; c1++) {
					for(c2 = c1 + 1; c2 < size - 2; c2++) {
						for(c3 = c2 + 1; c3 < size - 1; c3++) {
							for(c4 = c3 + 1; c4 < size; c4++) {
//						if the four cells form a quad
								if(BITCOUNT((	naked_subset_info[c1].p_cell->data |
												naked_subset_info[c2].p_cell->data |
												naked_subset_info[c3].p_cell->data |
												naked_subset_info[c4].p_cell->data)) == IS_QUAD ) {
//						collect info: cell c1 forms a quad with cell c2 cell C3 and cell c4
									naked_subset_info[c1].is_tuple = TRUE;
									naked_subset_info[c1].peers[0] = c2;
									naked_subset_info[c1].peers[1] = c3;
									naked_subset_info[c1].peers[2] = c4;
								}
							}
						}
					}
				}			
//						find the quad
				for(k = 0; k < index; k++) {
					if(naked_subset_info[k].is_tuple == TRUE) {
//						look for cells in the row that have candidates to exclude
						for(j = 0; j < 9; j++) {

							if(unit_type == ROW) 			{row = row_coordinates[i][j].x; col = row_coordinates[i][j].y;}
							else if(unit_type == COLUMN) 	{row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;}
							else if(unit_type == BLOCK) 	{row = box_coordinates[i][j].x; col = box_coordinates[i][j].y;}

//						pass cells c1 c2 c3 and c4
							if(space_table[row][col].type == 	IS_CANDIDATE &&  
															&space_table[row][col] != naked_subset_info[k].p_cell && 
															&space_table[row][col] != naked_subset_info[naked_subset_info[k].peers[0]].p_cell && 
															&space_table[row][col] != naked_subset_info[naked_subset_info[k].peers[1]].p_cell && 
															&space_table[row][col] != naked_subset_info[naked_subset_info[k].peers[2]].p_cell) {
//						if there are candidates to exclude in this cell do it
//						needed for hint method
								tuple = naked_subset_info[k].p_cell->data |
													naked_subset_info[naked_subset_info[k].peers[0]].p_cell->data |
													naked_subset_info[naked_subset_info[k].peers[1]].p_cell->data |
													naked_subset_info[naked_subset_info[k].peers[2]].p_cell->data;		
		
								if((to_exclude = space_table[row][col].data & tuple)) {
								
									if(flag_hint) {	
										show_tuple.tuple = tuple;	
										hint_info.type = NAKED_TUPLE; hint_info.tuple = IS_QUAD; 
										
										if(unit_type == ROW) {
											hint_info.zone = ROW; hint_info.unit[ROW] = row;
											hint_info.unit[COLUMN] = (naked_subset_info[k].p_cell - &space_table[0][0]) % 9; return 1;
										}
										else if(unit_type == COLUMN) {
											hint_info.zone = COLUMN; hint_info.unit[COLUMN] = col;
											hint_info.unit[ROW] = (naked_subset_info[k].p_cell - &space_table[0][0]) / 9; return 1;
										}
										else if(unit_type == BLOCK) {
											hint_info.zone = BLOCK; hint_info.unit[BLOCK] = BLOCK_NUMBER(row,col);
											hint_info.unit[ROW] 	= (naked_subset_info[k].p_cell - &space_table[0][0]) / 9;
											hint_info.unit[COLUMN] 	= (naked_subset_info[k].p_cell - &space_table[0][0]) % 9; return 1;
										}
									}
									
									candidates[EXCLUDED][row][col] |= to_exclude;
									pattern[NAKED_TUPLE]++;
								}
							}
						}
					}		
				}
			}//look for quad
		
	}//scan grid
	compute_candidates();
	return pattern[NAKED_TUPLE];
}

int hidden_subset(int space_type) {
	int i,j,k,index,size;
	int row,col,dim;
	int bitcount;
	int c1,c2,c3,c4;
	int size_cells[4];
	int to_exclude;
	int  tuple;
	NAKED_SUBSET naked_subset_info[9];
	pattern[HIDDEN_TUPLE] = 0;
		
	row = col = 0;
		
	if(space_type == ROW) 			fill_space_table(RN_SPACE);
	else if(space_type == COLUMN) 	fill_space_table(CN_SPACE);
	else if(space_type == BLOCK) 	fill_space_table(BN_SPACE);
	
//						scan the grid
	for(i = 0; i < 9; i++) {
//						initialise data
		memset((char*) naked_subset_info, 0, sizeof(naked_subset_info));
		memset((char*) size_cells, 0, sizeof(size_cells));
		index = 0;
		
		for(j = 0; j < 9; j++) {

			row = row_coordinates[i][j].x; col = row_coordinates[i][j].y;
			
//						look for candidate cells with at most 4 candidates
			if(space_table[row][col].type == IS_CANDIDATE) {
//						number of candidate cells in the row
				++size_cells[3];
				if((bitcount = BITCOUNT(space_table[row][col].data)) < 5  && bitcount > 1) {
//						number of cells with 2 3 4 candidates 
					++size_cells[bitcount-2];
//						cell address and size
					naked_subset_info[index].p_cell = &space_table[row][col];
					naked_subset_info[index++].size = bitcount;
				}
			}
		}
		
//						sort the cells according to their size
		qsort((void *) naked_subset_info, index, sizeof(NAKED_SUBSET), compare_nakedsubset);	

//-------------------------------------------------PAIRS---------------------------------------------------
		
		if( size_cells[0] > 1 && size_cells[3] > 2) {

//						form the combinaisons: C(size_cells[0],PAIR)
			for(c1 = 0; c1 < size_cells[0] - 1; c1++) {
				for(c2 = c1 + 1; c2 < size_cells[0]; c2++) {
//						if the two cells form a pair
					if( BITCOUNT((	naked_subset_info[c1].p_cell->data |
									naked_subset_info[c2].p_cell->data)) == IS_PAIR) {
//						collect info: cell c1 forms a pair with cell c2
						naked_subset_info[c1].is_tuple = TRUE;
						naked_subset_info[c1].peers[0] = c2;
					}
				}
			}
						
//						find the pairs
			for(k = 0; k < index; k++) {
				
				if(naked_subset_info[k].is_tuple == TRUE) {
					show_tuple.tuple = tuple = 0;

//						
					for(j = 0; j < 9; j++) {
						row = row_coordinates[i][j].x; col = row_coordinates[i][j].y;
					
						if(	&space_table[row][col] == naked_subset_info[k].p_cell ||
							&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[0]].p_cell) {
							
//						the columns are the values of the pair and the data are the others coordinates in the rc_space
							show_tuple.tuple |= (1 << col); tuple |= space_table[row][col].data; 
						}		
					}
					
					do {dim = bit_rank[tuple & -tuple];

//						in rn_space data is column			
						if(space_type == ROW) {
						
							if( (to_exclude = sdk_cellinfo[row][dim].data & ~show_tuple.tuple) ) {
							
								if(flag_hint) {
									hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_PAIR;
									hint_info.zone = ROW; hint_info.unit[ROW] = row;
									hint_info.unit[COLUMN] = dim; return 1;
								}
							
								candidates[EXCLUDED][row][dim] |= to_exclude;
								pattern[HIDDEN_TUPLE]++;
							}
						}
					
//						in cn_space data is row			
						else if(space_type == COLUMN) {
						
							if( (to_exclude = sdk_cellinfo[dim][row].data & ~show_tuple.tuple) ) {

								if(flag_hint) {
									hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_PAIR;
									hint_info.zone = COLUMN; hint_info.unit[ROW] = dim;
									hint_info.unit[COLUMN] = row; return 1;
								}
							
								candidates[EXCLUDED][dim][row] |= to_exclude;
								pattern[HIDDEN_TUPLE]++;
							}
						}
						
//						in bn_space row is block data is square			
						else if(space_type == BLOCK) {
						
							int bi = block_start[row][0] + (dim / 3);
							int bj = block_start[row][1] + (dim % 3);
								
							if( (to_exclude = sdk_cellinfo[bi][bj].data & ~show_tuple.tuple) ) {

								if(flag_hint) {
									hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_PAIR;
									hint_info.zone = BLOCK; hint_info.unit[BLOCK] = row;
									hint_info.unit[ROW] = bi; hint_info.unit[COLUMN] = bj; return 1;
								}
							
								candidates[EXCLUDED][bi][bj] |= to_exclude;
								pattern[HIDDEN_TUPLE]++;
							}
						}				
						
					} while((tuple &= tuple - 1));
				}
			}
		}//look for pairs	
		
//-------------------------------------------------TRIPLETS---------------------------------------------------
		
		if( (size_cells[0] + size_cells[1]) > 2 && size_cells[3] > 3) {
//						initialise flag			
			for(k = 0; k < index; naked_subset_info[k++].is_tuple = FALSE); 
//						form the combinaisons: C(size_cells[0]+size_cells[1],TRIPLET)
				size = size_cells[0] + size_cells[1];
				for(c1 = 0; c1 < size - 2; c1++) {
					for(c2 = c1 + 1; c2 < size - 1; c2++) {
						for(c3 = c2 + 1; c3 < size; c3++) {
//						if the three cells form a triplet
							if(BITCOUNT((	naked_subset_info[c1].p_cell->data | naked_subset_info[c2].p_cell->data |
											naked_subset_info[c3].p_cell->data)) == IS_TRIPLET ) {
//						collect info: cell c1 forms a triplet with cell c2 and cell C3
								naked_subset_info[c1].is_tuple = TRUE;
								naked_subset_info[c1].peers[0] = c2; naked_subset_info[c1].peers[1] = c3;
							}
						}
					}
				}			
				
//							find the triplets
				for(k = 0; k < index; k++) {
				
					if(naked_subset_info[k].is_tuple == TRUE) {
						show_tuple.tuple = tuple = 0;
					
						for(j = 0; j < 9; j++) {
							row = row_coordinates[i][j].x; col = row_coordinates[i][j].y;
					
							if(	&space_table[row][col] == naked_subset_info[k].p_cell ||
								&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[0]].p_cell ||
								&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[1]].p_cell) {
							
								show_tuple.tuple |= (1 << col); tuple |= space_table[row][col].data; 
							}		
						}
					
						do {dim = bit_rank[tuple & -tuple];

//							in rn_space data is column			
							if(space_type == ROW) {
						
								if( (to_exclude = sdk_cellinfo[row][dim].data & ~show_tuple.tuple) ) {
							
									if(flag_hint) {
										hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_TRIPLET;
										hint_info.zone = ROW; hint_info.unit[ROW] = row;
										hint_info.unit[COLUMN] = dim; return 1;
									}
							
									candidates[EXCLUDED][row][dim] |= to_exclude;
									pattern[HIDDEN_TUPLE]++;
								}
							}
					
//							in cn_space data is row			
							else if(space_type == COLUMN) {
						
								if( (to_exclude = sdk_cellinfo[dim][row].data & ~show_tuple.tuple) ) {
	
									if(flag_hint) {
										hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_TRIPLET;
										hint_info.zone = COLUMN; hint_info.unit[ROW] = dim;
										hint_info.unit[COLUMN] = row; return 1;
									}
							
									candidates[EXCLUDED][dim][row] |= to_exclude;
									pattern[HIDDEN_TUPLE]++;
								}
							}

//							in bn_space row is block data is square			
							else if(space_type == BLOCK) {
						
								int bi = block_start[row][0] + (dim / 3);
								int bj = block_start[row][1] + (dim % 3);
								
								if( (to_exclude = sdk_cellinfo[bi][bj].data & ~show_tuple.tuple) ) {

									if(flag_hint) {
										hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_TRIPLET;
										hint_info.zone = BLOCK; hint_info.unit[BLOCK] = row;
										hint_info.unit[ROW] = bi; hint_info.unit[COLUMN] = bj; return 1;
									}
							
									candidates[EXCLUDED][bi][bj] |= to_exclude;
									pattern[HIDDEN_TUPLE]++;
								}
							}				
						
						} while((tuple &= tuple - 1));
					
					}
				}
			}//look for triplets
			
//-------------------------------------------------QUADS-------------------------------------------------
			
			if( (size_cells[0] + size_cells[1] + size_cells[2]) > 3 && size_cells[3] > 4) {
//						initialise flag			
				for(k = 0; k < index; naked_subset_info[k++].is_tuple = FALSE); 
//						form the combinaisons: C(size_cells[0]+size_cells[1]+size_cells[2],QUAD)
				size = size_cells[0] + size_cells[1] + size_cells[2];
				for(c1 = 0; c1 < size - 3; c1++) {
					for(c2 = c1 + 1; c2 < size - 2; c2++) {
						for(c3 = c2 + 1; c3 < size - 1; c3++) {
							for(c4 = c3 + 1; c4 < size; c4++) {
//						if the four cells form a quad
								if(BITCOUNT((	naked_subset_info[c1].p_cell->data | naked_subset_info[c2].p_cell->data |
												naked_subset_info[c3].p_cell->data | naked_subset_info[c4].p_cell->data)) == IS_QUAD ) {
//						collect info: cell c1 forms a quad with cell c2 cell C3 and cell c4
									naked_subset_info[c1].is_tuple = TRUE; naked_subset_info[c1].peers[0] = c2; 
									naked_subset_info[c1].peers[1] = c3; naked_subset_info[c1].peers[2] = c4;
								}
							}
						}
					}
				}			

//							find the quads
				for(k = 0; k < index; k++) {
				
					if(naked_subset_info[k].is_tuple == TRUE) {
						show_tuple.tuple = tuple = 0;
					
						for(j = 0; j < 9; j++) {
							row = row_coordinates[i][j].x; col = row_coordinates[i][j].y;
					
							if(	&space_table[row][col] == naked_subset_info[k].p_cell ||
								&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[0]].p_cell ||
								&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[1]].p_cell ||
								&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[2]].p_cell) {
							
								show_tuple.tuple |= (1 << col); tuple |= space_table[row][col].data; 
							}		
						}
					
						do {dim = bit_rank[tuple & -tuple];

//							in rn_space data is column			
							if(space_type == ROW) {
						
								if( (to_exclude = sdk_cellinfo[row][dim].data & ~show_tuple.tuple) ) {
							
									if(flag_hint) {
										hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_QUAD;
										hint_info.zone = ROW; hint_info.unit[ROW] = row;
										hint_info.unit[COLUMN] = dim; return 1;
									}
									
									candidates[EXCLUDED][row][dim] |= to_exclude;
									pattern[HIDDEN_TUPLE]++;
								}
							}
					
//							in cn_space data is row			
							else if(space_type == COLUMN) {
							
								if( (to_exclude = sdk_cellinfo[dim][row].data & ~show_tuple.tuple) ) {
								
									if(flag_hint) {
										hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_QUAD;
									hint_info.zone = COLUMN; hint_info.unit[ROW] = dim;
									hint_info.unit[COLUMN] = row; return 1;
									}
									
									candidates[EXCLUDED][dim][row] |= to_exclude;
									pattern[HIDDEN_TUPLE]++;
								}
							}

//							in bn_space row is block data is square			
							else if(space_type == BLOCK) {
						
								int bi = block_start[row][0] + (dim / 3);
								int bj = block_start[row][1] + (dim % 3);
								
								if( (to_exclude = sdk_cellinfo[bi][bj].data & ~show_tuple.tuple) ) {

									if(flag_hint) {
										hint_info.type = HIDDEN_TUPLE; hint_info.tuple = IS_QUAD;
										hint_info.zone = BLOCK; hint_info.unit[BLOCK] = row;
										hint_info.unit[ROW] = bi; hint_info.unit[COLUMN] = bj; return 1;
									}
							
									candidates[EXCLUDED][bi][bj] |= to_exclude;
									pattern[HIDDEN_TUPLE]++;
								}
							}				
							
						} while((tuple &= tuple - 1));
					
					}
				}
			}//look for quad
		
	}//scan grid
	compute_candidates();
	return pattern[HIDDEN_TUPLE];
}

int fish(space_type) {
	int i,j,k,index,size;
	int row,col,dim,val;
	int bitcount;
	int c1,c2,c3,c4;
	int size_cells[4];
	int  tuple;
	NAKED_SUBSET naked_subset_info[9];
	pattern[FISH_ALL] = 0;
	
	int fish_info_index;

	row = col = val = 0;
	
	if(space_type == ROW) 			fill_space_table(RN_SPACE);
	else if(space_type == COLUMN) 	fill_space_table(CN_SPACE);
	
//						scan the grid
	for(i = 0; i < 9; i++) {
//						initialise data
		memset((char*) naked_subset_info, 0, sizeof(naked_subset_info));
		memset((char*) size_cells, 0, sizeof(size_cells));
		index = 0;
		
		for(j = 0; j < 9; j++) {

			row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;
			
//						look for candidate cells with at most 4 candidates
			if(space_table[row][col].type == IS_CANDIDATE) {
//						number of candidate cells in the row
				++size_cells[3];
				if((bitcount = BITCOUNT(space_table[row][col].data)) < 5  && bitcount > 1) {
//						number of cells with 2 3 4 candidates 
					++size_cells[bitcount-2];
//						cell address and size
					naked_subset_info[index].p_cell = &space_table[row][col];
					naked_subset_info[index++].size = bitcount;
				}
			}
		}
		
//						sort the cells according to their size
		qsort((void *) naked_subset_info, index, sizeof(NAKED_SUBSET), compare_nakedsubset);	

//-------------------------------------------------PAIRS---------------------------------------------------
		
		if( size_cells[0] > 1 && size_cells[3] > 2) {
		
//						form the combinaisons: C(size_cells[0],PAIR)
			for(c1 = 0; c1 < size_cells[0] - 1; c1++) {
				for(c2 = c1 + 1; c2 < size_cells[0]; c2++) {
//						if the two cells form a pair
					if( BITCOUNT((	naked_subset_info[c1].p_cell->data |
									naked_subset_info[c2].p_cell->data)) == IS_PAIR) {
//						collect info: cell c1 forms a pair with cell c2
						naked_subset_info[c1].is_tuple = TRUE;
						naked_subset_info[c1].peers[0] = c2;
					}
				}
			}
			
//						find the pairs
			for(k = 0; k < index; k++) {
				
				if(naked_subset_info[k].is_tuple == TRUE) {
					show_tuple.tuple = tuple = fish_info_index = 0;
					fish_all_info.type = X_WING; fish_all_info.space_type = space_type; 
					
					for(j = 0; j < 9; j++) {
						row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;

//						rn_space = x_wing in row: col = val  row = row  data = col 
//						cn_space = x_wing in col: col = val  row = col  data = row 

						if(	&space_table[row][col] == naked_subset_info[k].p_cell ||
							&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[0]].p_cell) {
							
							show_tuple.tuple |= (1 << row); tuple |= space_table[row][col].data; val = (1 << col);
							
							fish_all_info.dim[fish_info_index][0] = row;
							fish_all_info.dim[fish_info_index++][1] = space_table[row][col].data;
							fish_all_info.candidate = col;
						}		
					}

					c1 = bit_rank[show_tuple.tuple & -show_tuple.tuple]; c2 = bit_rank[show_tuple.tuple & (show_tuple.tuple - 1)]; 

//						in rc_space: look in the col=data for val=col passing row=row
					if(space_type == ROW) {
					
						do {dim = bit_rank[tuple & -tuple];

							for(j = 0; j < 9; j++) {
							
								if(sdk_cellinfo[j][dim].type == IS_CANDIDATE && j != c1 && j != c2 ) {
								
									if(sdk_cellinfo[j][dim].data & val) {	

										if(flag_hint) {
											hint_info.type = X_WING; hint_info.tuple = IS_PAIR; hint_info.candidate = fish_all_info.candidate;
											hint_info.zone = ROW; hint_info.unit[ROW] = c1;
											hint_info.unit[COLUMN] = dim; return 1;
										}

										candidates[EXCLUDED][j][dim] |= val;
										pattern[FISH_ALL]++;

									}
								}
							}
						} while((tuple &= tuple - 1));
					}
					
//						in rc_space: look in the row=data for val=col passing col=row				
					else if(space_type == COLUMN) {
					
						do {dim = bit_rank[tuple & -tuple];

							for(j = 0; j < 9; j++) {
							
								if(sdk_cellinfo[dim][j].type == IS_CANDIDATE && j != c1 && j != c2 ) {
								
									if(sdk_cellinfo[dim][j].data & val) {		

										if(flag_hint) {
											hint_info.type = X_WING; hint_info.tuple = IS_PAIR; hint_info.candidate = fish_all_info.candidate;
											hint_info.zone = COLUMN; hint_info.unit[COLUMN] = c1;
											hint_info.unit[ROW] = dim; return 1;
										}

										candidates[EXCLUDED][dim][j] |= val;
										pattern[FISH_ALL]++;

									}
								}
							}
						} while((tuple &= tuple - 1));
					}
				}
			}
		}//look for pairs	


//-------------------------------------------------TRIPLETS---------------------------------------------------
		
		if( (size_cells[0] + size_cells[1]) > 2 && size_cells[3] > 3) {
		
//						initialise flag			
			for(k = 0; k < index; naked_subset_info[k++].is_tuple = FALSE); 
			
//						form the combinaisons: C(size_cells[0]+size_cells[1],TRIPLET)
			size = size_cells[0] + size_cells[1];
			for(c1 = 0; c1 < size - 2; c1++) {
				for(c2 = c1 + 1; c2 < size - 1; c2++) {
					for(c3 = c2 + 1; c3 < size; c3++) {
//						if the three cells form a triplet
						if(BITCOUNT((	naked_subset_info[c1].p_cell->data | naked_subset_info[c2].p_cell->data |
										naked_subset_info[c3].p_cell->data)) == IS_TRIPLET ) {
//						collect info: cell c1 forms a triplet with cell c2 and cell C3
							naked_subset_info[c1].is_tuple = TRUE;
							naked_subset_info[c1].peers[0] = c2; naked_subset_info[c1].peers[1] = c3;
						}
					}
				}
			}			
				
//							find the triplets
			for(k = 0; k < index; k++) {
				
				if(naked_subset_info[k].is_tuple == TRUE) {
					show_tuple.tuple = tuple = fish_info_index = 0;
					fish_all_info.type = SWORDFISH; fish_all_info.space_type = space_type;
					
					for(j = 0; j < 9; j++) {
						row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;
					
						if(	&space_table[row][col] == naked_subset_info[k].p_cell ||
							&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[0]].p_cell ||
							&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[1]].p_cell) {
							
							show_tuple.tuple |= (1 << row); tuple |= space_table[row][col].data; val = (1 << col);
							
							fish_all_info.dim[fish_info_index][0] = row;
							fish_all_info.dim[fish_info_index++][1] = space_table[row][col].data;
							fish_all_info.candidate = col;

						}		
					}
					
					c1 = bit_rank[show_tuple.tuple & -show_tuple.tuple]; show_tuple.tuple &= show_tuple.tuple - 1;
					c2 = bit_rank[show_tuple.tuple & -show_tuple.tuple]; c3 = bit_rank[show_tuple.tuple & (show_tuple.tuple - 1)]; 

//						in rc_space: look in the col=data for val=col passing row=row
					if(space_type == ROW) {
					
						do {dim = bit_rank[tuple & -tuple];
	
							for(j = 0; j < 9; j++) {
							
								if(sdk_cellinfo[j][dim].type == IS_CANDIDATE && j != c1 && j != c2 && j != c3) {
								
									if(sdk_cellinfo[j][dim].data & val) {			

										if(flag_hint) {
											hint_info.type = SWORDFISH; hint_info.tuple = IS_TRIPLET; hint_info.candidate = fish_all_info.candidate;
											hint_info.zone = ROW; hint_info.unit[ROW] = c1;
											hint_info.unit[COLUMN] = dim; return 1;
										}

										candidates[EXCLUDED][j][dim] |= val;
										pattern[FISH_ALL]++;

									}
								}
							}
						} while((tuple &= tuple - 1));
					}
					
//						in rc_space: look in the row=data for val=col passing col=row				
					else if(space_type == COLUMN) {
					
						do {dim = bit_rank[tuple & -tuple];
	
							for(j = 0; j < 9; j++) {
							
								if(sdk_cellinfo[dim][j].type == IS_CANDIDATE && j != c1 && j != c2 && j != c3) {
								
									if(sdk_cellinfo[dim][j].data & val) {			

										if(flag_hint) {
											hint_info.type = SWORDFISH; hint_info.tuple = IS_TRIPLET; hint_info.candidate = fish_all_info.candidate;
											hint_info.zone = COLUMN; hint_info.unit[COLUMN] = c1;
											hint_info.unit[ROW] = dim; return 1;
										}

										candidates[EXCLUDED][dim][j] |= val;
										pattern[FISH_ALL]++;

									}
								}
							}
						} while((tuple &= tuple - 1));
					}
				}
			}
		}//look for triplets


//-------------------------------------------------QUADS-------------------------------------------------
			
			if( (size_cells[0] + size_cells[1] + size_cells[2]) > 3 && size_cells[3] > 4) {
			
//						initialise flag			
				for(k = 0; k < index; naked_subset_info[k++].is_tuple = FALSE); 
				
//						form the combinaisons: C(size_cells[0]+size_cells[1]+size_cells[2],QUAD)
				size = size_cells[0] + size_cells[1] + size_cells[2];
				for(c1 = 0; c1 < size - 3; c1++) {
					for(c2 = c1 + 1; c2 < size - 2; c2++) {
						for(c3 = c2 + 1; c3 < size - 1; c3++) {
							for(c4 = c3 + 1; c4 < size; c4++) {
//						if the four cells form a quad
								if(BITCOUNT((	naked_subset_info[c1].p_cell->data | naked_subset_info[c2].p_cell->data |
												naked_subset_info[c3].p_cell->data | naked_subset_info[c4].p_cell->data)) == IS_QUAD ) {
//						collect info: cell c1 forms a quad with cell c2 cell C3 and cell c4
									naked_subset_info[c1].is_tuple = TRUE; naked_subset_info[c1].peers[0] = c2; 
									naked_subset_info[c1].peers[1] = c3; naked_subset_info[c1].peers[2] = c4;
								}
							}
						}
					}
				}			

//							find the quads
				for(k = 0; k < index; k++) {
				
					if(naked_subset_info[k].is_tuple == TRUE) {
					show_tuple.tuple = tuple = fish_info_index = 0;
					fish_all_info.type = JELLYFISH; fish_all_info.space_type = space_type;
					
						for(j = 0; j < 9; j++) {
							row = col_coordinates[i][j].x; col = col_coordinates[i][j].y;
					
							if(	&space_table[row][col] == naked_subset_info[k].p_cell ||
								&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[0]].p_cell ||
								&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[1]].p_cell ||
								&space_table[row][col] == naked_subset_info[naked_subset_info[k].peers[2]].p_cell) {
							
								show_tuple.tuple |= (1 << row); tuple |= space_table[row][col].data; val = (1 << col);
								
								fish_all_info.dim[fish_info_index][0] = row;
								fish_all_info.dim[fish_info_index++][1] = space_table[row][col].data;
								fish_all_info.candidate = col;

							}		
						}
					
						c1 = bit_rank[show_tuple.tuple & -show_tuple.tuple]; show_tuple.tuple &= show_tuple.tuple - 1;
						c2 = bit_rank[show_tuple.tuple & -show_tuple.tuple]; show_tuple.tuple &= show_tuple.tuple - 1;
						c3 = bit_rank[show_tuple.tuple & -show_tuple.tuple]; c4 = bit_rank[show_tuple.tuple & (show_tuple.tuple - 1)]; 

//						in rc_space: look in the col=data for val=col passing row=row
						if(space_type == ROW) {
					
							do {dim = bit_rank[tuple & -tuple];
	
								for(j = 0; j < 9; j++) {
							
									if(sdk_cellinfo[j][dim].type == IS_CANDIDATE && j != c1 && j != c2 && j != c3 && j != c4) {
								
										if(sdk_cellinfo[j][dim].data & val) {			

											if(flag_hint) {
												hint_info.type = JELLYFISH; hint_info.tuple = IS_QUAD; hint_info.candidate = fish_all_info.candidate;
												hint_info.zone = ROW; hint_info.unit[ROW] = c1;
												hint_info.unit[COLUMN] = dim; return 1;
											}

											candidates[EXCLUDED][j][dim] |= val;
											pattern[FISH_ALL]++;

										}
									}
								}
							} while((tuple &= tuple - 1));
						}
					
//						in rc_space: look in the row=data for val=col passing col=row				
						else if(space_type == COLUMN) {
					
							do {dim = bit_rank[tuple & -tuple];
	
								for(j = 0; j < 9; j++) {
							
									if(sdk_cellinfo[dim][j].type == IS_CANDIDATE && j != c1 && j != c2 && j != c3 && j != c4) {
								
										if(sdk_cellinfo[dim][j].data & val) {			

											if(flag_hint) {
												hint_info.type = JELLYFISH; hint_info.tuple = IS_QUAD; hint_info.candidate = fish_all_info.candidate;
												hint_info.zone = COLUMN; hint_info.unit[COLUMN] = c1;
												hint_info.unit[ROW] = dim; return 1;
											}

											candidates[EXCLUDED][dim][j] |= val;
											pattern[FISH_ALL]++;

										}
									}
								}
							} while((tuple &= tuple - 1));
						}
					}
				}
			}//look for quad
		
	}//scan grid
	compute_candidates();
	return pattern[FISH_ALL];
}

int backtraking() {
	int min, bitcount;
	int i,j;
	int candidate;
	int c;
	CELL old_sdk_cellinfo[9][9];
	int old_mask_unit[2][MAX_MASK];
	int old_candidates[2][9][9];

	CELL *p_cell = NULL;
	min = 9;
	
	int success;
	do{
		success = 0;
		success |= naked_single();
		success |= hidden_single();
	} while(success);
	
	if(!grid_consistent()) {return 0;}

	if(is_grid_finished()) return 1;

    for (i = 0; i < 9; i++) {
        for (j = 0; j < 9; j++) {
			if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {
				bitcount = BITCOUNT(sdk_cellinfo[i][j].data);
				if(bitcount > 1 && bitcount < min) {
					min = bitcount;
					p_cell = &sdk_cellinfo[i][j];
				}
			}
		}
	}
  
    if (p_cell == 0) {return 0;}

	memcpy((void *) old_sdk_cellinfo, (void *) sdk_cellinfo, sizeof(sdk_cellinfo));
	memcpy((void *) old_mask_unit, (void *) mask_unit, sizeof(mask_unit));
	memcpy((void *) old_candidates, (void *) candidates, sizeof(candidates));

	candidate = p_cell->data;
	
	int x = (p_cell - &sdk_cellinfo[0][0]) / 9;
	int y = (p_cell - &sdk_cellinfo[0][0]) % 9;

	do {		
		c = bit_rank[candidate & -candidate];
		
		sdk_cellinfo[x][y].type = IS_FOUND; sdk_cellinfo[x][y].data = c + 1;
		mask_unit[MASK_FOUND][x] |= (1 << c);
		mask_unit[MASK_FOUND][y + 9] |= (1 << c);
		mask_unit[MASK_FOUND][BLOCK_NUMBER(x,y) + 18] |= (1 << c);
		compute_candidates(); 
		
		if(backtraking()) {return 1;}
		
		memcpy((void *) sdk_cellinfo, (void *) old_sdk_cellinfo, sizeof(sdk_cellinfo));
		memcpy((void *) mask_unit, (void *) old_mask_unit, sizeof(mask_unit));
		memcpy((void *) candidates, (void *) old_candidates, sizeof(candidates));
		
	} while((candidate &= candidate - 1));
	
	memcpy((void *) sdk_cellinfo, (void *) old_sdk_cellinfo, sizeof(sdk_cellinfo));
	memcpy((void *) mask_unit, (void *) old_mask_unit, sizeof(mask_unit));
	memcpy((void *) candidates, (void *) old_candidates, sizeof(candidates));

    return 0;
}

//
//												KEY FUNCTIONS		
//
void draw_grid_img() {
	int i,j,c;
	hpg_t *grid_img;
	int w,h,xoff,yoff;
	int row,col;

	w = h = 234; xoff = (w-131)/2; yoff = (h-80) /2;
	if((grid_img = hpg_alloc_gray16_image(w, h)) == NULL) {display_message(ERROR_MEMORY); restore_screen(); return;}	
	sys_slowOff(); hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_clear(); hpg_clear_on(grid_img);
	
	#define BIT2_SCROLL		2
	#define BIT4_SCROLL		4
	
	#define VX1				0
	#define VX2				(VX1 + 26)
	#define VX3				(VX1 + 51)
	#define VX4				(VX1 + 77)
	#define VX5				(VX4 + 26)
	#define VX6				(VX4 + 51)
	#define VX7				(VX4 + 77)
	#define VX8				(VX7 + 26)
	#define VX9				(VX7 + 51)
	#define VX10			232

	#define VY1				1
	#define VY2				(VY1 + 26)
	#define VY3				(VY1 + 51)
	#define VY4				(VY1 + 77)
	#define VY5				(VY4 + 26)
	#define VY6				(VY4 + 51)
	#define VY7				(VY4 + 77)
	#define VY8				(VY7 + 26)
	#define VY9				(VY7 + 51)
	#define VY10			232

	int offset[9][2] = {{0,0},{VX4,0},{VX7,0},{0,VY4-1},{VX4,VY4-1},{VX7,VY4-1},{0,VY7-1},{VX4,VY7-1},{VX7,VY7-1}};

//				draw the grid
	hpg_draw_rect_on(grid_img,VX1,VY1,VX10,VY10); hpg_draw_rect_on(grid_img,VX1+1,VY1+1,VX10-1,VY10-1);
//				horizontal
	hpg_draw_line_on(grid_img,VX1,VY2,VX10,VY2); hpg_draw_line_on(grid_img,VX1,VY3,VX10,VY3); 
	hpg_draw_line_on(grid_img,VX1,VY4,VX10,VY4); hpg_draw_line_on(grid_img,VX1,VY4-1,VX10,VY4-1); 
	hpg_draw_line_on(grid_img,VX1,VY5,VX10,VY5); hpg_draw_line_on(grid_img,VX1,VY6,VX10,VY6); 
	hpg_draw_line_on(grid_img,VX1,VY7,VX10,VY7); hpg_draw_line_on(grid_img,VX1,VY7-1,VX10,VY7-1); 
	hpg_draw_line_on(grid_img,VX1,VY8,VX10,VY8); hpg_draw_line_on(grid_img,VX1,VY9,VX10,VY9); 
//				vertical
	hpg_draw_line_on(grid_img,VX2,VY1,VX2,VY10); hpg_draw_line_on(grid_img,VX3,VY1,VX3,VY10); 
	hpg_draw_line_on(grid_img,VX4,VY1,VX4,VY10); hpg_draw_line_on(grid_img,VX4-1,VY1,VX4-1,VY10); 
	hpg_draw_line_on(grid_img,VX5,VY1,VX5,VY10); hpg_draw_line_on(grid_img,VX6,VY1,VX6,VY10); 
	hpg_draw_line_on(grid_img,VX7,VY1,VX7,VY10); hpg_draw_line_on(grid_img,VX7-1,VY1,VX7-1,VY10); 
	hpg_draw_line_on(grid_img,VX8,VY1,VX8,VY10); hpg_draw_line_on(grid_img,VX9,VY1,VX9,VY10); 

//				fill the grid
	for (i = 0; i < 9; i++)  for (j = 0; j < 9; j++) {
		row = box_coordinates[i][j].x; col = box_coordinates[i][j].y;
		int data = space_table[row][col].data;
		if(space_table[row][col].type == IS_CANDIDATE) {
			do {c = bit_rank[data & -data];
				hpg_draw_letter_on(grid_img,sdk_digit[c+1],coordinates[box_coordinates[j][c].x][box_coordinates[j][c].y].x + offset[i][0], 
															coordinates[box_coordinates[j][c].x][box_coordinates[j][c].y].y + offset[i][1]);
			} while((data &= data - 1));
		}
		else {
			hpg_set_font(grid_img, hpg_get_bigfont());
			hpg_draw_letter_on(grid_img,sdk_digit[data],	coordinates[box_coordinates[j][4].x][box_coordinates[j][4].y].x + offset[i][0], 
															coordinates[box_coordinates[j][4].x][box_coordinates[j][4].y].y + offset[i][1]);
			hpg_set_font(grid_img, hpg_get_minifont());	
		}
	}

	if(flag_xychain) {
		int l;int i0,j0,i1,j1;
		int c5[] = {11,36,61,88,113,138,165,190,215};
		for(l = 0; l < xy_chain_length - 1; l++) {
			i0 = xy_chain_array[l].cell_index/9; 	j0 = xy_chain_array[l].cell_index%9;			
			i1 = xy_chain_array[l+1].cell_index/9; j1 = xy_chain_array[l+1].cell_index%9;
			hpg_draw_line_on(grid_img,c5[j0],c5[i0],c5[j1],c5[i1]);		
			hpg_fill_rect_on(grid_img,c5[j0]-2,c5[i0]-1,c5[j0]+3,c5[i0]+5);
			hpg_set_color(grid_img,HPG_COLOR_WHITE);
			hpg_draw_letter_on(grid_img,sdk_digit[l+1], c5[j0],c5[i0]);
			hpg_set_color(grid_img, HPG_COLOR_BLACK);
		}
		i0 = xy_chain_array[xy_chain_length-1].cell_index/9; j0 = xy_chain_array[xy_chain_length-1].cell_index%9;			
		hpg_fill_rect_on(grid_img,c5[j0]-2,c5[i0]-1,c5[j0]+3,c5[i0]+5);
		hpg_set_color(grid_img,HPG_COLOR_WHITE);
		hpg_draw_letter_on(grid_img,sdk_digit[xy_chain_length], c5[j0],c5[i0]);
		hpg_set_color(grid_img, HPG_COLOR_BLACK);
		hpg_draw_line_on(grid_img, c5[hint_info.unit[1]]-4,c5[hint_info.unit[0]]-4,c5[hint_info.unit[1]]+8,c5[hint_info.unit[0]]+8);
		hpg_draw_line_on(grid_img, c5[hint_info.unit[1]]-4,c5[hint_info.unit[0]]+8,c5[hint_info.unit[1]]+8,c5[hint_info.unit[0]]-4);
	}
	
	hpg_blit (grid_img, xoff, yoff, 131, 80, hpg_stdscreen, 0, 0);
	
	while(1) {
		if(keyb_isRight() && ((xoff + 131) < w)) {
			if(keyb_isRS()) xoff = w - 131;else if(keyb_isLS()) xoff += BIT4_SCROLL; else xoff += BIT2_SCROLL;
			hpg_blit (grid_img, xoff, yoff, 131, 80, hpg_stdscreen, 0, 0);}
		
		else if(keyb_isLeft() && (xoff > 0)) {
			if(keyb_isRS()) xoff = 0; else if(keyb_isLS()) xoff -= BIT4_SCROLL; else xoff -= BIT2_SCROLL;
			hpg_blit (grid_img, xoff, yoff, 131, 80, hpg_stdscreen, 0, 0);}
		
		else if(keyb_isDown() && ((yoff + 80) < h)) {
			if(keyb_isRS()) yoff = h - 80; else if(keyb_isLS()) yoff += BIT4_SCROLL; else yoff += BIT2_SCROLL;
			hpg_blit (grid_img, xoff, yoff, 131, 80, hpg_stdscreen, 0, 0);}
		
		else if(keyb_isUp() && (yoff > 0)) {
			if(keyb_isRS()) yoff = 0; else if(keyb_isLS()) yoff -= BIT4_SCROLL; else yoff -= BIT2_SCROLL;
			hpg_blit (grid_img, xoff, yoff, 131, 80, hpg_stdscreen, 0, 0);}
			
		else if(keyb_isAnyKeyPressed()) {
			if(keyb_isKeyPressed(4,1)) return;
			else if(keyb_isKeyPressed(0,2)) {
				FILE *handle;
				char pgm_header[15] = {80,53,10,50,51,52,32,50,51,52,10,50,53,53,10};
				if((handle = fopen("screen.pgm", "w"))) {
					for(i = 0; i < 15; i++) fputc( pgm_header[i], handle);
					for(i = 0; i < 234; i++) for(j = 0; j <  234; j++) {
						fputc( ~hpg_get_pixel(grid_img, j, i), handle);
					}
					fclose(handle);
				}
			}	
		}
	}
	sys_slowOn(); hpg_free_image(grid_img); 
}

void screen_snapshot() {
	FILE *handle;
	int i,j;
	char pgm_header[14] = {80,53,10,49,51,49,32,56,48,10,50,53,53,10};
	if((handle = fopen("screen.pgm", "w")) == NULL) {display_message(ERROR_FILE); restore_screen();}
	else {
		for(i = 0; i < 14; i++) fputc( pgm_header[i], handle);
		
		for(i = 0; i < 80; i++) {
			for(j = 0; j <  131; j++) {
				fputc( ~hpg_get_pixel(hpg_stdscreen, j, i), handle);
			}
		}	
		fclose(handle);
		display_message(SNAPSHOT); restore_screen();
	}
}
void puzzle_to_string() {
	int i,j,k;
	char string[82]; 
	for(i = 0, k = 0; i < 9; i++) for(j = 0; j < 9; j++) {
		if(sdk_cellinfo[i][j].type == IS_GIVEN) string[k] = sdk_cellinfo[i][j].data + 48;
		else string[k] = 48; k++;
	}
	string[81] = '\0';
	sat_stack_push_string(string);
	++stack_level;
	display_message(PUSHED); restore_screen();
}

void string_to_puzzle() {
	int i,j,digit;
	char *string;
	if((string = sat_pick_str_alloc(stack_level)) && strlen(string) == 81) {
		for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) {
			if(!ISNUMBER((digit = (int) string[(i*9)+j]))) {sdk_cellinfo[i][j].type = IS_CANDIDATE;}
			else {sdk_cellinfo[i][j].type = IS_GIVEN; sdk_cellinfo[i][j].data = digit - 48;}
		}		
	free(string); 
	draw_grid(); fill_display_grid(); clear_display_area();
	while(!validate_grid()) get_grid_from_keyboard();
	manu_space_type = RC_SPACE; fill_space_table(manu_space_type);
	cursor.x = 4; cursor.y = 4; unit_type = -1; filtered_candidate = 0; flag_xychain = 0; restore_screen();			
	}
}

void show_subset(){
	int i,tuple;
	if(show_tuple.flag) {
		memset((char*) buffer, 0, sizeof(buffer));
		strcpy(buffer, show_tuple.name);
		i = strlen(buffer);
		buffer[i++] = ':'; buffer[i++] = ' ';
		tuple = show_tuple.tuple;
		do {
			buffer[i++] = bit_rank[tuple & -tuple]+49;
			buffer[i++] = ' ';
		} while((tuple &= tuple - 1));
		buffer[i] = '\0';
		message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); restore_screen();
		show_tuple.flag = 0;
	}
}

int resolve_grid() {
	int i,success;
	for(i = 0; i < MAX_PATTERN; pattern[i++] = 0);
	do{
		success = 0;
		success |= naked_single(); if(success) while(naked_single()); if(is_grid_finished()) return 1;
		
		success |= hidden_single(); if(success) while(naked_single() | hidden_single()); if(is_grid_finished()) return 1;
		
		success |= pointing(); if(success) while(naked_single() | hidden_single() | pointing()); if(is_grid_finished()) return 1;
		
		success |= claiming(); if(success) while(naked_single() | hidden_single() | claiming()); if(is_grid_finished()) return 1;
		
		success |= (naked_subset(ROW) | naked_subset(COLUMN) | naked_subset(BLOCK));
		if(success) while(naked_single() | hidden_single() | naked_subset(BLOCK) | naked_subset(COLUMN) | naked_subset(ROW));
		if(is_grid_finished()) return 1;
		
		success |= (hidden_subset(ROW) | hidden_subset(COLUMN) | hidden_subset(BLOCK));
		if(success) while(naked_single() | hidden_single() | hidden_subset(ROW) | hidden_subset(COLUMN) | hidden_subset(BLOCK));
		if(is_grid_finished()) return 1;
		
		success |= (fish(COLUMN) | fish(ROW));
		if(success) while(naked_single() | hidden_single() | fish(COLUMN) | fish(ROW));
		if(is_grid_finished()) return 1;
	
	} while(success);
	return 0;
}

void hint() {		
		int success = 0;
		for(;;) {
			if( (success = naked_single())) break;
			if( (success = hidden_single())) break;
			if( (success = pointing())) break;
			if( (success = claiming())) break;
			if( (success = naked_subset(ROW))) break;
			if( (success = naked_subset(COLUMN))) break;
			if( (success = naked_subset(BLOCK))) break;
			if( (success = hidden_subset(ROW))) break;
			if( (success = hidden_subset(COLUMN))) break;
			if( (success = hidden_subset(BLOCK))) break;
			if( (success = fish(ROW))) break;
			if( (success = fish(COLUMN))) break;
			break;
		}
		if(success) {
			manu_space_type = RC_SPACE;
			switch(hint_info.type) {
				case NAKED			:   sprintf(buffer, "Naked single in r%dc%d", hint_info.unit[ROW] + 1, hint_info.unit[COLUMN] + 1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
										fill_space_table(manu_space_type); restore_screen(); 
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN];
										unit_type = hint_info.zone; display_candidates();
										return;
				case HIDDEN			:	sprintf(buffer, "Hidden single in r%dc%d", hint_info.unit[ROW] + 1, hint_info.unit[COLUMN] + 1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
										fill_space_table(manu_space_type); restore_screen(); 
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN];
										unit_type = hint_info.zone; display_candidates();
										return;
				case POINTING		:	sprintf(buffer, "Pointing in block %d", hint_info.unit[hint_info.zone] + 1); 
										filtered_candidate = hint_info.candidate + 1; flag_locked = 1;
										cursor.x = block_start[hint_info.unit[hint_info.zone]][0] + 1;
										cursor.y = block_start[hint_info.unit[hint_info.zone]][1] + 1;
										break;
				case CLAIMING		:	sprintf(buffer, "Claiming in block %d", hint_info.unit[hint_info.zone] + 1); 
										filtered_candidate = hint_info.candidate + 1; flag_locked = 1;
										cursor.x = block_start[hint_info.unit[hint_info.zone]][0] + 1;
										cursor.y = block_start[hint_info.unit[hint_info.zone]][1] + 1;
										break;
				case NAKED_TUPLE	:	sprintf(buffer, "Naked %s in %s %d", hint_info.tuple_name[hint_info.tuple - 2],
												hint_info.zone_name[hint_info.zone], hint_info.unit[hint_info.zone] + 1); 
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
										fill_space_table(manu_space_type); restore_screen();
										show_tuple.name = hint_info.tuple_name[hint_info.tuple - 2];
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN]; 
										unit_type = hint_info.zone; display_candidates(); show_tuple.flag = 1;
										return;
				case HIDDEN_TUPLE	:	sprintf(buffer, "Hidden %s in %s %d", hint_info.tuple_name[hint_info.tuple - 2],
												hint_info.zone_name[hint_info.zone], hint_info.unit[hint_info.zone] + 1); 
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
										fill_space_table(manu_space_type); restore_screen();
										show_tuple.name = hint_info.tuple_name[hint_info.tuple - 2];
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN]; 
										unit_type = hint_info.zone; display_candidates(); show_tuple.flag = 1;
										return;
				case X_WING			:
				case SWORDFISH		:
				case JELLYFISH		:	sprintf(buffer, "%s in %s %d", hint_info.fish_name[hint_info.tuple - 2],
												hint_info.zone_name[hint_info.zone], hint_info.unit[hint_info.zone] + 1); 
										filtered_candidate = hint_info.candidate + 1; flag_fish = 1;
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN]; 
										break;
			}
			message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); fill_space_table(manu_space_type); restore_screen(); 
			return;
		}
	display_message(NO_HINT); fill_space_table(manu_space_type); restore_screen();
}


void select_hint(int method) {
	if(method >= 0) {
			manu_space_type = RC_SPACE;
			switch(method) {
				case NAKED			:   sprintf(buffer, "Naked single in r%dc%d", hint_info.unit[ROW] + 1, hint_info.unit[COLUMN] + 1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
										fill_space_table(manu_space_type); restore_screen(); 
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN];
										unit_type = hint_info.zone; display_candidates();
										return;
				case HIDDEN			:	sprintf(buffer, "Hidden single in r%dc%d", hint_info.unit[ROW] + 1, hint_info.unit[COLUMN] + 1);
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
										fill_space_table(manu_space_type); restore_screen(); 
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN];
										unit_type = hint_info.zone; display_candidates();
										return;
				case POINTING		:	sprintf(buffer, "Pointing in block %d", hint_info.unit[hint_info.zone] + 1); 
										filtered_candidate = hint_info.candidate + 1;  flag_locked = 1;
										cursor.x = block_start[hint_info.unit[hint_info.zone]][0] + 1;
										cursor.y = block_start[hint_info.unit[hint_info.zone]][1] + 1;
										break;
				case CLAIMING		:	sprintf(buffer, "Claiming in block %d", hint_info.unit[hint_info.zone] + 1); 
										filtered_candidate = hint_info.candidate + 1; flag_locked = 1;
										cursor.x = block_start[hint_info.unit[hint_info.zone]][0] + 1;
										cursor.y = block_start[hint_info.unit[hint_info.zone]][1] + 1;
										break;
				case NAKED_TUPLE	:	sprintf(buffer, "Naked %s in %s %d", hint_info.tuple_name[hint_info.tuple - 2],
												hint_info.zone_name[hint_info.zone], hint_info.unit[hint_info.zone] + 1); 
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
										fill_space_table(manu_space_type); restore_screen();
										show_tuple.name = hint_info.tuple_name[hint_info.tuple - 2];
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN]; 
										unit_type = hint_info.zone; display_candidates(); show_tuple.flag = 1;
										return;
				case HIDDEN_TUPLE	:	sprintf(buffer, "Hidden %s in %s %d", hint_info.tuple_name[hint_info.tuple - 2],
												hint_info.zone_name[hint_info.zone], hint_info.unit[hint_info.zone] + 1); 
										message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
										fill_space_table(manu_space_type); restore_screen();
										show_tuple.name = hint_info.tuple_name[hint_info.tuple - 2];
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN]; 
										unit_type = hint_info.zone; display_candidates(); show_tuple.flag = 1;
										return;
				case FISH_ALL		:	sprintf(buffer, "%s in %s %d", hint_info.fish_name[hint_info.tuple - 2],
												hint_info.zone_name[hint_info.zone], hint_info.unit[hint_info.zone] + 1); 
										filtered_candidate = hint_info.candidate + 1; flag_fish = 1;
										cursor.x = hint_info.unit[ROW]; cursor.y = hint_info.unit[COLUMN]; 
										break;
			}
			message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); return;
		}
	else {message[MESSAGE_SLOT] = select_hint_message[(-method)-1]; display_message(MESSAGE_SLOT);}
}

void restart_game() {
	int i,j; 
	for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) {
		if(sdk_cellinfo[i][j].type == IS_FOUND) sdk_cellinfo[i][j].type = IS_CANDIDATE; candidates[EXCLUDED][i][j] = 0;}
	for (i = 0; i < MAX_MASK; mask_unit[MASK_FOUND][i++] = 0); 
	if(filtered_candidate) filter_candidate(filtered_candidate);
	compute_candidates(); 
	fill_space_table(manu_space_type); draw_grid(); clear_display_area(); fill_display_grid(); 
	cursor.x = 4; cursor.y = 4; unit_type = -1;
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR);
}

void input_found(int found) {
	int old_found = 0; 
	int old_filtered = 0;
	int row,col,val;
	
	row = col = val = 0;
	

//					enter a digit into a cell: must be one of the candidates
	if(space_table[cursor.x][cursor.y].type == IS_GIVEN) return;
		
//					if filter is on turn it off
	if(filtered_candidate) {old_filtered = filtered_candidate; filter_candidate(filtered_candidate);}
	
//					if there is a found already erase it and recompute the candidates
	if(space_table[cursor.x][cursor.y].type == IS_FOUND) {
		old_found = space_table[cursor.x][cursor.y].data;
//					rcspace->rcspace
		if(manu_space_type == RC_SPACE) {row = cursor.x; col = cursor.y; val = old_found - 1;}
//					rnspace->rcspace
		else if(manu_space_type == RN_SPACE) {row = cursor.x; col = old_found - 1; val = cursor.y;}
//					cnspace->rcspace
		else if(manu_space_type == CN_SPACE) {col = cursor.x; row = old_found - 1;  val = cursor.y;}
//					bnspace->rcspace
		else if(manu_space_type == BN_SPACE) {
			row = block_start[cursor.x][0] + ((old_found - 1) / 3); 
			col = block_start[cursor.x][1] + ((old_found - 1) % 3);  
			val = cursor.y;
		}			
//					update info
		mask_unit[MASK_FOUND][row] &= ~(1 << val); 
		mask_unit[MASK_FOUND][col + 9] &= ~(1 << val);
		mask_unit[MASK_FOUND][BLOCK_NUMBER(row,col) + 18] &= ~(1 << val); 	
			
		candidates[EXCLUDED][row][col] = 0;
		sdk_cellinfo[row][col].type = IS_CANDIDATE; compute_candidates(); 
	}
		
//					if the found is not one of the candidates: if there was a found already put it back
//					an excluded is no longer in the candidates list
	if( !(space_table[cursor.x][cursor.y].data & (1 << (found - 1))) ) {
		if(old_found) found = old_found;
		else {if(old_filtered) filter_candidate(old_filtered); return;}
	}
		
//				update screen input cell
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
	hpg_fill_rect(	coordinates[cursor.x][cursor.y].x-3,coordinates[cursor.x][cursor.y].y-2,
					coordinates[cursor.x][cursor.y].x+4,coordinates[cursor.x][cursor.y].y+5);
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
		
//					rcspace->rcspace
	if(manu_space_type == RC_SPACE) {row = cursor.x; col = cursor.y; val = found - 1;}
//					rnspace->rcspace
	else if(manu_space_type == RN_SPACE) {row = cursor.x; col = found - 1; val = cursor.y; }
//					cnspace->rcspace
	else if(manu_space_type == CN_SPACE) {col = cursor.x; row = found - 1; val = cursor.y;}
//					bnspace->rcspace
	else if(manu_space_type == BN_SPACE) {
		row = block_start[cursor.x][0] + ((found - 1) / 3); 
		col = block_start[cursor.x][1] + ((found - 1) % 3);
		val = cursor.y;  
	}			
	
//				update info
	sdk_cellinfo[row][col].type = IS_FOUND; 
	sdk_cellinfo[row][col].data = val + 1; 

	mask_unit[MASK_FOUND][row] |= (1 << val); 
	mask_unit[MASK_FOUND][col + 9] |= (1 << val);
	mask_unit[MASK_FOUND][BLOCK_NUMBER(row,col) + 18] |= (1 << val); 	
		
	candidates[EXCLUDED][row][col] = 0; compute_candidates(); 
		
//				update display screen
	fill_space_table(manu_space_type); display_candidates();
		
//				if there was a filter put it back
	if(old_filtered) filter_candidate(old_filtered);
		
//				if grid is finished display a message
	if(is_grid_finished()) {display_message(FINISHED); fill_space_table(RC_SPACE); unit_type = -1; cursor.x = 4; cursor.y = 4; restore_screen();}
	
	check_grid(); flag_xychain = 0; restore_screen();
}

void exclude_candidate(int candidate) {
//					toggle on/off candidate from the candidates list of a cell
	int old_filtered = 0;
	int row,col,val;
	row = col = val = 0;

	if(space_table[cursor.x][cursor.y].type != IS_CANDIDATE) return;
	
//					if filter is on turn it off
	if(filtered_candidate == candidate) {old_filtered = filtered_candidate; filter_candidate(filtered_candidate);}	

//					rcspace->rcspace
	if(manu_space_type == RC_SPACE) {row = cursor.x; col = cursor.y; val = candidate - 1;}
//					rnspace->rcspace
	else if(manu_space_type == RN_SPACE) {row = cursor.x; col = candidate - 1; val = cursor.y; }
//					cnspace->rcspace
	else if(manu_space_type == CN_SPACE) {col = cursor.x; row = candidate - 1; val = cursor.y;}
//					bnspace->rcspace
	else if(manu_space_type == BN_SPACE) {
		row = block_start[cursor.x][0] + ((candidate - 1) / 3); 
		col = block_start[cursor.x][1] + ((candidate - 1) % 3);
		val = cursor.y;  
	}			

//					if it is already excluded put it back
	if(manu_space_type == RC_SPACE) {	
		if(candidates[EXCLUDED][cursor.x][cursor.y] & (1 << (candidate - 1))) {
			candidates[EXCLUDED][cursor.x][cursor.y] &= ~(1 << (candidate - 1));
		}
		else {
//					if the candidate exists and is not the only one exclude it
			if(sdk_cellinfo[row][col].data & (1 << val) && sdk_cellinfo[row][col].data & ~(1 << val)) {
				candidates[EXCLUDED][row][col] |= (1 << val);
			}
		}
	}
	else {
		if(space_table[cursor.x][cursor.y].data & (1 << (candidate - 1)) && space_table[cursor.x][cursor.y].data & ~(1 << (candidate - 1))) {
				candidates[EXCLUDED][row][col] |= (1 << val);
		}
	}

	compute_candidates(); fill_space_table(manu_space_type); display_candidates();

//					if there was a filter put it back
	if(old_filtered) filter_candidate(old_filtered);
	
	check_grid(); flag_xychain = 0; restore_screen();
}

void filter_candidate(int candidate) {
//					screen update on filter candidate
	int i,j; 
	
	if(flag_xychain) return;
	
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
//					if there is a filter already discard it
	if(filtered_candidate) {
		for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) {
			if(space_table[i][j].type == IS_CANDIDATE && space_table[i][j].data & (1 << (filtered_candidate - 1))) {
				hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
				hpg_fill_rect(coordinates[i][j].x-3,coordinates[i][j].y-2,coordinates[i][j].x+4,coordinates[i][j].y+5);
				hpg_set_color(hpg_stdscreen,HPG_COLOR_BLACK);
				hpg_draw_letter(sdk_digit[0], coordinates[i][j].x, coordinates[i][j].y);
			}
		}
	}
//					if it is a new filter filter it
	if(filtered_candidate != candidate) {
		filtered_candidate = candidate;
		for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) {
//					if the candidate exits 
			if(space_table[i][j].type == IS_CANDIDATE && space_table[i][j].data & (1 << (candidate - 1)) ) {
				hpg_set_color(hpg_stdscreen, HPG_COLOR_GRAY_10);
				hpg_fill_rect(coordinates[i][j].x-3,coordinates[i][j].y-2,coordinates[i][j].x+4,coordinates[i][j].y+5);
				hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
				hpg_draw_letter(sdk_digit[candidate], coordinates[i][j].x, coordinates[i][j].y);
			}
		}
	}
//					the filter has been discarded and not replaced	
	else filtered_candidate = 0;
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); 
}

void display_candidates() {
//				display the candidates of the row, the column or the block where the cursor is
//				which one depends on the value of unit_type
	int i,j,k,l,block,letter;
	clear_display_area();
	
	if(unit_type == -1) return;
	
	if(manu_space_type == RC_SPACE) unit_type %= MAX_UNIT;
	else if(manu_space_type == BN_SPACE) unit_type %= (MAX_UNIT - 2);
	else unit_type %= (MAX_UNIT - 1);
	
	if( unit_type == ROW ) {
			sprintf(buffer, "%s(%d,%d)", space_title[manu_space_type], cursor.x+1, cursor.y+1 ); hpg_draw_text(buffer, 80, 0);
			sprintf(buffer, "Row %d", cursor.x+1 ); hpg_draw_text(buffer, 95, 8);
			hpg_draw_letter('>',  DISPLAY_X - 5, DISPLAY_Y + (cursor.y * 7) );
		for( i = 0; i < 9; i++ ) {
			if( space_table[cursor.x][i].type == IS_CANDIDATE ) {
				for(j = 0; j < 9; j++)	{		
					letter = space_table[cursor.x][i].data & (1 << j) ? j+1 : 0; hpg_draw_letter(sdk_digit[letter], DISPLAY_X + (j * 5), DISPLAY_Y + (i * 7));
				}
			} 
			else {
				for(j = 0; j < 9; j++) hpg_draw_letter(sdk_digit[0], DISPLAY_X + (j * 5), DISPLAY_Y + (i * 7) );
			}
		}
	}
	else if( unit_type == COLUMN ) {
			sprintf(buffer, "%s(%d,%d)", space_title[manu_space_type], cursor.x+1, cursor.y+1 ); hpg_draw_text(buffer, 80, 0);
			sprintf(buffer, "Column %d", cursor.y+1 ); hpg_draw_text(buffer, 91, 8);
			hpg_draw_letter('>',  DISPLAY_X - 5, DISPLAY_Y + (cursor.x * 7) );
		for( i = 0; i < 9; i++ ) {
			if( space_table[i][cursor.y].type == IS_CANDIDATE ) {
				for(j = 0; j < 9; j++)	{		
					letter = space_table[i][cursor.y].data & (1 << j) ? j+1 : 0; hpg_draw_letter(sdk_digit[letter], DISPLAY_X + (j * 5), DISPLAY_Y + (i * 7));
				}
			} 
			else {
				for(j = 0; j < 9; j++) hpg_draw_letter(sdk_digit[0], DISPLAY_X + (j * 5), DISPLAY_Y + (i * 7) );
			}
		}
	}
	else if( unit_type == BLOCK ){	
		block = BLOCK_NUMBER(cursor.x,cursor.y); k = 0;
		sprintf(buffer, "%s(%d,%d)", space_title[manu_space_type], cursor.x+1, cursor.y+1 ); hpg_draw_text(buffer, 80, 0);
		sprintf(buffer, "Block %d", block+1 ); hpg_draw_text(buffer, 93, 8);
		hpg_draw_letter('>',  DISPLAY_X - 5, DISPLAY_Y + ( (((cursor.x % 3) * 3) + (cursor.y  % 3)) * 7) );
		for(i = block_start[block][0]; i < block_start[block][0] + 3; i++)
			for(j = block_start[block][1]; j < block_start[block][1] + 3; j++, k++) {
				if(space_table[i][j].type == IS_CANDIDATE) {
					for(l = 0; l < 9; l++)	{		
						letter = space_table[i][j].data & (1 << l) ? l+1 : 0; hpg_draw_letter(sdk_digit[letter], DISPLAY_X + (l * 5), DISPLAY_Y + (k * 7));
					}
			} 
				else {
					for(l = 0; l < 9; l++) hpg_draw_letter(sdk_digit[0], DISPLAY_X + (l * 5), DISPLAY_Y + (k * 7) );
			} 
		}
	}
}

void load_grid() {
	int i,j,key; FILE *handle; 
	int line_length, digit;
	if((handle = fopen(grid_filename, "r")) == NULL) {
		if(strlen(grid_filename) > 15) {grid_filename[13] = '.'; grid_filename[14] = '\0';}
		sprintf(buffer, "%s not found!", grid_filename); message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT);	
		draw_grid(); fill_display_grid(); hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR);
	}
	else {
		fgets_local(buffer, BUFFER_LENGTH, handle); line_length = strlen(buffer);		
		if(next_load_file == 1) ++grid_number;
		goto seek_grid;
		while(1) {
			sys_waitRTCTicks(15);
			if(( key = keyb_getkeyM(0) ) != 0) {		
				switch(key & 0xff) {
					case KB_UP : if(key & 0x20000000) grid_number += 500; else grid_number += 100; goto seek_grid;
					case KB_DN : if(key & 0x20000000) grid_number -= 500; else grid_number -= 100; if(grid_number < 1) grid_number = 1; goto seek_grid;
					case KB_LF : if(key & 0x20000000) {if(grid_number > 10) grid_number -= 10;} else {if(grid_number > 1) --grid_number;}; goto seek_grid;
					case KB_RT : if(key & 0x20000000) {grid_number += 10; goto seek_grid;} else grid_number++; goto get_grid;
					seek_grid  :fseek(handle, ((grid_number - 1) * line_length), SEEK_SET); 
					get_grid   :fgets(buffer, BUFFER_LENGTH, handle); 
								if (feof(handle)) {fseek (handle, 0, SEEK_SET); grid_number = 1; fgets(buffer, BUFFER_LENGTH, handle);}
								for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) {
									if(!ISNUMBER((digit = buffer[(i*9)+j]))) {sdk_cellinfo[i][j].type = IS_CANDIDATE;}
									else {sdk_cellinfo[i][j].type = IS_GIVEN; sdk_cellinfo[i][j].data = digit - 48;}
								}							
								fill_space_table(manu_space_type);
								draw_grid(); fill_display_grid(); clear_display_area();
								if(next_load_file) {next_load_file = 0; fclose(handle); return;}
								sprintf(buffer, "grid..%d", grid_number); hpg_draw_text(buffer, 103 - (strlen(buffer)<<1), 36);
								break;			
					case KB_ENT: clear_display_area(); fclose(handle); return;
				}
			}
		}
	}
}


void get_grid_from_keyboard() {
//												get or modify grid
	cursor.x = 0; cursor.y = 0; CURSOR c = cursor;
	int i,j;
	int letter, key, given;
	hpg_clear(); draw_grid(); 
	for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) 
		hpg_draw_letter(sdk_digit[sdk_cellinfo[i][j].type == IS_CANDIDATE ? 0 : sdk_cellinfo[i][j].data], coordinates[i][j].x, coordinates[i][j].y);
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR);
	while (1) {	
		if(sdk_cellinfo[cursor.x][cursor.y].type == IS_CANDIDATE) {
			hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
			hpg_draw_letter(sdk_digit[0], coordinates[cursor.x][cursor.y].x, coordinates[cursor.x][cursor.y].y);
			hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); letter = L_CURSOR;	
		}
		else letter = sdk_cellinfo[cursor.x][cursor.y].data;
		hpg_draw_letter(sdk_digit[letter], coordinates[cursor.x][cursor.y].x, coordinates[cursor.x][cursor.y].y);
		sys_waitRTCTicks(TICKS);
		if(( key = keyb_getkeyM(0) ) != 0) {
			switch(key & 0xff) {
				case KB_UP	:	c = cursor; if(key & 0x20000000) cursor.x -= 3; else cursor.x--; if(cursor.x < 0) cursor.x += 9; goto arrow;	
				case KB_DN	:	c = cursor; if(key & 0x20000000) cursor.x += 3; else  cursor.x++; cursor.x = cursor.x % 9; goto arrow;
				case KB_LF	:	c = cursor; if(key & 0x20000000) cursor.y -= 3; else  cursor.y--; if(cursor.y < 0) cursor.y += 9; goto arrow;
				case KB_RT	:	c = cursor; if(key & 0x20000000) cursor.y += 3; else cursor.y++; cursor.y = cursor.y % 9; goto arrow;
				case KB_1 :		given = 1; goto given;
				case KB_2 :		given = 2; goto given;
				case KB_3 :		given = 3; goto given;
				case KB_4 :		given = 4; goto given;
				case KB_5 :		given = 5; goto given;
				case KB_6 :		given = 6; goto given;
				case KB_7 :		given = 7; goto given;
				case KB_8 :		given = 8; goto given;
				case KB_9 :		given = 9; goto given;
				case KB_BKS:	if(key & 0x20000000) {
									clear_sdk_cellinfo(); hpg_clear(); draw_grid(); 
									for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) 
									hpg_draw_letter('.',coordinates[i][j].x, coordinates[i][j].y);
								}
								else {	
									hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
									hpg_draw_letter(sdk_digit[letter], coordinates[cursor.x][cursor.y].x, coordinates[cursor.x][cursor.y].y);
									hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
									sdk_cellinfo[cursor.x][cursor.y].type = IS_CANDIDATE; 
								}
								break;	
				arrow: 			hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
								if(letter == L_CURSOR) {
									hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); 
									hpg_draw_letter(sdk_digit[letter], coordinates[c.x][c.y].x, coordinates[c.x][c.y].y);
									letter = 0; hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
								}
								hpg_draw_letter(sdk_digit[letter], coordinates[c.x][c.y].x, coordinates[c.x][c.y].y);
								hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR);
								break;
				given: 			hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
								hpg_draw_letter(sdk_digit[letter], coordinates[cursor.x][cursor.y].x, coordinates[cursor.x][cursor.y].y);
								hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
								sdk_cellinfo[cursor.x][cursor.y].type = IS_GIVEN; sdk_cellinfo[cursor.x][cursor.y].data = given;
								break;
				case KB_ENT:	fill_space_table(manu_space_type); return;
			}
		}
	}
}

//
//												UTIL FUNCTIONS
//
int compare_bivalue_cell(const void *arg1, const void *arg2) {
	if( BITCOUNT(((BIVALUE_CELL *) arg1)->data) < BITCOUNT(((BIVALUE_CELL *) arg2)->data)) return(-1);
	else if( BITCOUNT(((BIVALUE_CELL *) arg1)->data) > BITCOUNT(((BIVALUE_CELL *) arg2)->data)) return(1);
	else return(0);
}

void fill_buddies_array(int space_type) {
	int i,j,k;
	if(space_type == RC_SPACE) {
		for(i = 0; i < 81; i++) { 
			for(j = 0, k = 0; j < 81; j++) {
				if(i != j && RC_share_unit_array.share_unit_array[i][j]) {
					RC_buddies_array.buddies_array[i][k++] = RC_share_unit_array.share_unit_array[i][j] - 1;
				}
			}
		}
	}
	else {
		for(i = 0; i < 81; i++) { 
			for(j = 0, k = 0; j < 81; j++) {
				if(i != j && RNCN_share_unit_array.share_unit_array[i][j]) {
				RNCN_buddies_array.buddies_array[i][k++] = RNCN_share_unit_array.share_unit_array[i][j] - 1;
				}
			}
		}
	}

}

void fill_share_unit_array(int space_type) {
	int i,j,k,l;
	if(space_type == RC_SPACE) {
		for(i = 0; i < 9; i++) { for(j = 0; j < 9; j++) {
			for(k = 0; k < 9; k++) { for(l = 0; l < 9; l++) {
				if(i == k || j == l || BLOCK_NUMBER(i,j) == BLOCK_NUMBER(k,l)) {
					RC_share_unit_array.share_unit_array[&space_table[i][j] - &space_table[0][0]][&space_table[k][l] - &space_table[0][0]]
						= (&space_table[k][l] - &space_table[0][0]) + 1;
				}				
			}}
		}}	
	}
	else {
		for(i = 0; i < 9; i++) { for(j = 0; j < 9; j++) {
			for(k = 0; k < 9; k++) { for(l = 0; l < 9; l++) {
				if(i == k || j == l) {
					RNCN_share_unit_array.share_unit_array[&space_table[i][j] - &space_table[0][0]][&space_table[k][l] - &space_table[0][0]]
						= (&space_table[k][l] - &space_table[0][0]) + 1;
				}
			}}
		}}
	}
}
void fill_space_table(int space_type) {
	int i,j,n,candidate, block,k;
		
	memset((char*) space_table, 0, sizeof(space_table));
	
	if(space_type == RC_SPACE) {
		memcpy((void *) space_table, (void *) sdk_cellinfo, sizeof(sdk_cellinfo));
	}
	else if(space_type == RN_SPACE) {
		for(i = 0; i < 9; i++) {
			for(j = 0; j < 9; j++) {
				if(sdk_cellinfo[i][j].type == IS_GIVEN || sdk_cellinfo[i][j].type == IS_FOUND) {
					space_table[i][sdk_cellinfo[i][j].data - 1].type = sdk_cellinfo[i][j].type;
					space_table[i][sdk_cellinfo[i][j].data - 1].data = j + 1;
				}
				else {
					candidate = sdk_cellinfo[i][j].data;	
					do {
						n = bit_rank[candidate & -candidate];
						space_table[i][n].type = IS_CANDIDATE;
						space_table[i][n].data |= (1 << j);
					} while((candidate &= candidate - 1));
				}
			}
		}
	}
	else if(space_type == CN_SPACE) {
		for(i = 0; i < 9; i++) {
			for(j = 0; j < 9; j++) {
				if(sdk_cellinfo[i][j].type == IS_GIVEN || sdk_cellinfo[i][j].type == IS_FOUND) {
					space_table[j][sdk_cellinfo[i][j].data - 1].type = sdk_cellinfo[i][j].type;
					space_table[j][sdk_cellinfo[i][j].data - 1].data = i + 1;
				}
				else {
					candidate = sdk_cellinfo[i][j].data;	
					do {
						n = bit_rank[candidate & -candidate];
						space_table[j][n].type = IS_CANDIDATE;
						space_table[j][n].data |= (1 << i);
					} while((candidate &= candidate - 1));
				}
			}
		}
	}
	else if(space_type == BN_SPACE) {
		for(block = 0; block < 9; block++) {
			for(i = block_start[block][0], k = 0; i < block_start[block][0] + 3; i++) {
				for(j = block_start[block][1]; j < block_start[block][1] + 3; j++, k++) {		
					if(sdk_cellinfo[i][j].type == IS_GIVEN || sdk_cellinfo[i][j].type == IS_FOUND) {
						space_table[block][sdk_cellinfo[i][j].data - 1].type = sdk_cellinfo[i][j].type;
						space_table[block][sdk_cellinfo[i][j].data - 1].data = k + 1;
					}
					else {
						candidate = sdk_cellinfo[i][j].data;	
						do {
							n = bit_rank[candidate & -candidate];
							space_table[block][n].type = IS_CANDIDATE;
							space_table[block][n].data |= (1 << k);
						} while((candidate &= candidate - 1));
					}
				}
			}
		}
	}
}

int check_grid() {
	int i,j;
	int mask, cell, candidate;
//							scan the rows
	for(i = 0; i < 9; i++) {
		mask = cell = candidate = 0;
		for(j = 0; j < 9; j++) {
//							not two equals founds in the same unit
			if(sdk_cellinfo[i][j].type == IS_FOUND) {
				if((mask & 1 << (sdk_cellinfo[i][j].data - 1)) != NULL) {
					sprintf(buffer, "Duplicate in r%dc%d !", i+1, j+1); message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
					cursor.x = i; cursor.y = j; manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); return 1;
				}
				mask |= 1 << (sdk_cellinfo[i][j].data - 1);
			}
//							at least one candidate in a candidate cell					
			else if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {
				if(!sdk_cellinfo[i][j].data) {
					sprintf(buffer, "No candidate in r%dc%d", i+1, j+1); message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT); 
					cursor.x = i; cursor.y = j; manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); return 1;
				}	
				++cell; candidate |= sdk_cellinfo[i][j].data;
			}
		}
//							as many candidates as there are candidate cells
		if(cell != BITCOUNT(candidate)) {
			sprintf(buffer, "Too few candidates in row %d", i+1); message[MESSAGE_SLOT] = (char*) &buffer; display_message(MESSAGE_SLOT);
			cursor.x = i; manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); return 1;
		}
	}	
	return 0;
}

int grid_consistent() {
	int i,j;
	int mask, cell, candidate;
	fill_space_table(RN_SPACE);
//							scan the rows
			for(i = 0; i < 9; i++) {
				mask = cell = candidate = 0;
				for(j = 0; j < 9; j++) {
//							not two equals founds in the same unit
					if(space_table[i][j].type == IS_FOUND) {
						if((mask & 1 << (space_table[i][j].data - 1)) != NULL) {return 0;}
						mask |= 1 << (space_table[i][j].data - 1);
					}
//							at least one candidate in a candidate cell					
					else if(space_table[i][j].type == IS_CANDIDATE) {
						if(!space_table[i][j].data) {return 0;}	
						++cell; candidate |= space_table[i][j].data;
					}
				}
//							as many candidates as there are candidate cells
				if(cell != BITCOUNT(candidate)) {return 0;}
			}				
	return 1;
}


int resolve_singles() {
	int success;
	do{
		success = 0;
		success |= naked_single();
		success |= hidden_single();
	} while(success);
	if(is_grid_finished()) {display_message(FINISHED); unit_type = -1; cursor.x = 4; cursor.y = 4;
	manu_space_type = RC_SPACE; fill_space_table(manu_space_type); restore_screen(); return 1;}
	return 0;
}

void resolve_subset() {
	int success;
	sys_slowOff();
	do{
		success = 0;
		success |= naked_subset(ROW);
		success |= naked_subset(COLUMN);
		success |= naked_subset(BLOCK);
		success |= hidden_subset(ROW);
		success |= hidden_subset(COLUMN);
		success |= hidden_subset(BLOCK);
	} while(success);
	sys_slowOn();
}

int is_subset() {
	if(naked_subset(ROW)) return 1;
	if(naked_subset(COLUMN)) return 1;
	if(naked_subset(BLOCK)) return 1;
	if(hidden_subset(ROW)) return 1;
	if(hidden_subset(COLUMN)) return 1;
	if(hidden_subset(BLOCK)) return 1;	
	return 0;
}

int compare_nakedsubset(const void *arg1, const void *arg2) {
	if( ((NAKED_SUBSET *) arg1)->size < ((NAKED_SUBSET *) arg2)->size) return(-1);
	else if( ((NAKED_SUBSET *) arg1)->size > ((NAKED_SUBSET *) arg2)->size) return(1);
	else return(0);
}

void highlight_locked() {
	int a1,a2,a3;
	flag_locked = 0;
	
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
	a1 = locked_candidate[hint_info.candidate].row; a2 = locked_candidate[hint_info.candidate].column;
	
	hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
	hpg_fill_rect(coordinates[a1][a2].x-3,coordinates[a1][a2].y-2,coordinates[a1][a2].x+4,coordinates[a1][a2].y+5);
	hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
	hpg_draw_letter(sdk_digit[hint_info.candidate+1], coordinates[a1][a2].x, coordinates[a1][a2].y);
	
	if(locked_candidate[hint_info.candidate].number == 2) {
	
		a3 = locked_candidate[hint_info.candidate].rank2;
		
		if(locked_candidate[hint_info.candidate].alignement == ROW) {
			hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
			hpg_fill_rect(coordinates[a1][a3].x-3,coordinates[a1][a3].y-2,coordinates[a1][a3].x+4,coordinates[a1][a3].y+5);
			hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
			hpg_draw_letter(sdk_digit[hint_info.candidate+1], coordinates[a1][a3].x, coordinates[a1][a3].y);
		}
		
		else {
			a1 = a3;
			hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
			hpg_fill_rect(coordinates[a1][a2].x-3,coordinates[a1][a2].y-2,coordinates[a1][a2].x+4,coordinates[a1][a2].y+5);
			hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
			hpg_draw_letter(sdk_digit[hint_info.candidate+1], coordinates[a1][a2].x, coordinates[a1][a2].y);
		}
	}
	
	else if(locked_candidate[hint_info.candidate].number == 3){
	
		if(locked_candidate[hint_info.candidate].alignement == ROW) {
			hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
			hpg_fill_rect(coordinates[a1][a2+1].x-3,coordinates[a1][a2+1].y-2,coordinates[a1][a2+1].x+4,coordinates[a1][a2+1].y+5);
			hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
			hpg_draw_letter(sdk_digit[hint_info.candidate+1], coordinates[a1][a2+1].x, coordinates[a1][a2+1].y);
			hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
			hpg_fill_rect(coordinates[a1][a2+2].x-3,coordinates[a1][a2+2].y-2,coordinates[a1][a2+2].x+4,coordinates[a1][a2+2].y+5);
			hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
			hpg_draw_letter(sdk_digit[hint_info.candidate+1], coordinates[a1][a2+2].x, coordinates[a1][a2+2].y);
		}
		
		else {
			hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
			hpg_fill_rect(coordinates[a1+1][a2].x-3,coordinates[a1+1][a2].y-2,coordinates[a1+1][a2].x+4,coordinates[a1+1][a2].y+5);
			hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
			hpg_draw_letter(sdk_digit[hint_info.candidate+1], coordinates[a1+1][a2].x, coordinates[a1+1][a2].y);
			hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
			hpg_fill_rect(coordinates[a1+2][a2].x-3,coordinates[a1+2][a2].y-2,coordinates[a1+2][a2].x+4,coordinates[a1+2][a2].y+5);
			hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
			hpg_draw_letter(sdk_digit[hint_info.candidate+1], coordinates[a1+2][a2].x, coordinates[a1+2][a2].y);
		}
	}
	
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); 
}

void highlight_fish_all() {
	int fish_info_index;
	int row,col,dim,swap;
	flag_fish = 0;
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT);
	
	for(fish_info_index = 0; fish_info_index < fish_all_info.type - 5; fish_info_index++) {
		
		dim = fish_all_info.dim[fish_info_index][1];
		
		do {col = bit_rank[dim & -dim];

			row = fish_all_info.dim[fish_info_index][0];
			if(fish_all_info.space_type == COLUMN) {swap = row; row = col; col = swap;}
			
				hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
				hpg_fill_rect(coordinates[row][col].x-3,coordinates[row][col].y-2,coordinates[row][col].x+4,coordinates[row][col].y+5);
				hpg_set_color(hpg_stdscreen,HPG_COLOR_WHITE);
				hpg_draw_letter(sdk_digit[fish_all_info.candidate+1], coordinates[row][col].x, coordinates[row][col].y);
				
		} while((dim &= dim - 1));
	}
	
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); 
}

void restore_screen() {
	int old_filtered; 
//  	clear grid area then draw grid and fill it , clear display area, restore filter if any, display candidates if on 
	draw_grid(); fill_display_grid(); clear_display_area();
	old_filtered = filtered_candidate; if(filtered_candidate) {filtered_candidate = 0; filter_candidate(old_filtered);}
	if(flag_fish) highlight_fish_all(); else if(flag_locked) highlight_locked(); 
	else if(flag_xychain) highlight_xychain();	else if(flag_cchain) highlight_cchain();
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); if(unit_type != -1) display_candidates();
}

void display_message(int message_index) {
	int l = strlen(message[message_index])<<1;
	int x = 60 - l; int y = 36 - 10;
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); 
	hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE); hpg_fill_rect(x, y, 70+l,40+10);
	hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK); hpg_draw_rect(x, y, 70+l,40+10);
	hpg_draw_text(message[message_index], 65 - l, 36); 
	while (!(keyb_isKeyPressed(4,1)));
}

int validate_grid() {
//							new grid: check the grid for inconsistencies and build the masks of the givens
	int i,j,block, mask;
	int mask_number = 0;
//							initialise mask_unit & excluded_candidates
	memset((char*) mask_unit, 0, sizeof(mask_unit));
	memset((char*) candidates, 0, sizeof(candidates));

//							scan the rows
	for(i = 0; i < 9; i++, mask_number++) for(j = 0; j < 9; j++) 
			if(sdk_cellinfo[i][j].type == IS_GIVEN) {	
				mask = 1 << (sdk_cellinfo[i][j].data - 1);
				if( (mask_unit[MASK_GIVEN][mask_number] & mask) != NULL) {display_message(ERROR_GRID); return 0;}
				mask_unit[MASK_GIVEN][mask_number] |= mask;
			} 
//							scan the columns
	for(j = 0; j < 9; j++, mask_number++) for(i = 0; i < 9; i++) 
			if(sdk_cellinfo[i][j].type == IS_GIVEN) {
				mask = 1 << (sdk_cellinfo[i][j].data - 1);
				if( (mask_unit[MASK_GIVEN][mask_number] & mask) != NULL) {display_message(ERROR_GRID); return 0;}
				mask_unit[MASK_GIVEN][mask_number] |= mask;
			} 
//							scan the blocks
	for(block = 0; block < 9; block++, mask_number++)
		 for(i = block_start[block][0]; i < block_start[block][0] + 3; i++)
			for(j = block_start[block][1]; j < block_start[block][1] + 3; j++) 
				if(sdk_cellinfo[i][j].type == IS_GIVEN) {
					mask = 1 << (sdk_cellinfo[i][j].data - 1);
					if( (mask_unit[MASK_GIVEN][mask_number] & mask) != NULL) {display_message(ERROR_GRID); return 0;}
					mask_unit[MASK_GIVEN][mask_number] |= mask;
			} 
//							compute state of the candidates at the start 	
	for(i = 0; i < 9; i++) for(j = 0; j < 9; j++) if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {
		candidates[START][i][j] = ALLCANDIDATES & ~(mask_unit[MASK_GIVEN][i] | mask_unit[MASK_GIVEN][j+9] | mask_unit[MASK_GIVEN][BLOCK_NUMBER(i,j)+18]);
		}
//							update sdk_cellinfo
	for(i = 0; i < 9; i++) for(j = 0; j < 9; j++) 
		if(sdk_cellinfo[i][j].type == IS_CANDIDATE) {
			sdk_cellinfo[i][j].data = candidates[START][i][j] &
			~(mask_unit[MASK_FOUND][i] | mask_unit[MASK_FOUND][j+9] | mask_unit[MASK_FOUND][BLOCK_NUMBER(i,j)+18]);	
		}
	return 1;
}

void compute_candidates() {
	int i,j;	
	for(i = 0; i < 9; i++) for(j = 0; j < 9; j++) 
		if(	sdk_cellinfo[i][j].type == IS_CANDIDATE ) 
			sdk_cellinfo[i][j].data = candidates[START][i][j] &
			~(mask_unit[MASK_FOUND][i] | mask_unit[MASK_FOUND][j+9] | mask_unit[MASK_FOUND][BLOCK_NUMBER(i,j)+18] | candidates[EXCLUDED][i][j]);			
}

int is_grid_finished() {
	int i;
	for(i = 0; i < 27; i++) {	
		if((mask_unit[MASK_GIVEN][i] | mask_unit[MASK_FOUND][i]) != ALLCANDIDATES) return 0;	
	}
	if(!grid_consistent()) {return 0;}
	return 1;
}

void clear_sdk_cellinfo() {
	int i,j; 
	for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) {sdk_cellinfo[i][j].type = IS_CANDIDATE; sdk_cellinfo[i][j].data = ALLCANDIDATES;}
}

void fill_display_grid() {
	int i,j; 
	for (i = 0; i < 9; i++) for (j = 0; j < 9; j++) {
		if(space_table[i][j].type == IS_GIVEN) {
			hpg_set_color(hpg_stdscreen,HPG_COLOR_GRAY_4);
			hpg_fill_rect(coordinates[i][j].x-3,coordinates[i][j].y-2,coordinates[i][j].x+4,coordinates[i][j].y+5);
			hpg_set_color(hpg_stdscreen,HPG_COLOR_BLACK);
		}
		hpg_draw_letter(sdk_digit[space_table[i][j].type == IS_CANDIDATE ? 0 : space_table[i][j].data], coordinates[i][j].x, coordinates[i][j].y);
	}
}

void draw_grid() {
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
	hpg_fill_rect(0,0,77,79);
	hpg_set_color(hpg_stdscreen,HPG_COLOR_BLACK);
	hpg_draw_rect(X1,Y1,X4,Y4); hpg_draw_rect(X1+1,Y1+1,X4-1,Y4-1);
	hpg_draw_line(X1,Y2,X4,Y2); hpg_draw_line(X1,Y3,X4,Y3); hpg_draw_line(X2,Y1,X2,Y4); hpg_draw_line(X3,Y1,X3,Y4); 
}	

void clear_display_area() {
	hpg_set_mode(hpg_stdscreen, HPG_MODE_PAINT); hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
	hpg_fill_rect(78,0,130,79);
	hpg_set_mode(hpg_stdscreen, HPG_MODE_XOR); hpg_set_color(hpg_stdscreen,HPG_COLOR_BLACK);
}

char * fgets_local(char *buf, int n, FILE *f) {
//			keeps CRs
    int i;
    char *ret = buf;
    
    for (i = 1; i < n; i++) { // read at most n-1 characters
        int c;
        c = fgetc(f);
        if (c == EOF) {
            if (i == 1)
                return NULL;
            else
                break;
        }
        *buf++ = (char) c;
        if ((char) c == '\n') break;
    }
    *buf = '\0';
    return ret;
}




















